@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,8 @@
  * gst-launch-1.0 filesrc location=test.dv ! dvdemux name=demux ! queue ! audioconvert ! alsasink demux. ! queue ! dvdec ! xvimagesink
  * ]| This pipeline decodes and renders the raw DV stream to an audio and a videosink.
  * </refsect2>
+ *
+ * Last reviewed on 2006-02-27 (0.10.3)
  */
 
 /* DV output has two modes, normal and wide. The resolution is the same in both
@@ -77,7 +79,7 @@
  *
  * For its part, GStreamer is concerned with accuracy and preservation of
  * information. This element outputs the 720x576 or 720x480 video that it
- * receives, noting the proper aspect ratio. This should not be a problem for
+ * recieves, noting the proper aspect ratio. This should not be a problem for
  * windowed applications, which can change size to fit the video. Applications
  * with fixed size requirements should decide whether to crop or pad which
  * an element such as videobox can do.
@@ -93,15 +95,15 @@
 #define PAL_FRAMERATE_NUMERATOR 25
 #define PAL_FRAMERATE_DENOMINATOR 1
 
-#define PAL_NORMAL_PAR_X        16
-#define PAL_NORMAL_PAR_Y        15
-#define PAL_WIDE_PAR_X          64
-#define PAL_WIDE_PAR_Y          45
+#define PAL_NORMAL_PAR_X        59
+#define PAL_NORMAL_PAR_Y        54
+#define PAL_WIDE_PAR_X          118
+#define PAL_WIDE_PAR_Y          81
 
-#define NTSC_NORMAL_PAR_X       8
-#define NTSC_NORMAL_PAR_Y       9
-#define NTSC_WIDE_PAR_X         32
-#define NTSC_WIDE_PAR_Y         27
+#define NTSC_NORMAL_PAR_X       10
+#define NTSC_NORMAL_PAR_Y       11
+#define NTSC_WIDE_PAR_X         40
+#define NTSC_WIDE_PAR_Y         33
 
 GST_DEBUG_CATEGORY_STATIC (dvdemux_debug);
 #define GST_CAT_DEFAULT dvdemux_debug
@@ -182,9 +184,12 @@ gst_dvdemux_class_init (GstDVDemuxClass * klass)
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_dvdemux_change_state);
   gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_dvdemux_send_event);
 
-  gst_element_class_add_static_pad_template (gstelement_class, &sink_temp);
-  gst_element_class_add_static_pad_template (gstelement_class, &video_src_temp);
-  gst_element_class_add_static_pad_template (gstelement_class, &audio_src_temp);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_temp));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&video_src_temp));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&audio_src_temp));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "DV system stream demuxer", "Codec/Demuxer",
@@ -255,7 +260,7 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->frame_offset = 0;
   dvdemux->audio_offset = 0;
   dvdemux->video_offset = 0;
-  dvdemux->discont = TRUE;
+  dvdemux->framecount = 0;
   g_atomic_int_set (&dvdemux->found_header, 0);
   dvdemux->frame_len = -1;
   dvdemux->need_segment = FALSE;
@@ -268,62 +273,13 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->wide = FALSE;
   gst_segment_init (&dvdemux->byte_segment, GST_FORMAT_BYTES);
   gst_segment_init (&dvdemux->time_segment, GST_FORMAT_TIME);
-  dvdemux->segment_seqnum = 0;
-  dvdemux->upstream_time_segment = FALSE;
-  dvdemux->have_group_id = FALSE;
-  dvdemux->group_id = G_MAXUINT;
-  dvdemux->tag_event = NULL;
-}
-
-static gboolean
-have_group_id (GstDVDemux * demux)
-{
-  GstEvent *event;
-
-  event = gst_pad_get_sticky_event (demux->sinkpad, GST_EVENT_STREAM_START, 0);
-  if (event) {
-    if (gst_event_parse_group_id (event, &demux->group_id))
-      demux->have_group_id = TRUE;
-    else
-      demux->have_group_id = FALSE;
-    gst_event_unref (event);
-  } else if (!demux->have_group_id) {
-    demux->have_group_id = TRUE;
-    demux->group_id = gst_util_group_id_next ();
-  }
-
-  return demux->have_group_id;
-}
-
-static GstEvent *
-gst_dvdemux_create_global_tag_event (GstDVDemux * dvdemux)
-{
-  gchar rec_datetime[40];
-  GstDateTime *rec_dt;
-  GstTagList *tags;
-
-  tags = gst_tag_list_new (GST_TAG_CONTAINER_FORMAT, "DV", NULL);
-  gst_tag_list_set_scope (tags, GST_TAG_SCOPE_GLOBAL);
-
-  if (dv_get_recording_datetime (dvdemux->decoder, rec_datetime)) {
-    rec_dt = gst_date_time_new_from_iso8601_string (rec_datetime);
-    if (rec_dt) {
-      gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_DATE_TIME,
-          rec_dt, NULL);
-      gst_date_time_unref (rec_dt);
-    }
-  }
-
-  return gst_event_new_tag (tags);
 }
 
 static GstPad *
-gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template,
-    GstCaps * caps)
+gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template)
 {
+  gboolean no_more_pads;
   GstPad *pad;
-  GstEvent *event;
-  gchar *stream_id;
 
   pad = gst_pad_new_from_static_template (template, template->name_template);
 
@@ -333,29 +289,23 @@ gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template,
       GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
   gst_pad_use_fixed_caps (pad);
   gst_pad_set_active (pad, TRUE);
+  gst_element_add_pad (GST_ELEMENT (dvdemux), pad);
 
-  stream_id =
-      gst_pad_create_stream_id (pad,
-      GST_ELEMENT_CAST (dvdemux),
-      template == &video_src_temp ? "video" : "audio");
-  event = gst_event_new_stream_start (stream_id);
-  if (have_group_id (dvdemux))
-    gst_event_set_group_id (event, dvdemux->group_id);
-  gst_pad_push_event (pad, event);
-  g_free (stream_id);
+  no_more_pads =
+      (dvdemux->videosrcpad != NULL && template == &audio_src_temp) ||
+      (dvdemux->audiosrcpad != NULL && template == &video_src_temp);
 
-  gst_pad_set_caps (pad, caps);
+  if (no_more_pads)
+    gst_element_no_more_pads (GST_ELEMENT (dvdemux));
 
   gst_pad_push_event (pad, gst_event_new_segment (&dvdemux->time_segment));
 
-  gst_element_add_pad (GST_ELEMENT (dvdemux), pad);
+  if (no_more_pads) {
+    GstTagList *tags;
 
-  if (!dvdemux->tag_event) {
-    dvdemux->tag_event = gst_dvdemux_create_global_tag_event (dvdemux);
-  }
-
-  if (dvdemux->tag_event) {
-    gst_pad_push_event (pad, gst_event_ref (dvdemux->tag_event));
+    tags = gst_tag_list_new (GST_TAG_CONTAINER_FORMAT, "DV", NULL);
+    gst_tag_list_set_scope (tags, GST_TAG_SCOPE_GLOBAL);
+    gst_pad_push_event (pad, gst_event_new_tag (tags));
   }
 
   return pad;
@@ -636,57 +586,6 @@ gst_dvdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
     }
-    case GST_QUERY_SEEKING:
-    {
-      GstFormat fmt;
-      GstQuery *peerquery;
-      gboolean seekable;
-
-      gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
-
-      /* We can only handle TIME seeks really */
-      if (fmt != GST_FORMAT_TIME) {
-        gst_query_set_seeking (query, fmt, FALSE, -1, -1);
-        break;
-      }
-
-      /* First ask upstream */
-      if (gst_pad_peer_query (dvdemux->sinkpad, query)) {
-        gst_query_parse_seeking (query, NULL, &seekable, NULL, NULL);
-        if (seekable) {
-          res = TRUE;
-          break;
-        }
-      }
-
-      res = TRUE;
-
-      peerquery = gst_query_new_seeking (GST_FORMAT_BYTES);
-      seekable = gst_pad_peer_query (dvdemux->sinkpad, peerquery);
-
-      if (seekable)
-        gst_query_parse_seeking (peerquery, NULL, &seekable, NULL, NULL);
-      gst_query_unref (peerquery);
-
-      if (seekable) {
-        peerquery = gst_query_new_duration (GST_FORMAT_TIME);
-        seekable = gst_dvdemux_src_query (pad, parent, peerquery);
-
-        if (seekable) {
-          gint64 duration;
-
-          gst_query_parse_duration (peerquery, NULL, &duration);
-          gst_query_set_seeking (query, GST_FORMAT_TIME, seekable, 0, duration);
-        } else {
-          gst_query_set_seeking (query, GST_FORMAT_TIME, FALSE, -1, -1);
-        }
-
-        gst_query_unref (peerquery);
-      } else {
-        gst_query_set_seeking (query, GST_FORMAT_TIME, FALSE, -1, -1);
-      }
-      break;
-    }
     default:
       res = gst_pad_query_default (pad, parent, query);
       break;
@@ -752,10 +651,8 @@ gst_dvdemux_push_event (GstDVDemux * dvdemux, GstEvent * event)
 
   if (dvdemux->audiosrcpad)
     res |= gst_pad_push_event (dvdemux->audiosrcpad, event);
-  else {
+  else
     gst_event_unref (event);
-    res = TRUE;
-  }
 
   return res;
 }
@@ -779,7 +676,6 @@ gst_dvdemux_handle_sink_event (GstPad * pad, GstObject * parent,
       GST_DEBUG ("cleared adapter");
       gst_segment_init (&dvdemux->byte_segment, GST_FORMAT_BYTES);
       gst_segment_init (&dvdemux->time_segment, GST_FORMAT_TIME);
-      dvdemux->discont = TRUE;
       res = gst_dvdemux_push_event (dvdemux, event);
       break;
     case GST_EVENT_SEGMENT:
@@ -791,15 +687,11 @@ gst_dvdemux_handle_sink_event (GstPad * pad, GstObject * parent,
         case GST_FORMAT_BYTES:
           gst_segment_copy_into (segment, &dvdemux->byte_segment);
           dvdemux->need_segment = TRUE;
-          dvdemux->segment_seqnum = gst_event_get_seqnum (event);
           gst_event_unref (event);
           break;
         case GST_FORMAT_TIME:
           gst_segment_copy_into (segment, &dvdemux->time_segment);
 
-          dvdemux->upstream_time_segment = TRUE;
-          dvdemux->segment_seqnum = gst_event_get_seqnum (event);
-
           /* and we can just forward this time event */
           res = gst_dvdemux_push_event (dvdemux, event);
           break;
@@ -964,7 +856,6 @@ gst_dvdemux_handle_push_seek (GstDVDemux * dvdemux, GstPad * pad,
     /* now this is the updated seek event on bytes */
     newevent = gst_event_new_seek (rate, GST_FORMAT_BYTES, flags,
         cur_type, start_position, stop_type, end_position);
-    gst_event_set_seqnum (newevent, gst_event_get_seqnum (event));
 
     res = gst_pad_push_event (dvdemux->sinkpad, newevent);
   }
@@ -972,23 +863,6 @@ done:
   return res;
 }
 
-static void
-gst_dvdemux_update_frame_offsets (GstDVDemux * dvdemux, GstClockTime timestamp)
-{
-  /* calculate current frame number */
-  gst_dvdemux_src_convert (dvdemux, dvdemux->videosrcpad,
-      dvdemux->time_segment.format, timestamp,
-      GST_FORMAT_DEFAULT, &dvdemux->video_offset);
-
-  /* calculate current audio number */
-  gst_dvdemux_src_convert (dvdemux, dvdemux->audiosrcpad,
-      dvdemux->time_segment.format, timestamp,
-      GST_FORMAT_DEFAULT, &dvdemux->audio_offset);
-
-  /* every DV frame corresponts with one video frame */
-  dvdemux->frame_offset = dvdemux->video_offset;
-}
-
 /* position ourselves to the configured segment, used in pull mode.
  * The input segment is in TIME format. We convert the time values
  * to bytes values into our byte_segment which we use to pull data from
@@ -1025,9 +899,18 @@ gst_dvdemux_do_seek (GstDVDemux * demux, GstSegment * segment)
       segment->format, segment->time, format,
       (gint64 *) & demux->byte_segment.time);
 
-  gst_dvdemux_update_frame_offsets (demux, segment->start);
+  /* calculate current frame number */
+  format = GST_FORMAT_DEFAULT;
+  gst_dvdemux_src_convert (demux, demux->videosrcpad,
+      segment->format, segment->start, format, &demux->video_offset);
 
-  demux->discont = TRUE;
+  /* calculate current audio number */
+  format = GST_FORMAT_DEFAULT;
+  gst_dvdemux_src_convert (demux, demux->audiosrcpad,
+      segment->format, segment->start, format, &demux->audio_offset);
+
+  /* every DV frame corresponts with one video frame */
+  demux->frame_offset = demux->video_offset;
 
 done:
   return res;
@@ -1050,7 +933,6 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
   gboolean flush;
   gboolean update;
   GstSegment seeksegment;
-  GstEvent *new_event;
 
   GST_DEBUG_OBJECT (demux, "doing seek");
 
@@ -1077,18 +959,13 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     flags = 0;
   }
 
-  demux->segment_seqnum = gst_event_get_seqnum (event);
-
   flush = flags & GST_SEEK_FLAG_FLUSH;
 
   /* send flush start */
-  if (flush) {
-    new_event = gst_event_new_flush_start ();
-    gst_event_set_seqnum (new_event, demux->segment_seqnum);
-    gst_dvdemux_push_event (demux, new_event);
-  } else {
+  if (flush)
+    gst_dvdemux_push_event (demux, gst_event_new_flush_start ());
+  else
     gst_pad_pause_task (demux->sinkpad);
-  }
 
   /* grab streaming lock, this should eventually be possible, either
    * because the task is paused or our streaming thread stopped
@@ -1116,9 +993,7 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
   if (flush) {
     /* send flush stop, peer will accept data and events again. We
      * are not yet providing data as we still have the STREAM_LOCK. */
-    new_event = gst_event_new_flush_stop (TRUE);
-    gst_event_set_seqnum (new_event, demux->segment_seqnum);
-    gst_dvdemux_push_event (demux, new_event);
+    gst_dvdemux_push_event (demux, gst_event_new_flush_stop (TRUE));
   }
 
   /* if successfull seek, we update our real segment and push
@@ -1127,12 +1002,9 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     memcpy (&demux->time_segment, &seeksegment, sizeof (GstSegment));
 
     if (demux->time_segment.flags & GST_SEEK_FLAG_SEGMENT) {
-      GstMessage *message;
-
-      message = gst_message_new_segment_start (GST_OBJECT_CAST (demux),
-          demux->time_segment.format, demux->time_segment.position);
-      gst_message_set_seqnum (message, demux->segment_seqnum);
-      gst_element_post_message (GST_ELEMENT_CAST (demux), message);
+      gst_element_post_message (GST_ELEMENT_CAST (demux),
+          gst_message_new_segment_start (GST_OBJECT_CAST (demux),
+              demux->time_segment.format, demux->time_segment.position));
     }
     if ((stop = demux->time_segment.stop) == -1)
       stop = demux->time_segment.duration;
@@ -1144,7 +1016,6 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
       gst_event_unref (demux->pending_segment);
 
     demux->pending_segment = gst_event_new_segment (&demux->time_segment);
-    gst_event_set_seqnum (demux->pending_segment, demux->segment_seqnum);
 
     demux->need_segment = FALSE;
   }
@@ -1198,9 +1069,10 @@ gst_dvdemux_send_event (GstElement * element, GstEvent * event)
       } else {
         GST_OBJECT_UNLOCK (dvdemux);
 
-        if (dvdemux->seek_handler)
+        if (dvdemux->seek_handler) {
           res = dvdemux->seek_handler (dvdemux, dvdemux->videosrcpad, event);
-        gst_event_unref (event);
+          gst_event_unref (event);
+        }
       }
       break;
     }
@@ -1217,7 +1089,7 @@ static gboolean
 gst_dvdemux_handle_src_event (GstPad * pad, GstObject * parent,
     GstEvent * event)
 {
-  gboolean res = FALSE;
+  gboolean res = TRUE;
   GstDVDemux *dvdemux;
 
   dvdemux = GST_DVDEMUX (parent);
@@ -1227,12 +1099,25 @@ gst_dvdemux_handle_src_event (GstPad * pad, GstObject * parent,
       /* seek handler is installed based on scheduling mode */
       if (dvdemux->seek_handler)
         res = dvdemux->seek_handler (dvdemux, pad, event);
-      gst_event_unref (event);
+      else
+        res = FALSE;
+      break;
+    case GST_EVENT_QOS:
+      /* we can't really (yet) do QoS */
+      res = FALSE;
+      break;
+    case GST_EVENT_NAVIGATION:
+    case GST_EVENT_CAPS:
+      /* no navigation or caps either... */
+      res = FALSE;
       break;
     default:
       res = gst_pad_push_event (dvdemux->sinkpad, event);
+      event = NULL;
       break;
   }
+  if (event)
+    gst_event_unref (event);
 
   return res;
 }
@@ -1256,15 +1141,25 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
     GstBuffer *outbuf;
     gint frequency, channels;
 
+    if (G_UNLIKELY (dvdemux->audiosrcpad == NULL))
+      dvdemux->audiosrcpad = gst_dvdemux_add_pad (dvdemux, &audio_src_temp);
+
     /* get initial format or check if format changed */
     frequency = dv_get_frequency (dvdemux->decoder);
     channels = dv_get_num_channels (dvdemux->decoder);
 
-    if (G_UNLIKELY ((dvdemux->audiosrcpad == NULL)
-            || (frequency != dvdemux->frequency)
+    if (G_UNLIKELY ((frequency != dvdemux->frequency)
             || (channels != dvdemux->channels))) {
       GstCaps *caps;
       GstAudioInfo info;
+      gchar *stream_id;
+
+      stream_id =
+          gst_pad_create_stream_id (dvdemux->audiosrcpad,
+          GST_ELEMENT_CAST (dvdemux), "audio");
+      gst_pad_push_event (dvdemux->audiosrcpad,
+          gst_event_new_stream_start (stream_id));
+      g_free (stream_id);
 
       dvdemux->frequency = frequency;
       dvdemux->channels = channels;
@@ -1273,16 +1168,7 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
       gst_audio_info_set_format (&info, GST_AUDIO_FORMAT_S16LE,
           frequency, channels, NULL);
       caps = gst_audio_info_to_caps (&info);
-      if (G_UNLIKELY (dvdemux->audiosrcpad == NULL)) {
-        dvdemux->audiosrcpad =
-            gst_dvdemux_add_pad (dvdemux, &audio_src_temp, caps);
-
-        if (dvdemux->videosrcpad && dvdemux->audiosrcpad)
-          gst_element_no_more_pads (GST_ELEMENT (dvdemux));
-
-      } else {
-        gst_pad_set_caps (dvdemux->audiosrcpad, caps);
-      }
+      gst_pad_set_caps (dvdemux->audiosrcpad, caps);
       gst_caps_unref (caps);
     }
 
@@ -1308,8 +1194,9 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
     dvdemux->audio_offset += num_samples;
     GST_BUFFER_OFFSET_END (outbuf) = dvdemux->audio_offset;
 
-    if (dvdemux->new_media || dvdemux->discont)
+    if (dvdemux->new_media)
       GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+
     ret = gst_pad_push (dvdemux->audiosrcpad, outbuf);
   } else {
     /* no samples */
@@ -1329,16 +1216,26 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   gboolean wide;
   GstFlowReturn ret = GST_FLOW_OK;
 
+  if (G_UNLIKELY (dvdemux->videosrcpad == NULL))
+    dvdemux->videosrcpad = gst_dvdemux_add_pad (dvdemux, &video_src_temp);
+
   /* get params */
   /* framerate is already up-to-date */
   height = dvdemux->decoder->height;
   wide = dv_format_wide (dvdemux->decoder);
 
   /* see if anything changed */
-  if (G_UNLIKELY ((dvdemux->videosrcpad == NULL) || (dvdemux->height != height)
-          || dvdemux->wide != wide)) {
-    gint par_x, par_y;
+  if (G_UNLIKELY ((dvdemux->height != height) || dvdemux->wide != wide)) {
     GstCaps *caps;
+    gint par_x, par_y;
+    gchar *stream_id;
+
+    stream_id =
+        gst_pad_create_stream_id (dvdemux->videosrcpad,
+        GST_ELEMENT_CAST (dvdemux), "video");
+    gst_pad_push_event (dvdemux->videosrcpad,
+        gst_event_new_stream_start (stream_id));
+    g_free (stream_id);
 
     dvdemux->height = height;
     dvdemux->wide = wide;
@@ -1368,17 +1265,7 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
         "framerate", GST_TYPE_FRACTION, dvdemux->framerate_numerator,
         dvdemux->framerate_denominator,
         "pixel-aspect-ratio", GST_TYPE_FRACTION, par_x, par_y, NULL);
-
-    if (G_UNLIKELY (dvdemux->videosrcpad == NULL)) {
-      dvdemux->videosrcpad =
-          gst_dvdemux_add_pad (dvdemux, &video_src_temp, caps);
-
-      if (dvdemux->videosrcpad && dvdemux->audiosrcpad)
-        gst_element_no_more_pads (GST_ELEMENT (dvdemux));
-
-    } else {
-      gst_pad_set_caps (dvdemux->videosrcpad, caps);
-    }
+    gst_pad_set_caps (dvdemux->videosrcpad, caps);
     gst_caps_unref (caps);
   }
 
@@ -1391,7 +1278,7 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   GST_BUFFER_OFFSET_END (outbuf) = dvdemux->video_offset + 1;
   GST_BUFFER_DURATION (outbuf) = duration;
 
-  if (dvdemux->new_media || dvdemux->discont)
+  if (dvdemux->new_media)
     GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
 
   GST_DEBUG ("pushing video %" GST_TIME_FORMAT,
@@ -1493,11 +1380,8 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
   GstSMPTETimeCode timecode;
   int frame_number;
 
-  if (dvdemux->need_segment) {
+  if (G_UNLIKELY (dvdemux->need_segment)) {
     GstFormat format;
-    GstEvent *event;
-
-    g_assert (!dvdemux->upstream_time_segment);
 
     /* convert to time and store as start/end_timestamp */
     format = GST_FORMAT_TIME;
@@ -1508,14 +1392,15 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
                 (gint64 *) & dvdemux->time_segment.stop)))
       goto segment_error;
 
-    dvdemux->time_segment.time = dvdemux->time_segment.start;
     dvdemux->time_segment.rate = dvdemux->byte_segment.rate;
+    dvdemux->time_segment.position = dvdemux->time_segment.start;
 
-    gst_dvdemux_sink_convert (dvdemux,
-        GST_FORMAT_BYTES, dvdemux->byte_segment.position,
-        GST_FORMAT_TIME, (gint64 *) & dvdemux->time_segment.position);
-
-    gst_dvdemux_update_frame_offsets (dvdemux, dvdemux->time_segment.position);
+    /* calculate current frame number */
+    format = GST_FORMAT_DEFAULT;
+    if (!(gst_dvdemux_src_convert (dvdemux, dvdemux->videosrcpad,
+                GST_FORMAT_TIME, dvdemux->time_segment.start,
+                format, &dvdemux->frame_offset)))
+      goto segment_error;
 
     GST_DEBUG_OBJECT (dvdemux, "sending segment start: %" GST_TIME_FORMAT
         ", stop: %" GST_TIME_FORMAT ", time: %" GST_TIME_FORMAT,
@@ -1523,10 +1408,8 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
         GST_TIME_ARGS (dvdemux->time_segment.stop),
         GST_TIME_ARGS (dvdemux->time_segment.start));
 
-    event = gst_event_new_segment (&dvdemux->time_segment);
-    if (dvdemux->segment_seqnum)
-      gst_event_set_seqnum (event, dvdemux->segment_seqnum);
-    gst_dvdemux_push_event (dvdemux, event);
+    gst_dvdemux_push_event (dvdemux,
+        gst_event_new_segment (&dvdemux->time_segment));
 
     dvdemux->need_segment = FALSE;
   }
@@ -1537,18 +1420,10 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
       GST_SMPTE_TIME_CODE_SYSTEM_25 : GST_SMPTE_TIME_CODE_SYSTEM_30,
       &frame_number, &timecode);
 
-  if (dvdemux->time_segment.rate < 0) {
-    next_ts = gst_util_uint64_scale_int (
-        (dvdemux->frame_offset >
-            0 ? dvdemux->frame_offset - 1 : 0) * GST_SECOND,
-        dvdemux->framerate_denominator, dvdemux->framerate_numerator);
-    duration = dvdemux->time_segment.position - next_ts;
-  } else {
-    next_ts = gst_util_uint64_scale_int (
-        (dvdemux->frame_offset + 1) * GST_SECOND,
-        dvdemux->framerate_denominator, dvdemux->framerate_numerator);
-    duration = next_ts - dvdemux->time_segment.position;
-  }
+  next_ts = gst_util_uint64_scale_int (
+      (dvdemux->frame_offset + 1) * GST_SECOND,
+      dvdemux->framerate_denominator, dvdemux->framerate_numerator);
+  duration = next_ts - dvdemux->time_segment.position;
 
   gst_buffer_map (buffer, &map, GST_MAP_READ);
   dv_parse_packs (dvdemux->decoder, map.data);
@@ -1579,24 +1454,11 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
     goto done;
   }
 
-  dvdemux->discont = FALSE;
   dvdemux->time_segment.position = next_ts;
-
-  if (dvdemux->time_segment.rate < 0) {
-    if (dvdemux->frame_offset > 0)
-      dvdemux->frame_offset--;
-    else
-      GST_WARNING_OBJECT (dvdemux,
-          "Got before frame offset 0 in reverse playback");
-  } else {
-    dvdemux->frame_offset++;
-  }
+  dvdemux->frame_offset++;
 
   /* check for the end of the segment */
-  if ((dvdemux->time_segment.rate > 0 && dvdemux->time_segment.stop != -1
-          && next_ts > dvdemux->time_segment.stop)
-      || (dvdemux->time_segment.rate < 0
-          && dvdemux->time_segment.start > next_ts))
+  if (dvdemux->time_segment.stop != -1 && next_ts > dvdemux->time_segment.stop)
     ret = GST_FLOW_EOS;
   else
     ret = GST_FLOW_OK;
@@ -1687,28 +1549,14 @@ gst_dvdemux_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   /* a discontinuity in the stream, we need to get rid of
    * accumulated data in the adapter and assume a new frame
    * starts after the discontinuity */
-  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT))) {
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)))
     gst_adapter_clear (dvdemux->adapter);
-    dvdemux->discont = TRUE;
-
-    /* Should recheck where we are */
-    if (!dvdemux->upstream_time_segment)
-      dvdemux->need_segment = TRUE;
-  }
 
   /* a timestamp always should be respected */
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
     dvdemux->time_segment.position = timestamp;
-
-    if (dvdemux->discont)
-      gst_dvdemux_update_frame_offsets (dvdemux,
-          dvdemux->time_segment.position);
-  } else if (dvdemux->upstream_time_segment && dvdemux->discont) {
-    /* This will probably fail later to provide correct
-     * timestamps and/or durations but also should not happen */
-    GST_ERROR_OBJECT (dvdemux,
-        "Upstream provides TIME segment but no PTS after discont");
+    /* FIXME, adjust frame_offset and other counters */
   }
 
   gst_adapter_push (dvdemux->adapter, buffer);
@@ -1813,6 +1661,7 @@ gst_dvdemux_loop (GstPad * pad)
     }
   }
 
+
   if (G_UNLIKELY (dvdemux->pending_segment)) {
 
     /* now send the newsegment */
@@ -1851,30 +1700,20 @@ done:
   /* ERRORS */
 parse_header_error:
   {
-    GstEvent *event;
-
     GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
         (NULL), ("Error parsing DV header"));
     gst_buffer_unref (buffer);
     gst_pad_pause_task (dvdemux->sinkpad);
-    event = gst_event_new_eos ();
-    if (dvdemux->segment_seqnum)
-      gst_event_set_seqnum (event, dvdemux->segment_seqnum);
-    gst_dvdemux_push_event (dvdemux, event);
+    gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
     goto done;
   }
 small_buffer:
   {
-    GstEvent *event;
-
     GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
         (NULL), ("Error reading buffer"));
     gst_buffer_unref (buffer);
     gst_pad_pause_task (dvdemux->sinkpad);
-    event = gst_event_new_eos ();
-    if (dvdemux->segment_seqnum)
-      gst_event_set_seqnum (event, dvdemux->segment_seqnum);
-    gst_dvdemux_push_event (dvdemux, event);
+    gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
     goto done;
   }
 pause:
@@ -1892,34 +1731,20 @@ pause:
         dvdemux->time_segment.position = dvdemux->time_segment.start;
       /* perform EOS logic */
       if (dvdemux->time_segment.flags & GST_SEEK_FLAG_SEGMENT) {
-        GstMessage *message;
-        GstEvent *event;
-
-        event = gst_event_new_segment_done (dvdemux->time_segment.format,
-            dvdemux->time_segment.position);
-        if (dvdemux->segment_seqnum)
-          gst_event_set_seqnum (event, dvdemux->segment_seqnum);
-
-        message = gst_message_new_segment_done (GST_OBJECT_CAST (dvdemux),
-            dvdemux->time_segment.format, dvdemux->time_segment.position);
-        if (dvdemux->segment_seqnum)
-          gst_message_set_seqnum (message, dvdemux->segment_seqnum);
-
-        gst_element_post_message (GST_ELEMENT (dvdemux), message);
-        gst_dvdemux_push_event (dvdemux, event);
+        gst_element_post_message (GST_ELEMENT (dvdemux),
+            gst_message_new_segment_done (GST_OBJECT_CAST (dvdemux),
+                dvdemux->time_segment.format, dvdemux->time_segment.position));
+        gst_dvdemux_push_event (dvdemux,
+            gst_event_new_segment_done (dvdemux->time_segment.format,
+                dvdemux->time_segment.position));
       } else {
-        GstEvent *event = gst_event_new_eos ();
-        if (dvdemux->segment_seqnum)
-          gst_event_set_seqnum (event, dvdemux->segment_seqnum);
-        gst_dvdemux_push_event (dvdemux, event);
+        gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
       }
     } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_EOS) {
-      GstEvent *event = gst_event_new_eos ();
       /* for fatal errors or not-linked we post an error message */
-      GST_ELEMENT_FLOW_ERROR (dvdemux, ret);
-      if (dvdemux->segment_seqnum)
-        gst_event_set_seqnum (event, dvdemux->segment_seqnum);
-      gst_dvdemux_push_event (dvdemux, event);
+      GST_ELEMENT_ERROR (dvdemux, STREAM, FAILED,
+          (NULL), ("streaming stopped, reason %s", gst_flow_get_name (ret)));
+      gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
     }
     goto done;
   }
@@ -2022,12 +1847,6 @@ gst_dvdemux_change_state (GstElement * element, GstStateChange transition)
       dvdemux->decoder = NULL;
 
       gst_dvdemux_remove_pads (dvdemux);
-
-      if (dvdemux->tag_event) {
-        gst_event_unref (dvdemux->tag_event);
-        dvdemux->tag_event = NULL;
-      }
-
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
     {
