@@ -1,6 +1,6 @@
 /* GStreamer
  * Copyright (C) 2008 Tristan Matthews <tristan@sat.qc.ca>
- *
+ * 
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -36,56 +36,55 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:element-jackaudiosrc
- * @title: jackaudiosrc
  * @see_also: #GstAudioBaseSrc, #GstAudioRingBuffer
  *
  * A Src that inputs data from Jack ports.
- *
- * It will create N Jack ports named in_&lt;name&gt;_&lt;num&gt; where
+ * 
+ * It will create N Jack ports named in_&lt;name&gt;_&lt;num&gt; where 
  * &lt;name&gt; is the element name and &lt;num&gt; is starting from 1.
  * Each port corresponds to a gstreamer channel.
- *
+ * 
  * The samplerate as exposed on the caps is always the same as the samplerate of
  * the jack server.
- *
+ * 
  * When the #GstJackAudioSrc:connect property is set to auto, this element
- * will try to connect each input port to a random physical jack output pin.
- *
+ * will try to connect each input port to a random physical jack output pin. 
+ * 
  * When the #GstJackAudioSrc:connect property is set to none, the element will
  * accept any number of output channels and will create (but not connect) an
  * input port for each channel.
- *
+ * 
  * The element will generate an error when the Jack server is shut down when it
  * was PAUSED or PLAYING. This element does not support dynamic rate and buffer
  * size changes at runtime.
- *
- * ## Example launch line
+ * 
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 jackaudiosrc connect=0 ! jackaudiosink connect=0
  * ]| Get audio input into gstreamer from jack.
+ * </refsect2>
  *
+ * Last reviewed on 2008-07-22 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <glib/gi18n-lib.h>
+#include <gst/gst-i18n-plugin.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <gst/audio/audio.h>
-
 #include "gstjackaudiosrc.h"
 #include "gstjackringbuffer.h"
 #include "gstjackutil.h"
-#include "gstjackloader.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_jack_audio_src_debug);
 #define GST_CAT_DEFAULT gst_jack_audio_src_debug
@@ -99,7 +98,7 @@ gst_jack_audio_src_allocate_channels (GstJackAudioSrc * src, gint channels)
 
   /* remove ports we don't need */
   while (src->port_count > channels)
-    gst_jack_port_unregister (client, src->ports[--src->port_count]);
+    jack_port_unregister (client, src->ports[--src->port_count]);
 
   /* alloc enough input ports */
   src->ports = g_realloc (src->ports, sizeof (jack_port_t *) * channels);
@@ -114,7 +113,7 @@ gst_jack_audio_src_allocate_channels (GstJackAudioSrc * src, gint channels)
         g_strdup_printf ("in_%s_%d", GST_ELEMENT_NAME (src),
         src->port_count + 1);
     src->ports[src->port_count] =
-        gst_jack_port_register (client, name, JACK_DEFAULT_AUDIO_TYPE,
+        jack_port_register (client, name, JACK_DEFAULT_AUDIO_TYPE,
         JackPortIsInput, 0);
     if (src->ports[src->port_count] == NULL)
       return FALSE;
@@ -137,7 +136,7 @@ gst_jack_audio_src_free_channels (GstJackAudioSrc * src)
   /* get rid of all ports */
   while (src->port_count) {
     GST_LOG_OBJECT (src, "unregister port %d", i);
-    if ((res = gst_jack_port_unregister (client, src->ports[i++])))
+    if ((res = jack_port_unregister (client, src->ports[i++])))
       GST_DEBUG_OBJECT (src, "unregister of port failed (%d)", res);
 
     src->port_count--;
@@ -152,7 +151,7 @@ gst_jack_audio_src_free_channels (GstJackAudioSrc * src)
 static GType
 gst_jack_ring_buffer_get_type (void)
 {
-  static gsize ringbuffer_type = 0;
+  static volatile gsize ringbuffer_type = 0;
 
   if (g_once_init_enter (&ringbuffer_type)) {
     static const GTypeInfo ringbuffer_info = { sizeof (GstJackRingBufferClass),
@@ -221,7 +220,7 @@ jack_process_cb (jack_nframes_t nframes, void *arg)
   /* get input buffers */
   for (i = 0; i < channels; i++)
     src->buffers[i] =
-        (sample_t *) gst_jack_port_get_buffer (src->ports[i], nframes);
+        (sample_t *) jack_port_get_buffer (src->ports[i], nframes);
 
   if (gst_audio_ring_buffer_prepare_read (buf, &writeseg, &writeptr, &len)) {
     flen = len / channels;
@@ -408,6 +407,7 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
 {
   GstJackAudioSrc *src;
   GstJackRingBuffer *abuf;
+  const char **ports;
   gint sample_rate, buffer_size;
   gint i, bpf, rate, channels, res;
   jack_client_t *client;
@@ -422,7 +422,7 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
   rate = GST_AUDIO_INFO_RATE (&spec->info);
 
   /* sample rate must be that of the server */
-  sample_rate = gst_jack_get_sample_rate (client);
+  sample_rate = jack_get_sample_rate (client);
   if (sample_rate != rate)
     goto wrong_samplerate;
 
@@ -434,7 +434,7 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
 
   gst_jack_set_layout (buf, spec);
 
-  buffer_size = gst_jack_get_buffer_size (client);
+  buffer_size = jack_get_buffer_size (client);
 
   /* the segment size in bytes, this is large enough to hold a buffer of 32bit floats
    * for all channels  */
@@ -443,9 +443,7 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
       (GST_SECOND / GST_USECOND), rate * bpf);
   /* segtotal based on buffer-time latency */
   spec->segtotal = spec->buffer_time / spec->latency_time;
-
-  /* Use small period when low-latency is enabled regardless of buffer-time */
-  if (spec->segtotal < 2 || src->low_latency) {
+  if (spec->segtotal < 2) {
     spec->segtotal = 2;
     spec->buffer_time = spec->latency_time * spec->segtotal;
   }
@@ -467,39 +465,14 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
   /* if we need to automatically connect the ports, do so now. We must do this
    * after activating the client. */
   if (src->connect == GST_JACK_CONNECT_AUTO
-      || src->connect == GST_JACK_CONNECT_AUTO_FORCED
-      || src->connect == GST_JACK_CONNECT_EXPLICIT) {
-    const char **available_ports = NULL;
-    const char **jack_ports = NULL;
-    char **user_ports = NULL;
-
+      || src->connect == GST_JACK_CONNECT_AUTO_FORCED) {
     /* find all the physical output ports. A physical output port is a port
      * associated with a hardware device. Someone needs connect to a physical
      * port in order to capture something. */
-
-    if (src->port_names) {
-      user_ports = gst_jack_audio_client_get_port_names_from_string (client,
-          src->port_names, JackPortIsOutput);
-
-      if (user_ports)
-        available_ports = (const char **) user_ports;
-    }
-
-    if (!available_ports && src->connect == GST_JACK_CONNECT_EXPLICIT)
-      goto wrong_port_names;
-
-    if (!available_ports) {
-      if (!src->port_pattern) {
-        jack_ports = gst_jack_get_ports (client, NULL, NULL,
-            JackPortIsPhysical | JackPortIsOutput);
-      } else {
-        jack_ports = gst_jack_get_ports (client, src->port_pattern, NULL,
-            JackPortIsOutput);
-      }
-      available_ports = jack_ports;
-    }
-
-    if (!available_ports) {
+    ports =
+        jack_get_ports (client, NULL, NULL,
+        JackPortIsPhysical | JackPortIsOutput);
+    if (ports == NULL) {
       /* no ports? fine then we don't do anything except for posting a warning
        * message. */
       GST_ELEMENT_WARNING (src, RESOURCE, NOT_FOUND, (NULL),
@@ -509,28 +482,21 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
 
     for (i = 0; i < channels; i++) {
       /* stop when all output ports are exhausted */
-      if (!available_ports[i]) {
+      if (ports[i] == NULL) {
         /* post a warning that we could not connect all ports */
         GST_ELEMENT_WARNING (src, RESOURCE, NOT_FOUND, (NULL),
             ("No more physical ports, leaving some ports unconnected"));
         break;
       }
       GST_DEBUG_OBJECT (src, "try connecting to %s",
-          gst_jack_port_name (src->ports[i]));
+          jack_port_name (src->ports[i]));
 
       /* connect the physical port to a port */
-      res = gst_jack_connect (client,
-          available_ports[i], gst_jack_port_name (src->ports[i]));
-      if (res != 0 && res != EEXIST) {
-        gst_jack_free (jack_ports);
-        g_strfreev (user_ports);
-
+      res = jack_connect (client, ports[i], jack_port_name (src->ports[i]));
+      if (res != 0 && res != EEXIST)
         goto cannot_connect;
-      }
     }
-
-    gst_jack_free (jack_ports);
-    g_strfreev (user_ports);
+    free (ports);
   }
 done:
 
@@ -565,12 +531,7 @@ cannot_connect:
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("Could not connect input ports to physical ports (%d:%s)",
             res, g_strerror (res)));
-    return FALSE;
-  }
-wrong_port_names:
-  {
-    GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
-        ("Invalid port-names was provided"));
+    free (ports);
     return FALSE;
   }
 }
@@ -619,7 +580,7 @@ gst_jack_ring_buffer_start (GstAudioRingBuffer * buf)
     jack_client_t *client;
 
     client = gst_jack_audio_client_get_client (src->client);
-    gst_jack_transport_start (client);
+    jack_transport_start (client);
   }
 
   return TRUE;
@@ -638,7 +599,7 @@ gst_jack_ring_buffer_pause (GstAudioRingBuffer * buf)
     jack_client_t *client;
 
     client = gst_jack_audio_client_get_client (src->client);
-    gst_jack_transport_stop (client);
+    jack_transport_stop (client);
   }
 
   return TRUE;
@@ -657,12 +618,13 @@ gst_jack_ring_buffer_stop (GstAudioRingBuffer * buf)
     jack_client_t *client;
 
     client = gst_jack_audio_client_get_client (src->client);
-    gst_jack_transport_stop (client);
+    jack_transport_stop (client);
   }
 
   return TRUE;
 }
 
+#if defined (HAVE_JACK_0_120_1) || defined(HAVE_JACK_1_9_7)
 static guint
 gst_jack_ring_buffer_delay (GstAudioRingBuffer * buf)
 {
@@ -673,7 +635,7 @@ gst_jack_ring_buffer_delay (GstAudioRingBuffer * buf)
   src = GST_JACK_AUDIO_SRC (GST_OBJECT_PARENT (buf));
 
   for (i = 0; i < src->port_count; i++) {
-    gst_jack_port_get_latency_range (src->ports[i], JackCaptureLatency, &range);
+    jack_port_get_latency_range (src->ports[i], JackCaptureLatency, &range);
     if (range.max > res)
       res = range.max;
   }
@@ -682,6 +644,30 @@ gst_jack_ring_buffer_delay (GstAudioRingBuffer * buf)
 
   return res;
 }
+#else /* !(defined (HAVE_JACK_0_120_1) || defined(HAVE_JACK_1_9_7)) */
+static guint
+gst_jack_ring_buffer_delay (GstAudioRingBuffer * buf)
+{
+  GstJackAudioSrc *src;
+  guint i, res = 0;
+  guint latency;
+  jack_client_t *client;
+
+  src = GST_JACK_AUDIO_SRC (GST_OBJECT_PARENT (buf));
+
+  client = gst_jack_audio_client_get_client (src->client);
+
+  for (i = 0; i < src->port_count; i++) {
+    latency = jack_port_get_total_latency (client, src->ports[i]);
+    if (latency > res)
+      res = latency;
+  }
+
+  GST_DEBUG_OBJECT (src, "delay %u", res);
+
+  return res;
+}
+#endif
 
 /* Audiosrc signals and args */
 enum
@@ -694,8 +680,6 @@ enum
 #define DEFAULT_PROP_SERVER 		NULL
 #define DEFAULT_PROP_CLIENT_NAME	NULL
 #define DEFAULT_PROP_TRANSPORT	GST_JACK_TRANSPORT_AUTONOMOUS
-#define DEFAULT_PROP_PORT_PATTERN     	NULL
-#define DEFAULT_PROP_LOW_LATENCY  FALSE
 
 enum
 {
@@ -704,13 +688,11 @@ enum
   PROP_SERVER,
   PROP_CLIENT,
   PROP_CLIENT_NAME,
-  PROP_PORT_PATTERN,
   PROP_TRANSPORT,
-  PROP_LOW_LATENCY,
-  PROP_PORT_NAMES,
   PROP_LAST
 };
 
+
 /* the capabilities of the inputs and outputs.
  *
  * describe the real formats here.
@@ -727,8 +709,6 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_jack_audio_src_parent_class parent_class
 G_DEFINE_TYPE (GstJackAudioSrc, gst_jack_audio_src, GST_TYPE_AUDIO_BASE_SRC);
-GST_ELEMENT_REGISTER_DEFINE (jackaudiosrc, "jackaudiosrc",
-    GST_RANK_PRIMARY, GST_TYPE_JACK_AUDIO_SRC);
 
 static void gst_jack_audio_src_dispose (GObject * object);
 static void gst_jack_audio_src_set_property (GObject * object, guint prop_id,
@@ -776,9 +756,11 @@ gst_jack_audio_src_class_init (GstJackAudioSrcClass * klass)
           DEFAULT_PROP_SERVER, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstJackAudioSrc:client-name:
+   * GstJackAudioSrc:client-name
    *
    * The client name to use.
+   *
+   * Since: 0.10.31
    */
   g_object_class_install_property (gobject_class, PROP_CLIENT_NAME,
       g_param_spec_string ("client-name", "Client name",
@@ -791,23 +773,13 @@ gst_jack_audio_src_class_init (GstJackAudioSrcClass * klass)
           GST_TYPE_JACK_CLIENT,
           GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
           G_PARAM_STATIC_STRINGS));
-   /**
-    * GstJackAudioSrc:port-pattern
-    *
-    * autoconnect to ports matching pattern, when NULL connect to physical ports
-    *
-    * Since: 1.6
-    */
-  g_object_class_install_property (gobject_class, PROP_PORT_PATTERN,
-      g_param_spec_string ("port-pattern", "port pattern",
-          "A pattern to select which ports to connect to (NULL = first physical ports)",
-          DEFAULT_PROP_PORT_PATTERN,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstJackAudioSink:transport:
+   * GstJackAudioSink:transport
    *
    * The jack transport behaviour for the client.
+   *
+   * Since: 0.10.31
    */
   g_object_class_install_property (gobject_class, PROP_TRANSPORT,
       g_param_spec_flags ("transport", "Transport mode",
@@ -815,37 +787,8 @@ gst_jack_audio_src_class_init (GstJackAudioSrcClass * klass)
           GST_TYPE_JACK_TRANSPORT, DEFAULT_PROP_TRANSPORT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstJackAudioSrc:low-latency:
-   *
-   * Optimize all settings for lowest latency. When enabled,
-   * #GstAudioBaseSrc:buffer-time and #GstAudioBaseSrc:latency-time will be
-   * ignored.
-   *
-   * Since: 1.20
-   */
-  g_object_class_install_property (gobject_class, PROP_LOW_LATENCY,
-      g_param_spec_boolean ("low-latency", "Low latency",
-          "Optimize all settings for lowest latency. When enabled, "
-          "\"buffer-time\" and \"latency-time\" will be ignored",
-          DEFAULT_PROP_LOW_LATENCY,
-          GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
-          G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstJackAudioSrc:port-names:
-   *
-   * Comma-separated list of port name including "client_name:" prefix
-   *
-   * Since: 1.20
-   */
-  g_object_class_install_property (gobject_class, PROP_PORT_NAMES,
-      g_param_spec_string ("port-names", "Port Names",
-          "Comma-separated list of port name including \"client_name:\" prefix",
-          NULL, GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
-          G_PARAM_STATIC_STRINGS));
-
-  gst_element_class_add_static_pad_template (gstelement_class, &src_factory);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Audio Source (Jack)", "Source/Audio",
@@ -875,7 +818,6 @@ gst_jack_audio_src_init (GstJackAudioSrc * src)
   src->buffers = NULL;
   src->client_name = g_strdup (DEFAULT_PROP_CLIENT_NAME);
   src->transport = DEFAULT_PROP_TRANSPORT;
-  src->low_latency = DEFAULT_PROP_LOW_LATENCY;
 }
 
 static void
@@ -890,13 +832,6 @@ gst_jack_audio_src_dispose (GObject * object)
     src->client_name = NULL;
   }
 
-  if (src->port_pattern != NULL) {
-    g_free (src->port_pattern);
-    src->port_pattern = NULL;
-  }
-
-  g_clear_pointer (&src->port_names, g_free);
-
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -911,10 +846,6 @@ gst_jack_audio_src_set_property (GObject * object, guint prop_id,
       g_free (src->client_name);
       src->client_name = g_value_dup_string (value);
       break;
-    case PROP_PORT_PATTERN:
-      g_free (src->port_pattern);
-      src->port_pattern = g_value_dup_string (value);
-      break;
     case PROP_CONNECT:
       src->connect = g_value_get_enum (value);
       break;
@@ -931,13 +862,6 @@ gst_jack_audio_src_set_property (GObject * object, guint prop_id,
     case PROP_TRANSPORT:
       src->transport = g_value_get_flags (value);
       break;
-    case PROP_LOW_LATENCY:
-      src->low_latency = g_value_get_boolean (value);
-      break;
-    case PROP_PORT_NAMES:
-      g_free (src->port_names);
-      src->port_names = g_value_dup_string (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -954,9 +878,6 @@ gst_jack_audio_src_get_property (GObject * object, guint prop_id,
     case PROP_CLIENT_NAME:
       g_value_set_string (value, src->client_name);
       break;
-    case PROP_PORT_PATTERN:
-      g_value_set_string (value, src->port_pattern);
-      break;
     case PROP_CONNECT:
       g_value_set_enum (value, src->connect);
       break;
@@ -969,12 +890,6 @@ gst_jack_audio_src_get_property (GObject * object, guint prop_id,
     case PROP_TRANSPORT:
       g_value_set_flags (value, src->transport);
       break;
-    case PROP_LOW_LATENCY:
-      g_value_set_boolean (value, src->low_latency);
-      break;
-    case PROP_PORT_NAMES:
-      g_value_set_string (value, src->port_names);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -993,46 +908,18 @@ gst_jack_audio_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
   if (src->client == NULL)
     goto no_client;
 
-  if (src->connect == GST_JACK_CONNECT_EXPLICIT && !src->port_names)
-    goto no_port_names;
-
   client = gst_jack_audio_client_get_client (src->client);
 
-  if (src->connect == GST_JACK_CONNECT_AUTO ||
-      src->connect == GST_JACK_CONNECT_EXPLICIT) {
-    max = 0;
-
-    if (src->port_names) {
-      gchar **user_ports =
-          gst_jack_audio_client_get_port_names_from_string (client,
-          src->port_names, JackPortIsOutput);
-
-      if (user_ports) {
-        max = g_strv_length (user_ports);
-      } else {
-        GST_ELEMENT_WARNING (src, RESOURCE, NOT_FOUND,
-            ("Invalid \"port-names\" was requested"),
-            ("Requested \"port-names\" %s contains invalid name",
-                src->port_names));
-      }
-
-      g_strfreev (user_ports);
-    }
-
-    if (max > 0)
-      goto found;
-
-    if (src->connect == GST_JACK_CONNECT_EXPLICIT)
-      goto no_port_names;
-
+  if (src->connect == GST_JACK_CONNECT_AUTO) {
     /* get a port count, this is the number of channels we can automatically
      * connect. */
-    ports = gst_jack_get_ports (client, NULL, NULL,
+    ports = jack_get_ports (client, NULL, NULL,
         JackPortIsPhysical | JackPortIsOutput);
+    max = 0;
     if (ports != NULL) {
       for (; ports[max]; max++);
 
-      gst_jack_free (ports);
+      free (ports);
     } else
       max = 0;
   } else {
@@ -1040,28 +927,18 @@ gst_jack_audio_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
      * pads. */
     max = G_MAXINT;
   }
+  min = MIN (1, max);
 
-found:
-  if (src->connect == GST_JACK_CONNECT_EXPLICIT) {
-    min = max;
-  } else {
-    min = MIN (1, max);
-  }
-
-  rate = gst_jack_get_sample_rate (client);
+  rate = jack_get_sample_rate (client);
 
   GST_DEBUG_OBJECT (src, "got %d-%d ports, samplerate: %d", min, max, rate);
 
   if (!src->caps) {
     src->caps = gst_caps_new_simple ("audio/x-raw",
         "format", G_TYPE_STRING, GST_JACK_FORMAT_STR,
-        "layout", G_TYPE_STRING, "interleaved", "rate", G_TYPE_INT, rate, NULL);
-    if (min == max) {
-      gst_caps_set_simple (src->caps, "channels", G_TYPE_INT, min, NULL);
-    } else {
-      gst_caps_set_simple (src->caps,
-          "channels", GST_TYPE_INT_RANGE, min, max, NULL);
-    }
+        "layout", G_TYPE_STRING, "interleaved",
+        "rate", G_TYPE_INT, rate,
+        "channels", GST_TYPE_INT_RANGE, min, max, NULL);
   }
   GST_INFO_OBJECT (src, "returning caps %" GST_PTR_FORMAT, src->caps);
 
@@ -1074,13 +951,6 @@ no_client:
     /* base class will get template caps for us when we return NULL */
     return NULL;
   }
-no_port_names:
-  {
-    GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS,
-        ("User must provide valid port names"),
-        ("\"port-names\" contains invalid name or NULL string"));
-    return NULL;
-  }
 }
 
 static GstAudioRingBuffer *
