@@ -85,6 +85,7 @@ gst_pngdec_class_init (GstPngDecClass * klass)
       "Wim Taymans <wim@fluendo.com>");
 
   vdec_class->start = gst_pngdec_start;
+  vdec_class->stop = gst_pngdec_stop;
   vdec_class->reset = gst_pngdec_reset;
   vdec_class->set_format = gst_pngdec_set_format;
   vdec_class->handle_frame = gst_pngdec_handle_frame;
@@ -417,17 +418,6 @@ gst_pngdec_decide_allocation (GstVideoDecoder * bdec, GstQuery * query)
   return TRUE;
 }
 
-
-/* Clean up the libpng structures */
-static gboolean
-gst_pngdec_reset (GstVideoDecoder * decoder, gboolean hard)
-{
-  gst_pngdec_stop (decoder);
-  gst_pngdec_start (decoder);
-
-  return TRUE;
-}
-
 static gboolean
 gst_pngdec_libpng_init (GstPngDec * pngdec)
 {
@@ -476,20 +466,10 @@ endinfo_failed:
   }
 }
 
-static gboolean
-gst_pngdec_start (GstVideoDecoder * decoder)
+
+static void
+gst_pngdec_libpng_clear (GstPngDec * pngdec)
 {
-  GstPngDec *pngdec = (GstPngDec *) decoder;
-
-  gst_pngdec_libpng_init (pngdec);
-
-  return TRUE;
-}
-
-static gboolean
-gst_pngdec_stop (GstVideoDecoder * decoder)
-{
-  GstPngDec *pngdec = (GstPngDec *) decoder;
   png_infopp info = NULL, endinfo = NULL;
 
   GST_LOG ("cleaning up libpng structures");
@@ -510,6 +490,24 @@ gst_pngdec_stop (GstVideoDecoder * decoder)
   }
 
   pngdec->color_type = -1;
+}
+
+static gboolean
+gst_pngdec_start (GstVideoDecoder * decoder)
+{
+  GstPngDec *pngdec = (GstPngDec *) decoder;
+
+  gst_pngdec_libpng_init (pngdec);
+
+  return TRUE;
+}
+
+static gboolean
+gst_pngdec_stop (GstVideoDecoder * decoder)
+{
+  GstPngDec *pngdec = (GstPngDec *) decoder;
+
+  gst_pngdec_libpng_clear (pngdec);
 
   if (pngdec->input_state) {
     gst_video_codec_state_unref (pngdec->input_state);
@@ -522,3 +520,13 @@ gst_pngdec_stop (GstVideoDecoder * decoder)
 
   return TRUE;
 }
+
+/* Clean up the libpng structures */
+static gboolean
+gst_pngdec_reset (GstVideoDecoder * decoder, gboolean hard)
+{
+  gst_pngdec_libpng_clear ((GstPngDec *) decoder);
+  gst_pngdec_libpng_init ((GstPngDec *) decoder);
+
+  return TRUE;
+}
