@@ -14,13 +14,12 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  */
 /**
  * SECTION:element-pngenc
- * @title: pngenc
  *
  * Encodes png images.
  */
@@ -30,12 +29,11 @@
 #endif
 #include <string.h>
 #include <gst/gst.h>
+#include "gstpngenc.h"
 #include <gst/video/video.h>
 #include <gst/video/gstvideometa.h>
 #include <zlib.h>
 
-#include "gstpngenc.h"
-
 GST_DEBUG_CATEGORY_STATIC (pngenc_debug);
 #define GST_CAT_DEFAULT pngenc_debug
 
@@ -61,8 +59,8 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("image/png, "
-        "width = (int) [ 1, 1000000 ], "
-        "height = (int) [ 1, 1000000 ], " "framerate = " GST_VIDEO_FPS_RANGE)
+        "width = (int) [ 16, 1000000 ], "
+        "height = (int) [ 16, 1000000 ], " "framerate = " GST_VIDEO_FPS_RANGE)
     );
 
 static GstStaticPadTemplate pngenc_sink_template =
@@ -74,8 +72,6 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 
 #define parent_class gst_pngenc_parent_class
 G_DEFINE_TYPE (GstPngEnc, gst_pngenc, GST_TYPE_VIDEO_ENCODER);
-GST_ELEMENT_REGISTER_DEFINE (pngenc, "pngenc", GST_RANK_PRIMARY,
-    GST_TYPE_PNGENC);
 
 static void gst_pngenc_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -86,8 +82,6 @@ static GstFlowReturn gst_pngenc_handle_frame (GstVideoEncoder * encoder,
     GstVideoCodecFrame * frame);
 static gboolean gst_pngenc_set_format (GstVideoEncoder * encoder,
     GstVideoCodecState * state);
-static gboolean gst_pngenc_flush (GstVideoEncoder * encoder);
-static gboolean gst_pngenc_start (GstVideoEncoder * encoder);
 static gboolean gst_pngenc_propose_allocation (GstVideoEncoder * encoder,
     GstQuery * query);
 
@@ -133,10 +127,10 @@ gst_pngenc_class_init (GstPngEncClass * klass)
           DEFAULT_COMPRESSION_LEVEL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template
-      (element_class, &pngenc_sink_template);
-  gst_element_class_add_static_pad_template
-      (element_class, &pngenc_src_template);
+  gst_element_class_add_pad_template
+      (element_class, gst_static_pad_template_get (&pngenc_sink_template));
+  gst_element_class_add_pad_template
+      (element_class, gst_static_pad_template_get (&pngenc_src_template));
   gst_element_class_set_static_metadata (element_class, "PNG image encoder",
       "Codec/Encoder/Image",
       "Encode a video frame to a .png image",
@@ -145,8 +139,6 @@ gst_pngenc_class_init (GstPngEncClass * klass)
   venc_class->set_format = gst_pngenc_set_format;
   venc_class->handle_frame = gst_pngenc_handle_frame;
   venc_class->propose_allocation = gst_pngenc_propose_allocation;
-  venc_class->flush = gst_pngenc_flush;
-  venc_class->start = gst_pngenc_start;
   gobject_class->finalize = gst_pngenc_finalize;
 
   GST_DEBUG_CATEGORY_INIT (pngenc_debug, "pngenc", 0, "PNG image encoder");
@@ -206,8 +198,6 @@ done:
 static void
 gst_pngenc_init (GstPngEnc * pngenc)
 {
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_VIDEO_ENCODER_SINK_PAD (pngenc));
-
   /* init settings */
   pngenc->png_struct_ptr = NULL;
   pngenc->png_info_ptr = NULL;
@@ -232,106 +222,38 @@ user_flush_data (png_structp png_ptr G_GNUC_UNUSED)
 {
 }
 
-/* Copied from glib/gutilsprivate.h
- *
- * Returns the smallest power of 2 greater than or equal to n,
- * or 0 if such power does not fit in a gsize
- */
-static inline gsize
-gst_pngenc_g_nearest_pow (gsize num)
-{
-  gsize n = num - 1;
-
-  g_assert (num > 0 && num <= G_MAXSIZE / 2);
-
-  n |= n >> 1;
-  n |= n >> 2;
-  n |= n >> 4;
-  n |= n >> 8;
-  n |= n >> 16;
-#if GLIB_SIZEOF_SIZE_T == 8
-  n |= n >> 32;
-#endif
-
-  return n + 1;
-}
-
-static void
-ensure_memory_is_enough (GstPngEnc * pngenc, unsigned int extra_length)
-{
-  GstMemory *new_memory;
-  GstMapInfo map;
-  gsize old_size, desired_size;
-  guint8 *new_data;
-
-  old_size = pngenc->output_map.size;
-  desired_size = gst_pngenc_g_nearest_pow (old_size + extra_length);
-  g_assert (desired_size != 0);
-
-  /* Our output memory wasn't big enough.
-   * Make a new memory that's twice the size, */
-  new_memory = gst_allocator_alloc (NULL, desired_size, NULL);
-  gst_memory_map (new_memory, &map, GST_MAP_READWRITE);
-  new_data = map.data;
-
-  /* copy previous data  */
-  memcpy (new_data, pngenc->output_map.data, old_size);
-  gst_memory_unmap (pngenc->output_mem, &pngenc->output_map);
-  gst_memory_unref (pngenc->output_mem);
-
-  /* drop it into place, */
-  pngenc->output_mem = new_memory;
-  pngenc->output_map = map;
-}
-
 static void
 user_write_data (png_structp png_ptr, png_bytep data, png_uint_32 length)
 {
   GstPngEnc *pngenc;
+  GstMemory *mem;
+  GstMapInfo minfo;
 
   pngenc = (GstPngEnc *) png_get_io_ptr (png_ptr);
 
-  GST_TRACE_OBJECT (pngenc,
-      "Memory size: %" G_GSIZE_FORMAT "\nLength to be written: %u",
-      pngenc->output_map.size, length);
-
-  if (pngenc->output_map.size > G_MAXSIZE - length) {
-    GST_ERROR_OBJECT (pngenc,
-        "Memory buffer would overflow after the png write, aborting.");
-    png_error (png_ptr, "Buffer would overflow, aborting the write.");
+  mem = gst_allocator_alloc (NULL, length, NULL);
+  if (!mem) {
+    GST_ERROR_OBJECT (pngenc, "Failed to allocate memory");
+    png_error (png_ptr, "Failed to allocate memory");
 
     /* never reached */
-    /* libpng will longjmp and we will catch it later on */
     return;
   }
 
-  if ((pngenc->output_mem_pos + length) > pngenc->output_map.size) {
-    GST_INFO_OBJECT (pngenc, "Memory not enough, Allocating more.");
-    ensure_memory_is_enough (pngenc, length);
+  if (!gst_memory_map (mem, &minfo, GST_MAP_WRITE)) {
+    GST_ERROR_OBJECT (pngenc, "Failed to map memory");
+    gst_memory_unref (mem);
+
+    png_error (png_ptr, "Failed to map memory");
+
+    /* never reached */
+    return;
   }
 
-  memcpy (&pngenc->output_map.data[pngenc->output_mem_pos], data, length);
-  pngenc->output_mem_pos += length;
-}
+  memcpy (minfo.data, data, length);
+  gst_memory_unmap (mem, &minfo);
 
-static gboolean
-gst_pngenc_flush (GstVideoEncoder * encoder)
-{
-  GstPngEnc *pngenc = GST_PNGENC (encoder);
-
-  pngenc->frame_count = 0;
-
-  return TRUE;
-}
-
-static gboolean
-gst_pngenc_start (GstVideoEncoder * encoder)
-{
-  GstPngEnc *pngenc = GST_PNGENC (encoder);
-
-  pngenc->frame_count = 0;
-
-  return TRUE;
+  gst_buffer_append_memory (pngenc->buffer_out, mem);
 }
 
 static GstFlowReturn
@@ -341,16 +263,10 @@ gst_pngenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
   gint row_index;
   png_byte **row_pointers;
   GstFlowReturn ret = GST_FLOW_OK;
-  const GstVideoInfo *info;
+  GstVideoInfo *info;
   GstVideoFrame vframe;
-  gsize memory_size;
-  GstBuffer *outbuf;
 
   pngenc = GST_PNGENC (encoder);
-
-  if (pngenc->snapshot && pngenc->frame_count > 0)
-    return GST_FLOW_EOS;
-
   info = &pngenc->input_state->info;
 
   GST_DEBUG_OBJECT (pngenc, "BEGINNING");
@@ -385,6 +301,7 @@ gst_pngenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
   png_set_write_fn (pngenc->png_struct_ptr, pngenc,
       (png_rw_ptr) user_write_data, user_flush_data);
 
+  row_pointers = g_new (png_byte *, GST_VIDEO_INFO_HEIGHT (info));
   if (!gst_video_frame_map (&vframe, &pngenc->input_state->info,
           frame->input_buffer, GST_MAP_READ)) {
     GST_ELEMENT_ERROR (pngenc, STREAM, FORMAT, (NULL),
@@ -393,53 +310,31 @@ gst_pngenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
     goto done;
   }
 
-  row_pointers = g_new (png_byte *, GST_VIDEO_INFO_HEIGHT (info));
-
   for (row_index = 0; row_index < GST_VIDEO_INFO_HEIGHT (info); row_index++) {
     row_pointers[row_index] = GST_VIDEO_FRAME_COMP_DATA (&vframe, 0) +
         (row_index * GST_VIDEO_FRAME_COMP_STRIDE (&vframe, 0));
   }
 
   /* allocate the output buffer */
-  pngenc->output_mem_pos = 0;
-  pngenc->output_mem =
-      gst_allocator_alloc (NULL, MAX (4096, GST_VIDEO_INFO_SIZE (info)), NULL);
-  if (!pngenc->output_mem) {
-    GST_ERROR_OBJECT (pngenc, "Failed to allocate memory");
-    return GST_FLOW_ERROR;
-  }
-
-  gst_memory_map (pngenc->output_mem, &pngenc->output_map, GST_MAP_READWRITE);
+  pngenc->buffer_out = gst_buffer_new ();
 
   png_write_info (pngenc->png_struct_ptr, pngenc->png_info_ptr);
   png_write_image (pngenc->png_struct_ptr, row_pointers);
   png_write_end (pngenc->png_struct_ptr, NULL);
 
   g_free (row_pointers);
-  gst_video_frame_unmap (&vframe);
 
   png_destroy_info_struct (pngenc->png_struct_ptr, &pngenc->png_info_ptr);
   png_destroy_write_struct (&pngenc->png_struct_ptr, (png_infopp) NULL);
 
   /* Set final size and store */
-  gst_memory_unmap (pngenc->output_mem, &pngenc->output_map);
-  /* Trim the buffer size */
-  memory_size = pngenc->output_mem_pos;
-  gst_memory_resize (pngenc->output_mem, 0, memory_size);
-  pngenc->output_mem_pos = 0;
+  frame->output_buffer = pngenc->buffer_out;
 
-  outbuf = gst_buffer_new ();
-  gst_buffer_append_memory (outbuf, pngenc->output_mem);
-  pngenc->output_mem = NULL;
-  frame->output_buffer = outbuf;
-
-  GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
+  pngenc->buffer_out = NULL;
 
   if ((ret = gst_video_encoder_finish_frame (encoder, frame)) != GST_FLOW_OK)
     goto done;
 
-  ++pngenc->frame_count;
-
   if (pngenc->snapshot)
     ret = GST_FLOW_EOS;
 
