@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
@@ -31,8 +31,8 @@
  * Tags sent by upstream elements will be picked up automatically (and merged
  * according to the merge mode set via the tag setter interface).
  *
- * ## Example pipelines
- *
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
  * gst-launch-1.0 -v filesrc location=foo.ogg ! decodebin ! audioconvert ! lame ! id3v2mux ! filesink location=foo.mp3
  * ]| A pipeline that transcodes a file from Ogg/Vorbis to mp3 format with an
@@ -41,6 +41,7 @@
  * |[
  * gst-launch-1.0 -m filesrc location=foo.mp3 ! id3demux ! fakesink silent=TRUE 2&gt; /dev/null | grep taglist
  * ]| Verify that tags have been written.
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -93,8 +94,10 @@ gst_id3v2_mux_class_init (GstId3v2MuxClass * klass)
   GST_TAG_MUX_CLASS (klass)->render_end_tag =
       GST_DEBUG_FUNCPTR (gst_id3v2_mux_render_end_tag);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 
   gst_element_class_set_static_metadata (element_class,
       "TagLib-based ID3v2 Muxer", "Formatter/Metadata",
@@ -424,14 +427,8 @@ add_image_tag (ID3v2::Tag * id3v2tag, const GstTagList * list,
       mime_type = gst_structure_get_name (s);
       if (mime_type != NULL) {
         ID3v2::AttachedPictureFrame * frame;
-        const gchar *desc = NULL;
+        const gchar *desc;
         GstMapInfo map;
-        const GstStructure *info_struct;
-
-        info_struct = gst_sample_get_info (sample);
-        if (!info_struct
-            || !gst_structure_has_name (info_struct, "GstTagImageInfo"))
-          info_struct = NULL;
 
         if (strcmp (mime_type, "text/uri-list") == 0)
           mime_type = "-->";
@@ -450,28 +447,14 @@ add_image_tag (ID3v2::Tag * id3v2tag, const GstTagList * list,
 
         gst_buffer_unmap (image, &map);
 
-        if (info_struct)
-          desc = gst_structure_get_string (info_struct, "image-description");
-
+        desc = gst_structure_get_string (s, "image-description");
         frame->setDescription ((desc) ? desc : "");
 
+        /* FIXME set image type properly from caps */
         if (strcmp (tag, GST_TAG_PREVIEW_IMAGE) == 0) {
           frame->setType (ID3v2::AttachedPictureFrame::FileIcon);
         } else {
-          int image_type = ID3v2::AttachedPictureFrame::Other;
-
-          if (info_struct) {
-            if (gst_structure_get (info_struct, "image-type",
-                    GST_TYPE_TAG_IMAGE_TYPE, &image_type, (void *) NULL)) {
-              if (image_type > 0 && image_type <= 18) {
-                image_type += 2;
-              } else {
-                image_type = ID3v2::AttachedPictureFrame::Other;
-              }
-            }
-          }
-
-          frame->setType ((TagLib::ID3v2::AttachedPictureFrame::Type) image_type);
+          frame->setType (ID3v2::AttachedPictureFrame::Other);
         }
       }
     } else {
@@ -741,7 +724,7 @@ foreach_add_tag (const GstTagList * list, const gchar * tag, gpointer userdata)
   GST_LOG ("Processing tag %s (num=%u)", tag, num_tags);
 
   if (num_tags > 1 && gst_tag_is_fixed (tag)) {
-    GST_WARNING ("Multiple occurrences of fixed tag '%s', ignoring some", tag);
+    GST_WARNING ("Multiple occurences of fixed tag '%s', ignoring some", tag);
     num_tags = 1;
   }
 
