@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -39,7 +39,7 @@ gst_wavpack_stream_reader_read_bytes (void *id, void *data, int32_t bcount)
       rid->length, rid->position);
 
   if (to_read > 0) {
-    memmove (data, rid->buffer + rid->position, to_read);
+    g_memmove (data, rid->buffer + rid->position, to_read);
     rid->position += to_read;
     return to_read;
   } else {
