@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
  
 #ifndef __GST_AUDIO_PANORAMA_H__
@@ -37,24 +37,18 @@ G_BEGIN_DECLS
 typedef struct _GstAudioPanorama      GstAudioPanorama;
 typedef struct _GstAudioPanoramaClass GstAudioPanoramaClass;
 
-typedef void (*GstAudioPanoramaProcessFunc)(gfloat, guint8*, guint8*, guint);
-
-typedef enum
-{
-  METHOD_PSYCHOACOUSTIC = 0,
-  METHOD_SIMPLE
-} GstAudioPanoramaMethod;
+typedef void (*GstAudioPanoramaProcessFunc)(GstAudioPanorama*, guint8*, guint8*, guint);
 
 struct _GstAudioPanorama {
   GstBaseTransform element;
 
-  /* properties */
   gfloat panorama;
-  GstAudioPanoramaMethod method;
 
   /* < private > */
   GstAudioPanoramaProcessFunc process;
+
   GstAudioInfo info;
+  gint method;
 };
 
 struct _GstAudioPanoramaClass {
@@ -63,8 +57,6 @@ struct _GstAudioPanoramaClass {
 
 GType gst_audio_panorama_get_type (void);
 
-GST_ELEMENT_REGISTER_DECLARE (audiopanorama);
-
 G_END_DECLS
 
 #endif /* __GST_AUDIO_PANORAMA_H__ */
