@@ -16,26 +16,26 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 /**
  * SECTION:element-ac3parse
- * @title: ac3parse
  * @short_description: AC3 parser
  * @see_also: #GstAmrParse, #GstAACParse
  *
  * This is an AC3 parser.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 filesrc location=abc.ac3 ! ac3parse ! a52dec ! audioresample ! audioconvert ! autoaudiosink
  * ]|
- *
+ * </refsect2>
  */
 
 /* TODO:
- *  - audio/ac3 to audio/x-private1-ac3 is not implemented (done in the muxer)
+ *  - add support for audio/x-private1-ac3 as well
  *  - should accept framed and unframed input (needs decodebin fixes first)
  */
 
@@ -45,10 +45,9 @@
 
 #include <string.h>
 
-#include "gstaudioparserselements.h"
 #include "gstac3parse.h"
-#include <gst/base/base.h>
-#include <gst/pbutils/pbutils.h>
+#include <gst/base/gstbytereader.h>
+#include <gst/base/gstbitreader.h>
 
 GST_DEBUG_CATEGORY_STATIC (ac3_parse_debug);
 #define GST_CAT_DEFAULT ac3_parse_debug
@@ -59,82 +58,82 @@ static const struct
   const guint frame_size[3];    /* frame size for 32kHz, 44kHz, and 48kHz */
 } frmsizcod_table[38] = {
   {
-        32, {
-          64, 69, 96}}, {
-        32, {
-          64, 70, 96}}, {
-        40, {
-          80, 87, 120}}, {
-        40, {
-          80, 88, 120}}, {
-        48, {
-          96, 104, 144}}, {
-        48, {
-          96, 105, 144}}, {
-        56, {
-          112, 121, 168}}, {
-        56, {
-          112, 122, 168}}, {
-        64, {
-          128, 139, 192}}, {
-        64, {
-          128, 140, 192}}, {
-        80, {
-          160, 174, 240}}, {
-        80, {
-          160, 175, 240}}, {
-        96, {
-          192, 208, 288}}, {
-        96, {
-          192, 209, 288}}, {
-        112, {
-          224, 243, 336}}, {
-        112, {
-          224, 244, 336}}, {
-        128, {
-          256, 278, 384}}, {
-        128, {
-          256, 279, 384}}, {
-        160, {
-          320, 348, 480}}, {
-        160, {
-          320, 349, 480}}, {
-        192, {
-          384, 417, 576}}, {
-        192, {
-          384, 418, 576}}, {
-        224, {
-          448, 487, 672}}, {
-        224, {
-          448, 488, 672}}, {
-        256, {
-          512, 557, 768}}, {
-        256, {
-          512, 558, 768}}, {
-        320, {
-          640, 696, 960}}, {
-        320, {
-          640, 697, 960}}, {
-        384, {
-          768, 835, 1152}}, {
-        384, {
-          768, 836, 1152}}, {
-        448, {
-          896, 975, 1344}}, {
-        448, {
-          896, 976, 1344}}, {
-        512, {
-          1024, 1114, 1536}}, {
-        512, {
-          1024, 1115, 1536}}, {
-        576, {
-          1152, 1253, 1728}}, {
-        576, {
-          1152, 1254, 1728}}, {
-        640, {
-          1280, 1393, 1920}}, {
-        640, {
-          1280, 1394, 1920}}
+    32, {
+  64, 69, 96}}, {
+    32, {
+  64, 70, 96}}, {
+    40, {
+  80, 87, 120}}, {
+    40, {
+  80, 88, 120}}, {
+    48, {
+  96, 104, 144}}, {
+    48, {
+  96, 105, 144}}, {
+    56, {
+  112, 121, 168}}, {
+    56, {
+  112, 122, 168}}, {
+    64, {
+  128, 139, 192}}, {
+    64, {
+  128, 140, 192}}, {
+    80, {
+  160, 174, 240}}, {
+    80, {
+  160, 175, 240}}, {
+    96, {
+  192, 208, 288}}, {
+    96, {
+  192, 209, 288}}, {
+    112, {
+  224, 243, 336}}, {
+    112, {
+  224, 244, 336}}, {
+    128, {
+  256, 278, 384}}, {
+    128, {
+  256, 279, 384}}, {
+    160, {
+  320, 348, 480}}, {
+    160, {
+  320, 349, 480}}, {
+    192, {
+  384, 417, 576}}, {
+    192, {
+  384, 418, 576}}, {
+    224, {
+  448, 487, 672}}, {
+    224, {
+  448, 488, 672}}, {
+    256, {
+  512, 557, 768}}, {
+    256, {
+  512, 558, 768}}, {
+    320, {
+  640, 696, 960}}, {
+    320, {
+  640, 697, 960}}, {
+    384, {
+  768, 835, 1152}}, {
+    384, {
+  768, 836, 1152}}, {
+    448, {
+  896, 975, 1344}}, {
+    448, {
+  896, 976, 1344}}, {
+    512, {
+  1024, 1114, 1536}}, {
+    512, {
+  1024, 1115, 1536}}, {
+    576, {
+  1152, 1253, 1728}}, {
+    576, {
+  1152, 1254, 1728}}, {
+    640, {
+  1280, 1393, 1920}}, {
+    640, {
+  1280, 1394, 1920}}
 };
 
 static const guint fscod_rates[4] = { 48000, 44100, 32000, 0 };
@@ -154,8 +153,7 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-ac3; " "audio/x-eac3; " "audio/ac3; "
-        "audio/x-private1-ac3"));
+    GST_STATIC_CAPS ("audio/x-ac3; " "audio/x-eac3; " "audio/ac3"));
 
 static void gst_ac3_parse_finalize (GObject * object);
 
@@ -163,19 +161,13 @@ static gboolean gst_ac3_parse_start (GstBaseParse * parse);
 static gboolean gst_ac3_parse_stop (GstBaseParse * parse);
 static GstFlowReturn gst_ac3_parse_handle_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame, gint * skipsize);
-static GstFlowReturn gst_ac3_parse_pre_push_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame);
 static gboolean gst_ac3_parse_src_event (GstBaseParse * parse,
     GstEvent * event);
 static GstCaps *gst_ac3_parse_get_sink_caps (GstBaseParse * parse,
     GstCaps * filter);
-static gboolean gst_ac3_parse_set_sink_caps (GstBaseParse * parse,
-    GstCaps * caps);
 
 #define gst_ac3_parse_parent_class parent_class
 G_DEFINE_TYPE (GstAc3Parse, gst_ac3_parse, GST_TYPE_BASE_PARSE);
-GST_ELEMENT_REGISTER_DEFINE (ac3parse, "ac3parse",
-    GST_RANK_PRIMARY + 1, GST_TYPE_AC3_PARSE);
 
 static void
 gst_ac3_parse_class_init (GstAc3ParseClass * klass)
@@ -189,8 +181,10 @@ gst_ac3_parse_class_init (GstAc3ParseClass * klass)
 
   object_class->finalize = gst_ac3_parse_finalize;
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 
   gst_element_class_set_static_metadata (element_class,
       "AC3 audio stream parser", "Codec/Parser/Converter/Audio",
@@ -199,11 +193,8 @@ gst_ac3_parse_class_init (GstAc3ParseClass * klass)
   parse_class->start = GST_DEBUG_FUNCPTR (gst_ac3_parse_start);
   parse_class->stop = GST_DEBUG_FUNCPTR (gst_ac3_parse_stop);
   parse_class->handle_frame = GST_DEBUG_FUNCPTR (gst_ac3_parse_handle_frame);
-  parse_class->pre_push_frame =
-      GST_DEBUG_FUNCPTR (gst_ac3_parse_pre_push_frame);
   parse_class->src_event = GST_DEBUG_FUNCPTR (gst_ac3_parse_src_event);
   parse_class->get_sink_caps = GST_DEBUG_FUNCPTR (gst_ac3_parse_get_sink_caps);
-  parse_class->set_sink_caps = GST_DEBUG_FUNCPTR (gst_ac3_parse_set_sink_caps);
 }
 
 static void
@@ -213,19 +204,14 @@ gst_ac3_parse_reset (GstAc3Parse * ac3parse)
   ac3parse->sample_rate = -1;
   ac3parse->blocks = -1;
   ac3parse->eac = FALSE;
-  ac3parse->sent_codec_tag = FALSE;
   g_atomic_int_set (&ac3parse->align, GST_AC3_PARSE_ALIGN_NONE);
 }
 
 static void
 gst_ac3_parse_init (GstAc3Parse * ac3parse)
 {
-  gst_base_parse_set_min_frame_size (GST_BASE_PARSE (ac3parse), 8);
+  gst_base_parse_set_min_frame_size (GST_BASE_PARSE (ac3parse), 6);
   gst_ac3_parse_reset (ac3parse);
-  ac3parse->baseparse_chainfunc =
-      GST_BASE_PARSE_SINK_PAD (GST_BASE_PARSE (ac3parse))->chainfunc;
-  GST_PAD_SET_ACCEPT_INTERSECT (GST_BASE_PARSE_SINK_PAD (ac3parse));
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_BASE_PARSE_SINK_PAD (ac3parse));
 }
 
 static void
@@ -480,8 +466,7 @@ gst_ac3_parse_frame_header (GstAc3Parse * parse, GstBuffer * buf, gint skip,
     goto cleanup;
   } else {
     GST_DEBUG_OBJECT (parse, "unexpected bsid %d", bsid);
-    ret = FALSE;
-    goto cleanup;
+    return FALSE;
   }
 
   GST_DEBUG_OBJECT (parse, "unexpected bsid %d", bsid);
@@ -508,10 +493,11 @@ gst_ac3_parse_handle_frame (GstBaseParse * parse,
   gint have_blocks = 0;
   GstMapInfo map;
   gboolean ret = FALSE;
+  GstFlowReturn res = GST_FLOW_OK;
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
 
-  if (G_UNLIKELY (map.size < 8)) {
+  if (G_UNLIKELY (map.size < 6)) {
     *skipsize = 1;
     goto cleanup;
   }
@@ -603,7 +589,7 @@ gst_ac3_parse_handle_frame (GstBaseParse * parse,
     if (more || !gst_byte_reader_skip (&reader, frmsiz) ||
         !gst_byte_reader_get_uint16_be (&reader, &word)) {
       GST_DEBUG_OBJECT (ac3parse, "... but not sufficient data");
-      gst_base_parse_set_min_frame_size (parse, framesize + 8);
+      gst_base_parse_set_min_frame_size (parse, framesize + 6);
       *skipsize = 0;
       goto cleanup;
     } else {
@@ -620,7 +606,7 @@ gst_ac3_parse_handle_frame (GstBaseParse * parse,
 
   /* expect to have found a frame here */
   g_assert (framesize);
-  ret = framesize <= map.size;
+  ret = TRUE;
 
   /* arrange for metadata setup */
   if (G_UNLIKELY (sid)) {
@@ -667,152 +653,11 @@ gst_ac3_parse_handle_frame (GstBaseParse * parse,
 cleanup:
   gst_buffer_unmap (buf, &map);
 
-  if (ret) {
-    return gst_base_parse_finish_frame (parse, frame, framesize);
+  if (ret && framesize <= map.size) {
+    res = gst_base_parse_finish_frame (parse, frame, framesize);
   }
 
-  return GST_FLOW_OK;
-}
-
-
-/*
- * MPEG-PS private1 streams add a 2 bytes "Audio Substream Headers" for each
- * buffer (not each frame) with the offset of the next frame's start.
- *
- * Buffer 1:
- * -------------------------------------------
- * |firstAccUnit|AC3SyncWord|xxxxxxxxxxxxxxxxx
- * -------------------------------------------
- * Buffer 2:
- * -------------------------------------------
- * |firstAccUnit|xxxxxx|AC3SyncWord|xxxxxxxxxx
- * -------------------------------------------
- *
- * These 2 bytes can be dropped safely as they do not include any timing
- * information, only the offset to the start of the next frame.
- *
- * From http://stnsoft.com/DVD/ass-hdr.html:
- * "FirstAccUnit offset to frame which corresponds to PTS value offset 0 is the
- * last byte of FirstAccUnit, ie add the offset of byte 2 to get the AU's offset
- * The value 0000 indicates there is no first access unit"
- * */
-
-static GstFlowReturn
-gst_ac3_parse_chain_priv (GstPad * pad, GstObject * parent, GstBuffer * buf)
-{
-  GstAc3Parse *ac3parse = GST_AC3_PARSE (parent);
-  GstFlowReturn ret;
-  gsize size;
-  guint8 data[2];
-  gint offset;
-  gint len;
-  GstBuffer *subbuf;
-  gint first_access;
-
-  size = gst_buffer_get_size (buf);
-  if (size < 2)
-    goto not_enough_data;
-
-  gst_buffer_extract (buf, 0, data, 2);
-  first_access = (data[0] << 8) | data[1];
-
-  /* Skip the first_access header */
-  offset = 2;
-
-  if (first_access > 1) {
-    /* Length of data before first_access */
-    len = first_access - 1;
-
-    if (len <= 0 || offset + len > size)
-      goto bad_first_access_parameter;
-
-    subbuf = gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, offset, len);
-    GST_BUFFER_DTS (subbuf) = GST_CLOCK_TIME_NONE;
-    GST_BUFFER_PTS (subbuf) = GST_CLOCK_TIME_NONE;
-    ret = ac3parse->baseparse_chainfunc (pad, parent, subbuf);
-    if (ret != GST_FLOW_OK && ret != GST_FLOW_NOT_LINKED) {
-      gst_buffer_unref (buf);
-      goto done;
-    }
-
-    offset += len;
-    len = size - offset;
-
-    if (len > 0) {
-      subbuf = gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, offset, len);
-      GST_BUFFER_PTS (subbuf) = GST_BUFFER_PTS (buf);
-      GST_BUFFER_DTS (subbuf) = GST_BUFFER_DTS (buf);
-
-      ret = ac3parse->baseparse_chainfunc (pad, parent, subbuf);
-    }
-    gst_buffer_unref (buf);
-  } else {
-    /* first_access = 0 or 1, so if there's a timestamp it applies to the first byte */
-    subbuf =
-        gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, offset,
-        size - offset);
-    GST_BUFFER_PTS (subbuf) = GST_BUFFER_PTS (buf);
-    GST_BUFFER_DTS (subbuf) = GST_BUFFER_DTS (buf);
-    gst_buffer_unref (buf);
-    ret = ac3parse->baseparse_chainfunc (pad, parent, subbuf);
-  }
-
-done:
-  return ret;
-
-/* ERRORS */
-not_enough_data:
-  {
-    GST_ELEMENT_ERROR (GST_ELEMENT (ac3parse), STREAM, FORMAT, (NULL),
-        ("Insufficient data in buffer. Can't determine first_acess"));
-    gst_buffer_unref (buf);
-    return GST_FLOW_ERROR;
-  }
-bad_first_access_parameter:
-  {
-    GST_ELEMENT_ERROR (GST_ELEMENT (ac3parse), STREAM, FORMAT, (NULL),
-        ("Bad first_access parameter (%d) in buffer", first_access));
-    gst_buffer_unref (buf);
-    return GST_FLOW_ERROR;
-  }
-}
-
-static GstFlowReturn
-gst_ac3_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
-{
-  GstAc3Parse *ac3parse = GST_AC3_PARSE (parse);
-
-  if (!ac3parse->sent_codec_tag) {
-    GstTagList *taglist;
-    GstCaps *caps;
-
-    /* codec tag */
-    caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (parse));
-    if (G_UNLIKELY (caps == NULL)) {
-      if (GST_PAD_IS_FLUSHING (GST_BASE_PARSE_SRC_PAD (parse))) {
-        GST_INFO_OBJECT (parse, "Src pad is flushing");
-        return GST_FLOW_FLUSHING;
-      } else {
-        GST_INFO_OBJECT (parse, "Src pad is not negotiated!");
-        return GST_FLOW_NOT_NEGOTIATED;
-      }
-    }
-
-    taglist = gst_tag_list_new_empty ();
-    gst_pb_utils_add_codec_description_to_tag_list (taglist,
-        GST_TAG_AUDIO_CODEC, caps);
-    gst_caps_unref (caps);
-
-    gst_base_parse_merge_tags (parse, taglist, GST_TAG_MERGE_REPLACE);
-    gst_tag_list_unref (taglist);
-
-    /* also signals the end of first-frame processing */
-    ac3parse->sent_codec_tag = TRUE;
-  }
-
-  frame->flags |= GST_BASE_PARSE_FRAME_FLAG_CLIP;
-
-  return GST_FLOW_OK;
+  return res;
 }
 
 static gboolean
@@ -845,57 +690,6 @@ gst_ac3_parse_src_event (GstBaseParse * parse, GstEvent * event)
   return GST_BASE_PARSE_CLASS (parent_class)->src_event (parse, event);
 }
 
-static void
-remove_fields (GstCaps * caps)
-{
-  guint i, n;
-
-  n = gst_caps_get_size (caps);
-  for (i = 0; i < n; i++) {
-    GstStructure *s = gst_caps_get_structure (caps, i);
-
-    gst_structure_remove_field (s, "framed");
-    gst_structure_remove_field (s, "alignment");
-  }
-}
-
-static GstCaps *
-extend_caps (GstCaps * caps, gboolean add_private)
-{
-  guint i, n;
-  GstCaps *ncaps = gst_caps_new_empty ();
-
-  n = gst_caps_get_size (caps);
-  for (i = 0; i < n; i++) {
-    GstStructure *s = gst_caps_get_structure (caps, i);
-
-    if (add_private && !gst_structure_has_name (s, "audio/x-private1-ac3")) {
-      GstStructure *ns = gst_structure_copy (s);
-      gst_structure_set_name (ns, "audio/x-private1-ac3");
-      gst_caps_append_structure (ncaps, ns);
-    } else if (!add_private &&
-        gst_structure_has_name (s, "audio/x-private1-ac3")) {
-      GstStructure *ns = gst_structure_copy (s);
-      gst_structure_set_name (ns, "audio/x-ac3");
-      gst_caps_append_structure (ncaps, ns);
-      ns = gst_structure_copy (s);
-      gst_structure_set_name (ns, "audio/x-eac3");
-      gst_caps_append_structure (ncaps, ns);
-    } else if (!add_private) {
-      gst_caps_append_structure (ncaps, gst_structure_copy (s));
-    }
-  }
-
-  if (add_private) {
-    gst_caps_append (caps, ncaps);
-  } else {
-    gst_caps_unref (caps);
-    caps = ncaps;
-  }
-
-  return caps;
-}
-
 static GstCaps *
 gst_ac3_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
 {
@@ -903,28 +697,31 @@ gst_ac3_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
   GstCaps *res;
 
   templ = gst_pad_get_pad_template_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  if (filter) {
-    GstCaps *fcopy = gst_caps_copy (filter);
-    /* Remove the fields we convert */
-    remove_fields (fcopy);
-    /* we do not ask downstream to handle x-private1-ac3 */
-    fcopy = extend_caps (fcopy, FALSE);
-    peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), fcopy);
-    gst_caps_unref (fcopy);
-  } else
-    peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), NULL);
+  peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), filter);
 
   if (peercaps) {
+    guint i, n;
+
     /* Remove the framed and alignment field. We can convert
      * between different alignments. */
     peercaps = gst_caps_make_writable (peercaps);
-    remove_fields (peercaps);
-    /* also allow for x-private1-ac3 input */
-    peercaps = extend_caps (peercaps, TRUE);
+    n = gst_caps_get_size (peercaps);
+    for (i = 0; i < n; i++) {
+      GstStructure *s = gst_caps_get_structure (peercaps, i);
+
+      gst_structure_remove_field (s, "framed");
+      gst_structure_remove_field (s, "alignment");
+    }
 
     res = gst_caps_intersect_full (peercaps, templ, GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (peercaps);
-    gst_caps_unref (templ);
+    res = gst_caps_make_writable (res);
+
+    /* Append the template caps because we still want to accept
+     * caps without any fields in the case upstream does not
+     * know anything.
+     */
+    gst_caps_append (res, templ);
   } else {
     res = templ;
   }
@@ -940,18 +737,3 @@ gst_ac3_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
 
   return res;
 }
-
-static gboolean
-gst_ac3_parse_set_sink_caps (GstBaseParse * parse, GstCaps * caps)
-{
-  GstStructure *s;
-  GstAc3Parse *ac3parse = GST_AC3_PARSE (parse);
-
-  s = gst_caps_get_structure (caps, 0);
-  if (gst_structure_has_name (s, "audio/x-private1-ac3")) {
-    gst_pad_set_chain_function (parse->sinkpad, gst_ac3_parse_chain_priv);
-  } else {
-    gst_pad_set_chain_function (parse->sinkpad, ac3parse->baseparse_chainfunc);
-  }
-  return TRUE;
-}
