@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
@@ -174,8 +174,6 @@ gst_au_parse_reset (GstAuParse * auparse)
 
   gst_adapter_clear (auparse->adapter);
 
-  gst_caps_replace (&auparse->src_caps, NULL);
-
   /* gst_segment_init (&auparse->segment, GST_FORMAT_TIME); */
 }
 
@@ -421,6 +419,7 @@ gst_au_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   gint64 timestamp;
   gint64 duration;
   gint64 offset;
+  GstSegment segment;
 
   auparse = GST_AU_PARSE (parent);
 
@@ -442,11 +441,8 @@ gst_au_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     if (ret != GST_FLOW_OK)
       goto out;
 
-    if (auparse->need_segment) {
-      gst_pad_push_event (auparse->srcpad,
-          gst_event_new_segment (&auparse->segment));
-      auparse->need_segment = FALSE;
-    }
+    gst_segment_init (&segment, GST_FORMAT_TIME);
+    gst_pad_push_event (auparse->srcpad, gst_event_new_segment (&segment));
   }
 
   avail = gst_adapter_available (auparse->adapter);
@@ -692,6 +688,7 @@ gst_au_parse_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     {
       gint64 start, stop, offset = 0;
       GstSegment segment;
+      GstEvent *new_event = NULL;
 
       /* some debug output */
       gst_event_copy_segment (event, &segment);
@@ -723,17 +720,9 @@ gst_au_parse_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
       gst_segment_init (&segment, GST_FORMAT_TIME);
       segment.start = segment.time = start;
       segment.stop = stop;
+      new_event = gst_event_new_segment (&segment);
 
-      gst_segment_copy_into (&segment, &auparse->segment);
-
-      if (!gst_pad_has_current_caps (auparse->srcpad)) {
-        auparse->need_segment = TRUE;
-        ret = TRUE;
-      } else {
-        auparse->need_segment = FALSE;
-        ret = gst_pad_push_event (auparse->srcpad,
-            gst_event_new_segment (&segment));
-      }
+      ret = gst_pad_push_event (auparse->srcpad, new_event);
 
       auparse->buffer_offset = offset;
 
@@ -765,7 +754,6 @@ gst_au_parse_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
       ret = gst_au_parse_handle_seek (auparse, event);
-      gst_event_unref (event);
       break;
     default:
       ret = gst_pad_event_default (pad, parent, event);
