@@ -14,23 +14,27 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:element-avisubtitle
- * @title: avisubtitle
  *
+ * <refsect2>
+ * <para>
  * Parses the subtitle stream from an avi file.
- *
- * ## Example launch line
- *
- * |[
+ * </para>
+ * <title>Example launch line</title>
+ * <para>
+ * <programlisting>
  * gst-launch-1.0 filesrc location=subtitle.avi ! avidemux name=demux ! queue ! avisubtitle ! subparse ! textoverlay name=overlay ! videoconvert ! autovideosink demux. ! queue ! decodebin ! overlay.
- * ]|
+ * </programlisting>
  * This plays an avi file with a video and subtitle stream.
+ * </para>
+ * </refsect2>
  *
+ * Last reviewed on 2008-02-01
  */
 
 /* example of a subtitle chunk in an avi file
@@ -325,8 +329,10 @@ gst_avi_subtitle_class_init (GstAviSubtitleClass * klass)
   gstelement_class->send_event =
       GST_DEBUG_FUNCPTR (gst_avi_subtitle_send_event);
 
-  gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
-  gst_element_class_add_static_pad_template (gstelement_class, &src_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Avi subtitle parser", "Codec/Parser/Subtitle",
