@@ -22,42 +22,37 @@
 
 /**
  * SECTION:element-rgvolume
- * @title: rgvolume
  * @see_also: #GstRgLimiter, #GstRgAnalysis
  *
  * This element applies volume changes to streams as lined out in the proposed
- * [ReplayGain standard](https://wiki.hydrogenaud.io/index.php?title=ReplayGain)
- * and in the [EBU R 128](https://tech.ebu.ch/publications/r128/) standard.
- * It interprets the ReplayGain and R128 meta data tags and carries out the
- * adjustment (by using a volume element internally).
- * If R128 tags are present, ReplayGain tags are ignored.
- *
- * The relevant tags are:
- * * #GST_TAG_TRACK_GAIN
- * * #GST_TAG_TRACK_GAIN_R128
- * * #GST_TAG_TRACK_PEAK
- * * #GST_TAG_ALBUM_GAIN
- * * #GST_TAG_ALBUM_GAIN_R128
- * * #GST_TAG_ALBUM_PEAK
- * * #GST_TAG_REFERENCE_LEVEL
- *
+ * <ulink url="http://replaygain.org">ReplayGain standard</ulink>.  It
+ * interprets the ReplayGain meta data tags and carries out the adjustment (by
+ * using a volume element internally).  The relevant tags are:
+ * <itemizedlist>
+ * <listitem>#GST_TAG_TRACK_GAIN</listitem>
+ * <listitem>#GST_TAG_TRACK_PEAK</listitem>
+ * <listitem>#GST_TAG_ALBUM_GAIN</listitem>
+ * <listitem>#GST_TAG_ALBUM_PEAK</listitem>
+ * <listitem>#GST_TAG_REFERENCE_LEVEL</listitem>
+ * </itemizedlist>
  * The information carried by these tags must have been calculated beforehand by
  * performing the ReplayGain analysis.  This is implemented by the <link
  * linkend="GstRgAnalysis">rganalysis</link> element.
- *
+ * 
  * The signal compression/limiting recommendations outlined in the proposed
  * standard are not implemented by this element.  This has to be handled by
  * separate elements because applications might want to have additional filters
  * between the volume adjustment and the limiting stage.  A basic limiter is
  * included with this plugin: The <link linkend="GstRgLimiter">rglimiter</link>
  * element applies -6 dB hard limiting as mentioned in the ReplayGain standard.
- *
- * ## Example launch line
+ * 
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 filesrc location=filename.ext ! decodebin ! audioconvert \
  *     ! rgvolume ! audioconvert ! audioresample ! alsasink
  * ]| Playback of a file
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -122,8 +117,6 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_rg_volume_parent_class parent_class
 G_DEFINE_TYPE (GstRgVolume, gst_rg_volume, GST_TYPE_BIN);
-GST_ELEMENT_REGISTER_DEFINE (rgvolume, "rgvolume", GST_RANK_NONE,
-    GST_TYPE_RG_VOLUME);
 
 static void gst_rg_volume_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -169,7 +162,8 @@ gst_rg_volume_class_init (GstRgVolumeClass * klass)
    *
    * If album mode is enabled but the album gain tag is absent in the stream,
    * the track gain is used instead.  If both gain tags are missing, the value
-   * of the #GstRgVolume:fallback-gain property is used instead.
+   * of the <link linkend="GstRgVolume--fallback-gain">fallback-gain</link>
+   * property is used instead.
    */
   g_object_class_install_property (gobject_class, PROP_ALBUM_MODE,
       g_param_spec_boolean ("album-mode", "Album mode",
@@ -204,7 +198,9 @@ gst_rg_volume_class_init (GstRgVolumeClass * klass)
    * range.
    *
    * Note that the default value is 0 dB because the ReplayGain reference value
-   * was adjusted by +6 dB (from 83 to 89 dB). The original proposal stated
+   * was adjusted by +6 dB (from 83 to 89 dB).  At the time of this writing, the
+   * <ulink url="http://replaygain.org">webpage</ulink> is still outdated and
+   * does not reflect this change however.  Where the original proposal states
    * that a proper default pre-amp value is +6 dB, this translates to the used 0
    * dB.
    */
@@ -227,22 +223,24 @@ gst_rg_volume_class_init (GstRgVolumeClass * klass)
    *
    * Applied gain [dB].  This gain is applied to processed buffer data.
    *
-   * This is set to the #GstRgVolume:target-gain if amplification by that amount
-   * can be applied safely. "Safely" means that the volume adjustment does not
-   * inflict clipping distortion.  Should this not be the case, the result gain
-   * is set to an appropriately reduced value (by applying peak normalization).
-   * The proposed standard calls this "clipping prevention".
+   * This is set to the <link linkend="GstRgVolume--target-gain">target
+   * gain</link> if amplification by that amount can be applied safely.
+   * "Safely" means that the volume adjustment does not inflict clipping
+   * distortion.  Should this not be the case, the result gain is set to an
+   * appropriately reduced value (by applying peak normalization).  The proposed
+   * standard calls this "clipping prevention".
    *
    * The difference between target and result gain reflects the necessary amount
    * of reduction.  Applications can make use of this information to temporarily
-   * reduce the #GstRgVolume:pre-amp for subsequent streams, as recommended by
-   * the ReplayGain standard.
+   * reduce the <link linkend="GstRgVolume--pre-amp">pre-amp</link> for
+   * subsequent streams, as recommended by the ReplayGain standard.
    *
    * Note that target and result gain differing for a great majority of streams
    * indicates a problem: What happens in this case is that most streams receive
    * peak normalization instead of amplification by the ideal replay gain.  To
-   * prevent this, the #GstRgVolume:pre-amp has to be lowered and/or a limiter
-   * has to be used which facilitates the use of #GstRgVolume:headroom.
+   * prevent this, the <link linkend="GstRgVolume--pre-amp">pre-amp</link> has
+   * to be lowered and/or a limiter has to be used which facilitates the use of
+   * <link linkend="GstRgVolume--headroom">headroom</link>.
    */
   g_object_class_install_property (gobject_class, PROP_RESULT_GAIN,
       g_param_spec_double ("result-gain", "Result-gain", "Applied gain [dB]",
@@ -252,15 +250,19 @@ gst_rg_volume_class_init (GstRgVolumeClass * klass)
    *
    * Applicable gain [dB].  This gain is supposed to be applied.
    *
-   * Depending on the value of the #GstRgVolume:album-mode property and the
+   * Depending on the value of the <link
+   * linkend="GstRgVolume--album-mode">album-mode</link> property and the
    * presence of ReplayGain tags in the stream, this is set according to one of
    * these simple formulas:
    *
-   *
-   * * #GstRgVolume:pre-amp + album gain of the stream
-   * * #GstRgVolume:pre-amp + track gain of the stream
-   * * #GstRgVolume:pre-amp + #GstRgVolume:fallback-gain
-   *
+   * <itemizedlist>
+   * <listitem><link linkend="GstRgVolume--pre-amp">pre-amp</link> + album gain
+   * of the stream</listitem>
+   * <listitem><link linkend="GstRgVolume--pre-amp">pre-amp</link> + track gain
+   * of the stream</listitem>
+   * <listitem><link linkend="GstRgVolume--pre-amp">pre-amp</link> + <link
+   * linkend="GstRgVolume--fallback-gain">fallback gain</link></listitem>
+   * </itemizedlist>
    */
   g_object_class_install_property (gobject_class, PROP_TARGET_GAIN,
       g_param_spec_double ("target-gain", "Target-gain",
@@ -276,8 +278,10 @@ gst_rg_volume_class_init (GstRgVolumeClass * klass)
   bin_class->add_element = NULL;
   bin_class->remove_element = NULL;
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
   gst_element_class_set_static_metadata (element_class, "ReplayGain volume",
       "Filter/Effect/Audio",
       "Apply ReplayGain volume adjustment",
@@ -323,14 +327,14 @@ gst_rg_volume_init (GstRgVolume * self)
 
   volume_pad = gst_element_get_static_pad (self->volume_element, "sink");
   ghost_pad = gst_ghost_pad_new_from_template ("sink", volume_pad,
-      GST_PAD_PAD_TEMPLATE (volume_pad));
+      gst_pad_get_pad_template (volume_pad));
   gst_object_unref (volume_pad);
   gst_pad_set_event_function (ghost_pad, gst_rg_volume_sink_event);
   gst_element_add_pad (GST_ELEMENT_CAST (self), ghost_pad);
 
   volume_pad = gst_element_get_static_pad (self->volume_element, "src");
   ghost_pad = gst_ghost_pad_new_from_template ("src", volume_pad,
-      GST_PAD_PAD_TEMPLATE (volume_pad));
+      gst_pad_get_pad_template (volume_pad));
   gst_object_unref (volume_pad);
   gst_element_add_pad (GST_ELEMENT_CAST (self), ghost_pad);
 }
@@ -444,10 +448,12 @@ static gboolean
 gst_rg_volume_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   GstRgVolume *self;
+  GstPad *volume_sink_pad;
   GstEvent *send_event = event;
   gboolean res;
 
   self = GST_RG_VOLUME (parent);
+  volume_sink_pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_TAG:
@@ -471,10 +477,12 @@ gst_rg_volume_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
   }
 
   if (G_LIKELY (send_event != NULL))
-    res = gst_pad_event_default (pad, parent, send_event);
+    res = gst_pad_send_event (volume_sink_pad, send_event);
   else
     res = TRUE;
 
+  gst_object_unref (volume_sink_pad);
+
   return res;
 }
 
@@ -483,9 +491,7 @@ gst_rg_volume_tag_event (GstRgVolume * self, GstEvent * event)
 {
   GstTagList *tag_list;
   gboolean has_track_gain, has_track_peak, has_album_gain, has_album_peak;
-  gboolean has_track_gain_r128, has_album_gain_r128;
   gboolean has_ref_level;
-  gdouble track_gain_r128, album_gain_r128;
 
   g_return_val_if_fail (event != NULL, NULL);
   g_return_val_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_TAG, event);
@@ -497,21 +503,16 @@ gst_rg_volume_tag_event (GstRgVolume * self, GstEvent * event)
 
   has_track_gain = gst_tag_list_get_double (tag_list, GST_TAG_TRACK_GAIN,
       &self->track_gain);
-  has_track_gain_r128 = gst_tag_list_get_double (tag_list,
-      GST_TAG_TRACK_GAIN_R128, &track_gain_r128);
   has_track_peak = gst_tag_list_get_double (tag_list, GST_TAG_TRACK_PEAK,
       &self->track_peak);
   has_album_gain = gst_tag_list_get_double (tag_list, GST_TAG_ALBUM_GAIN,
       &self->album_gain);
-  has_album_gain_r128 = gst_tag_list_get_double (tag_list,
-      GST_TAG_ALBUM_GAIN_R128, &album_gain_r128);
   has_album_peak = gst_tag_list_get_double (tag_list, GST_TAG_ALBUM_PEAK,
       &self->album_peak);
   has_ref_level = gst_tag_list_get_double (tag_list, GST_TAG_REFERENCE_LEVEL,
       &self->reference_level);
 
-  if (!has_track_gain && !has_track_gain_r128 && !has_track_peak &&
-      !has_album_gain && !has_album_gain_r128 && !has_album_peak)
+  if (!has_track_gain && !has_track_peak && !has_album_gain && !has_album_peak)
     return event;
 
   if (has_ref_level && (has_track_gain || has_album_gain)
@@ -528,25 +529,6 @@ gst_rg_volume_tag_event (GstRgVolume * self, GstEvent * event)
     self->album_gain += RG_REFERENCE_LEVEL - self->reference_level;
   }
 
-  /*
-   * R128 tags can be present together with RG tags, should take priority,
-   * and should not take interference from RG reference level or peak tags.
-   */
-  if (has_track_gain_r128) {
-    has_track_peak = FALSE;
-    self->track_gain = track_gain_r128 + 5.;
-    GST_DEBUG_OBJECT (self,
-        "decoded R128 track gain %lf as %" GAIN_FORMAT,
-        track_gain_r128, self->track_gain);
-  }
-  if (has_album_gain_r128) {
-    has_album_peak = FALSE;
-    self->album_gain = album_gain_r128 + 5.;
-    GST_DEBUG_OBJECT (self,
-        "decoded R128 album gain %lf as %" GAIN_FORMAT,
-        album_gain_r128, self->album_gain);
-  }
-
   /* Ignore values that are obviously invalid. */
   if (G_UNLIKELY (has_track_gain && !VALID_GAIN (self->track_gain))) {
     GST_DEBUG_OBJECT (self,
@@ -583,29 +565,28 @@ gst_rg_volume_tag_event (GstRgVolume * self, GstEvent * event)
     self->album_peak = 1.0;
   }
 
-  self->has_track_gain |= has_track_gain | has_track_gain_r128;
+  self->has_track_gain |= has_track_gain;
   self->has_track_peak |= has_track_peak;
-  self->has_album_gain |= has_album_gain | has_album_gain_r128;
+  self->has_album_gain |= has_album_gain;
   self->has_album_peak |= has_album_peak;
 
-  tag_list = gst_tag_list_copy (tag_list);
+  event = (GstEvent *) gst_mini_object_make_writable (GST_MINI_OBJECT (event));
+  gst_event_parse_tag (event, &tag_list);
+
   gst_tag_list_remove_tag (tag_list, GST_TAG_TRACK_GAIN);
-  gst_tag_list_remove_tag (tag_list, GST_TAG_TRACK_GAIN_R128);
   gst_tag_list_remove_tag (tag_list, GST_TAG_TRACK_PEAK);
   gst_tag_list_remove_tag (tag_list, GST_TAG_ALBUM_GAIN);
-  gst_tag_list_remove_tag (tag_list, GST_TAG_ALBUM_GAIN_R128);
   gst_tag_list_remove_tag (tag_list, GST_TAG_ALBUM_PEAK);
   gst_tag_list_remove_tag (tag_list, GST_TAG_REFERENCE_LEVEL);
 
   gst_rg_volume_update_gain (self);
 
-  gst_event_unref (event);
   if (gst_tag_list_is_empty (tag_list)) {
-    gst_tag_list_unref (tag_list);
-    return NULL;
+    gst_event_unref (event);
+    event = NULL;
   }
 
-  return gst_event_new_tag (tag_list);
+  return event;
 }
 
 static void
