@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,11 +26,11 @@
 #include <string.h>
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/base/gstadapter.h>
-#include <gst/audio/audio.h>
 
-#include "gstrtpelements.h"
 #include "gstrtpg723pay.h"
-#include "gstrtputils.h"
+
+#define GST_RTP_PAYLOAD_G723 4
+#define GST_RTP_PAYLOAD_G723_STRING "4"
 
 #define G723_FRAME_DURATION (30 * GST_MSECOND)
 
@@ -69,8 +69,6 @@ static GstStateChangeReturn gst_rtp_g723_pay_change_state (GstElement * element,
 
 #define gst_rtp_g723_pay_parent_class parent_class
 G_DEFINE_TYPE (GstRTPG723Pay, gst_rtp_g723_pay, GST_TYPE_RTP_BASE_PAYLOAD);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpg723pay, "rtpg723pay",
-    GST_RANK_SECONDARY, GST_TYPE_RTP_G723_PAY, rtp_element_init (plugin));
 
 static void
 gst_rtp_g723_pay_class_init (GstRTPG723PayClass * klass)
@@ -87,10 +85,10 @@ gst_rtp_g723_pay_class_init (GstRTPG723PayClass * klass)
 
   gstelement_class->change_state = gst_rtp_g723_pay_change_state;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_g723_pay_sink_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_g723_pay_src_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_g723_pay_sink_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_g723_pay_src_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP G.723 payloader", "Codec/Payloader/Network/RTP",
@@ -109,6 +107,7 @@ gst_rtp_g723_pay_init (GstRTPG723Pay * pay)
   pay->adapter = gst_adapter_new ();
 
   payload->pt = GST_RTP_PAYLOAD_G723;
+  gst_rtp_base_payload_set_options (payload, "audio", FALSE, "G723", 8000);
 }
 
 static void
@@ -129,9 +128,16 @@ static gboolean
 gst_rtp_g723_pay_set_caps (GstRTPBasePayload * payload, GstCaps * caps)
 {
   gboolean res;
+  GstStructure *structure;
+  gint pt;
+
+  structure = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_get_int (structure, "payload", &pt))
+    pt = GST_RTP_PAYLOAD_G723;
+
+  payload->pt = pt;
+  payload->dynamic = pt != GST_RTP_PAYLOAD_G723;
 
-  gst_rtp_base_payload_set_options (payload, "audio",
-      payload->pt != GST_RTP_PAYLOAD_G723, "G723", 8000);
   res = gst_rtp_base_payload_set_outcaps (payload, NULL);
 
   return res;
@@ -140,38 +146,37 @@ gst_rtp_g723_pay_set_caps (GstRTPBasePayload * payload, GstCaps * caps)
 static GstFlowReturn
 gst_rtp_g723_pay_flush (GstRTPG723Pay * pay)
 {
-  GstBuffer *outbuf, *payload_buf;
+  GstBuffer *outbuf;
   GstFlowReturn ret;
+  guint8 *payload;
   guint avail;
   GstRTPBuffer rtp = { NULL };
 
   avail = gst_adapter_available (pay->adapter);
 
-  outbuf =
-      gst_rtp_base_payload_allocate_output_buffer (GST_RTP_BASE_PAYLOAD (pay),
-      0, 0, 0);
-  gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
+  outbuf = gst_rtp_buffer_new_allocate (avail, 0, 0);
 
-  GST_BUFFER_PTS (outbuf) = pay->timestamp;
+  gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
+  payload = gst_rtp_buffer_get_payload (&rtp);
+
+  GST_BUFFER_TIMESTAMP (outbuf) = pay->timestamp;
   GST_BUFFER_DURATION (outbuf) = pay->duration;
 
   /* copy G723 data as payload */
-  payload_buf = gst_adapter_take_buffer_fast (pay->adapter, avail);
+  gst_adapter_copy (pay->adapter, payload, 0, avail);
 
+  /* flush bytes from adapter */
+  gst_adapter_flush (pay->adapter, avail);
   pay->timestamp = GST_CLOCK_TIME_NONE;
   pay->duration = 0;
 
   /* set discont and marker */
   if (pay->discont) {
     GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
-    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_MARKER);
     gst_rtp_buffer_set_marker (&rtp, TRUE);
     pay->discont = FALSE;
   }
   gst_rtp_buffer_unmap (&rtp);
-  gst_rtp_copy_audio_meta (pay, outbuf, payload_buf);
-
-  outbuf = gst_buffer_append (outbuf, payload_buf);
 
   ret = gst_rtp_base_payload_push (GST_RTP_BASE_PAYLOAD (pay), outbuf);
 
@@ -199,7 +204,7 @@ gst_rtp_g723_pay_handle_buffer (GstRTPBasePayload * payload, GstBuffer * buf)
   pay = GST_RTP_G723_PAY (payload);
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
-  timestamp = GST_BUFFER_PTS (buf);
+  timestamp = GST_BUFFER_TIMESTAMP (buf);
 
   if (GST_BUFFER_IS_DISCONT (buf)) {
     /* flush everything on discont */
@@ -302,3 +307,11 @@ gst_rtp_g723_pay_change_state (GstElement * element, GstStateChange transition)
 
   return ret;
 }
+
+/*Plugin init functions*/
+gboolean
+gst_rtp_g723_pay_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rtpg723pay", GST_RANK_SECONDARY,
+      gst_rtp_g723_pay_get_type ());
+}
