@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,13 +24,7 @@
 #include <string.h>
 
 #include <gst/rtp/gstrtpbuffer.h>
-#include <gst/video/video.h>
-#include "gstrtpelements.h"
 #include "gstrtph263pdepay.h"
-#include "gstrtputils.h"
-
-GST_DEBUG_CATEGORY_STATIC (rtph263pdepay_debug);
-#define GST_CAT_DEFAULT (rtph263pdepay_debug)
 
 static GstStaticPadTemplate gst_rtp_h263p_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -45,6 +39,7 @@ static GstStaticPadTemplate gst_rtp_h263p_depay_sink_template =
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) [1, MAX], "
         "encoding-name = (string) \"H263-1998\"; "
         /* optional params */
@@ -70,6 +65,7 @@ static GstStaticPadTemplate gst_rtp_h263p_depay_sink_template =
          */
         "application/x-rtp, "
         "media = (string) \"video\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) [1, MAX], "
         "encoding-name = (string) \"H263-2000\" "
         /* optional params */
@@ -85,8 +81,6 @@ static GstStaticPadTemplate gst_rtp_h263p_depay_sink_template =
 #define gst_rtp_h263p_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpH263PDepay, gst_rtp_h263p_depay,
     GST_TYPE_RTP_BASE_DEPAYLOAD);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtph263pdepay, "rtph263pdepay",
-    GST_RANK_SECONDARY, GST_TYPE_RTP_H263P_DEPAY, rtp_element_init (plugin));
 
 static void gst_rtp_h263p_depay_finalize (GObject * object);
 
@@ -94,7 +88,7 @@ static GstStateChangeReturn gst_rtp_h263p_depay_change_state (GstElement *
     element, GstStateChange transition);
 
 static GstBuffer *gst_rtp_h263p_depay_process (GstRTPBaseDepayload * depayload,
-    GstRTPBuffer * rtp);
+    GstBuffer * buf);
 gboolean gst_rtp_h263p_depay_setcaps (GstRTPBaseDepayload * filter,
     GstCaps * caps);
 
@@ -113,30 +107,24 @@ gst_rtp_h263p_depay_class_init (GstRtpH263PDepayClass * klass)
 
   gstelement_class->change_state = gst_rtp_h263p_depay_change_state;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_h263p_depay_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_h263p_depay_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_h263p_depay_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_h263p_depay_sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP H263 depayloader", "Codec/Depayloader/Network/RTP",
       "Extracts H263/+/++ video from RTP packets (RFC 4629)",
       "Wim Taymans <wim.taymans@gmail.com>");
 
-  gstrtpbasedepayload_class->process_rtp_packet = gst_rtp_h263p_depay_process;
+  gstrtpbasedepayload_class->process = gst_rtp_h263p_depay_process;
   gstrtpbasedepayload_class->set_caps = gst_rtp_h263p_depay_setcaps;
-
-  GST_DEBUG_CATEGORY_INIT (rtph263pdepay_debug, "rtph263pdepay", 0,
-      "H263+ Video RTP Depayloader");
 }
 
 static void
 gst_rtp_h263p_depay_init (GstRtpH263PDepay * rtph263pdepay)
 {
   rtph263pdepay->adapter = gst_adapter_new ();
-
-  gst_rtp_base_depayload_set_aggregate_hdrext_enabled (GST_RTP_BASE_DEPAYLOAD
-      (rtph263pdepay), TRUE);
 }
 
 static void
@@ -237,101 +225,8 @@ no_caps:
   }
 }
 
-static void
-gst_rtp_h263p_depay_decorate_output_buffer (GstRtpH263PDepay * rtph263pdepay,
-    GstBuffer * outbuf)
-{
-  gboolean is_intra = FALSE;
-  GstBitReader bits;
-  guint8 pic_hdr[16];
-  gsize pic_hdr_len = 0;
-  guint32 psc, ptype, mpptype;
-  guint8 ufep;
-
-  pic_hdr_len = gst_buffer_extract (outbuf, 0, pic_hdr, sizeof (pic_hdr));
-
-  GST_MEMDUMP_OBJECT (rtph263pdepay, "pic_hdr", pic_hdr, pic_hdr_len);
-
-#if 0
-  if (gst_debug_category_get_threshold (GST_CAT_DEFAULT) >= GST_LEVEL_MEMDUMP) {
-    gchar bit_str[1 + sizeof (pic_hdr) * 8] = { 0, };
-    guint8 b;
-
-    gst_bit_reader_init (&bits, pic_hdr, pic_hdr_len);
-    while ((gst_bit_reader_get_bits_uint8 (&bits, &b, 1))) {
-      g_strlcat (bit_str, b ? "1" : "0", sizeof (bit_str));
-    }
-    GST_TRACE_OBJECT (rtph263pdepay, "pic_hdr bits: %s", bit_str);
-  }
-#endif
-
-  gst_bit_reader_init (&bits, pic_hdr, pic_hdr_len);
-
-  /* PSC - Picture Start Code: 22 bits: 0000 0000 0000 0000 10 0000 */
-  if (!gst_bit_reader_get_bits_uint32 (&bits, &psc, 22) || psc != 0x20) {
-    GST_WARNING_OBJECT (rtph263pdepay, "No picture start code");
-    return;
-  }
-
-  /* TR - Temporal Reference: 8 bits */
-  if (!gst_bit_reader_skip (&bits, 8)) {
-    GST_WARNING_OBJECT (rtph263pdepay, "Short picture header: no TR");
-    return;
-  }
-
-  /* PTYPE (first 8 bits) */
-  if (!gst_bit_reader_get_bits_uint32 (&bits, &ptype, 8) || (ptype >> 6) != 2) {
-    GST_WARNING_OBJECT (rtph263pdepay, "Short picture header: no PTYPE");
-    return;
-  }
-
-  /* PTYPE: check for extended PTYPE (bits 6-8 = 111) */
-  if ((ptype & 7) != 7) {
-    /* No extended PTYPE, read remaining 5 bits */
-    if (!gst_bit_reader_get_bits_uint32 (&bits, &ptype, 5)) {
-      GST_WARNING_OBJECT (rtph263pdepay, "Short picture header: no PTYPE");
-      return;
-    }
-    is_intra = (ptype & 0x10) == 0;
-    goto done;
-  }
-
-  /* UFEP - Update Full Extended PTYPE */
-  ufep = 0;
-  if (!gst_bit_reader_get_bits_uint8 (&bits, &ufep, 3) || ufep > 1) {
-    GST_WARNING_OBJECT (rtph263pdepay, "Short picture header: no PLUSPTYPE, %d",
-        ufep);
-    return;
-  }
-
-  /* Skip optional part of PLUSPTYPE (OPPTYPE) */
-  if (ufep == 1 && !gst_bit_reader_skip (&bits, 18)) {
-    GST_WARNING_OBJECT (rtph263pdepay, "Short picture header: no OPPTYPE");
-    return;
-  }
-
-  /* Mandatory part of PLUSPTYPE (MPPTYPE) */
-  if (!gst_bit_reader_get_bits_uint32 (&bits, &mpptype, 9)
-      || (mpptype & 7) != 1) {
-    GST_WARNING_OBJECT (rtph263pdepay, "Short picture header: no MPPTYPE");
-    return;
-  }
-
-  is_intra = (mpptype >> 6) == 0;
-
-done:
-
-  if (is_intra) {
-    GST_LOG_OBJECT (rtph263pdepay, "I-frame");
-    GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
-  } else {
-    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
-  }
-}
-
 static GstBuffer *
-gst_rtp_h263p_depay_process (GstRTPBaseDepayload * depayload,
-    GstRTPBuffer * rtp)
+gst_rtp_h263p_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
 {
   GstRtpH263PDepay *rtph263pdepay;
   GstBuffer *outbuf;
@@ -340,25 +235,28 @@ gst_rtp_h263p_depay_process (GstRTPBaseDepayload * depayload,
   gboolean P, V, M;
   guint header_len;
   guint8 PLEN, PEBIT;
+  GstRTPBuffer rtp = { NULL };
 
   rtph263pdepay = GST_RTP_H263P_DEPAY (depayload);
 
   /* flush remaining data on discont */
-  if (GST_BUFFER_IS_DISCONT (rtp->buffer)) {
+  if (GST_BUFFER_IS_DISCONT (buf)) {
     GST_LOG_OBJECT (depayload, "DISCONT, flushing adapter");
     gst_adapter_clear (rtph263pdepay->adapter);
     rtph263pdepay->wait_start = TRUE;
   }
 
-  payload_len = gst_rtp_buffer_get_payload_len (rtp);
+  gst_rtp_buffer_map (buf, GST_MAP_READ, &rtp);
+
+  payload_len = gst_rtp_buffer_get_payload_len (&rtp);
   header_len = 2;
 
   if (payload_len < header_len)
     goto too_small;
 
-  payload = gst_rtp_buffer_get_payload (rtp);
+  payload = gst_rtp_buffer_get_payload (&rtp);
 
-  M = gst_rtp_buffer_get_marker (rtp);
+  M = gst_rtp_buffer_get_marker (&rtp);
 
   /*  0                   1
    *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
@@ -385,8 +283,12 @@ gst_rtp_h263p_depay_process (GstRTPBaseDepayload * depayload,
     goto too_small;
 
   if (P) {
+    /* FIXME, have to make the packet writable hear. Better to reset these
+     * bytes when we copy the packet below */
     rtph263pdepay->wait_start = FALSE;
     header_len -= 2;
+    payload[header_len] = 0;
+    payload[header_len + 1] = 0;
   }
 
   if (rtph263pdepay->wait_start)
@@ -398,51 +300,46 @@ gst_rtp_h263p_depay_process (GstRTPBaseDepayload * depayload,
   /* FIXME do not ignore the VRC header (See RFC 2429 section 4.2) */
   /* FIXME actually use the RTP picture header when it is lost in the network */
   /* for now strip off header */
+  payload += header_len;
   payload_len -= header_len;
 
   if (M) {
     /* frame is completed: append to previous, push it out */
     guint len, padlen;
     guint avail;
-    GstBuffer *padbuf;
+    GstMapInfo map;
 
     GST_LOG_OBJECT (depayload, "Frame complete");
 
-    outbuf =
-        gst_rtp_buffer_get_payload_subbuffer (rtp, header_len, payload_len);
-    if (P)
-      gst_buffer_memset (outbuf, 0, 0, 2);
-    gst_adapter_push (rtph263pdepay->adapter, outbuf);
-    outbuf = NULL;
-
     avail = gst_adapter_available (rtph263pdepay->adapter);
     len = avail + payload_len;
     padlen = (len % 4) + 4;
 
-    if (avail == 0)
-      goto empty_frame;
+    outbuf = gst_buffer_new_and_alloc (len + padlen);
 
-    outbuf = gst_adapter_take_buffer (rtph263pdepay->adapter, avail);
-    if (padlen) {
-      padbuf = gst_buffer_new_and_alloc (padlen);
-      gst_buffer_memset (padbuf, 0, 0, padlen);
-      outbuf = gst_buffer_append (outbuf, padbuf);
+    gst_buffer_map (outbuf, &map, GST_MAP_WRITE);
+    memset (map.data + len, 0, padlen);
+
+    /* prepend previous data */
+    if (avail > 0) {
+      gst_adapter_copy (rtph263pdepay->adapter, map.data, 0, avail);
+      gst_adapter_flush (rtph263pdepay->adapter, avail);
     }
-
-    gst_rtp_drop_non_video_meta (rtph263pdepay, outbuf);
-
-    gst_rtp_h263p_depay_decorate_output_buffer (rtph263pdepay, outbuf);
+    memcpy (map.data + avail, payload, payload_len);
+    gst_buffer_unmap (outbuf, &map);
+    gst_rtp_buffer_unmap (&rtp);
 
     return outbuf;
+
   } else {
     /* frame not completed: store in adapter */
+    outbuf = gst_buffer_new_and_alloc (payload_len);
+
     GST_LOG_OBJECT (depayload, "Frame incomplete, storing %d", payload_len);
+    gst_buffer_fill (outbuf, 0, payload, payload_len);
 
-    outbuf =
-        gst_rtp_buffer_get_payload_subbuffer (rtp, header_len, payload_len);
-    if (P)
-      gst_buffer_memset (outbuf, 0, 0, 2);
     gst_adapter_push (rtph263pdepay->adapter, outbuf);
+    gst_rtp_buffer_unmap (&rtp);
   }
   return NULL;
 
@@ -450,19 +347,13 @@ too_small:
   {
     GST_ELEMENT_WARNING (rtph263pdepay, STREAM, DECODE,
         ("Packet payload was too small"), (NULL));
-    gst_rtp_base_depayload_dropped (depayload);
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 waiting_start:
   {
     GST_DEBUG_OBJECT (rtph263pdepay, "waiting for picture start");
-    gst_rtp_base_depayload_dropped (depayload);
-    return NULL;
-  }
-empty_frame:
-  {
-    GST_WARNING_OBJECT (rtph263pdepay, "Depayloaded frame is empty, dropping");
-    gst_rtp_base_depayload_dropped (depayload);
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 }
@@ -497,3 +388,10 @@ gst_rtp_h263p_depay_change_state (GstElement * element,
   }
   return ret;
 }
+
+gboolean
+gst_rtp_h263p_depay_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rtph263pdepay",
+      GST_RANK_SECONDARY, GST_TYPE_RTP_H263P_DEPAY);
+}
