@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,7 +26,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "gstrtpelements.h"
 #include "gstrtpmparobustdepay.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpmparobustdepay_debug);
@@ -45,11 +44,13 @@ static GstStaticPadTemplate gst_rtp_mpa_robust_depay_sink_template =
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) 90000, "
         "encoding-name = (string) \"MPA-ROBUST\" " "; "
         /* draft versions appear still in use out there */
         "application/x-rtp, "
         "media = (string) \"audio\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) [1, MAX], "
         "encoding-name = (string) { \"X-MP3-DRAFT-00\", \"X-MP3-DRAFT-01\", "
         " \"X-MP3-DRAFT-02\", \"X-MP3-DRAFT-03\", \"X-MP3-DRAFT-04\", "
@@ -71,9 +72,6 @@ typedef struct _GstADUFrame
 #define gst_rtp_mpa_robust_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpMPARobustDepay, gst_rtp_mpa_robust_depay,
     GST_TYPE_RTP_BASE_DEPAYLOAD);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpmparobustdepay, "rtpmparobustdepay",
-    GST_RANK_SECONDARY, GST_TYPE_RTP_MPA_ROBUST_DEPAY,
-    rtp_element_init (plugin));
 
 static GstStateChangeReturn gst_rtp_mpa_robust_change_state (GstElement *
     element, GstStateChange transition);
@@ -81,7 +79,7 @@ static GstStateChangeReturn gst_rtp_mpa_robust_change_state (GstElement *
 static gboolean gst_rtp_mpa_robust_depay_setcaps (GstRTPBaseDepayload *
     depayload, GstCaps * caps);
 static GstBuffer *gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload *
-    depayload, GstRTPBuffer * rtp);
+    depayload, GstBuffer * buf);
 
 static void
 gst_rtp_mpa_robust_depay_finalize (GObject * object)
@@ -115,10 +113,10 @@ gst_rtp_mpa_robust_depay_class_init (GstRtpMPARobustDepayClass * klass)
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_rtp_mpa_robust_change_state);
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_mpa_robust_depay_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_mpa_robust_depay_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_mpa_robust_depay_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_mpa_robust_depay_sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP MPEG audio depayloader", "Codec/Depayloader/Network/RTP",
@@ -126,8 +124,7 @@ gst_rtp_mpa_robust_depay_class_init (GstRtpMPARobustDepayClass * klass)
       "Mark Nauwelaerts <mark.nauwelaerts@collabora.co.uk>");
 
   gstrtpbasedepayload_class->set_caps = gst_rtp_mpa_robust_depay_setcaps;
-  gstrtpbasedepayload_class->process_rtp_packet =
-      gst_rtp_mpa_robust_depay_process;
+  gstrtpbasedepayload_class->process = gst_rtp_mpa_robust_depay_process;
 }
 
 static void
@@ -278,7 +275,7 @@ gst_rtp_mpa_robust_depay_generate_dummy_frame (GstRtpMPARobustDepay *
   GstADUFrame *dummy;
   GstMapInfo map;
 
-  dummy = g_memdup2 (frame, sizeof (GstADUFrame));
+  dummy = g_slice_dup (GstADUFrame, frame);
 
   /* go for maximum bitrate */
   dummy->header = (frame->header & ~(0xf << 12)) | (0xe << 12);
@@ -295,7 +292,7 @@ gst_rtp_mpa_robust_depay_generate_dummy_frame (GstRtpMPARobustDepay *
   GST_WRITE_UINT32_BE (map.data, dummy->header);
   gst_buffer_unmap (dummy->buffer, &map);
 
-  GST_BUFFER_PTS (dummy->buffer) = GST_BUFFER_PTS (frame->buffer);
+  GST_BUFFER_TIMESTAMP (dummy->buffer) = GST_BUFFER_TIMESTAMP (frame->buffer);
 
   return dummy;
 }
@@ -319,7 +316,7 @@ gst_rtp_mpa_robust_depay_queue_frame (GstRtpMPARobustDepay * rtpmpadepay,
   if (map.size < 6)
     goto corrupt_frame;
 
-  frame = g_new0 (GstADUFrame, 1);
+  frame = g_slice_new0 (GstADUFrame);
   frame->header = GST_READ_UINT32_BE (map.data);
 
   size = mp3_type_frame_length_from_header (GST_ELEMENT_CAST (rtpmpadepay),
@@ -377,7 +374,7 @@ corrupt_frame:
     gst_buffer_unmap (buf, &map);
     gst_buffer_unref (buf);
     if (frame)
-      g_free (frame);
+      g_slice_free (GstADUFrame, frame);
     return FALSE;
   }
 }
@@ -387,7 +384,7 @@ gst_rtp_mpa_robust_depay_free_frame (GstADUFrame * frame)
 {
   if (frame->buffer)
     gst_buffer_unref (frame->buffer);
-  g_free (frame);
+  g_slice_free (GstADUFrame, frame);
 }
 
 static inline void
@@ -500,7 +497,7 @@ gst_rtp_mpa_robust_depay_push_mp3_frames (GstRtpMPARobustDepay * rtpmpadepay)
           frame->buffer);
       frame->buffer = NULL;
       /* and remove it from any further consideration */
-      g_free (frame);
+      g_slice_free (GstADUFrame, frame);
       g_queue_delete_link (rtpmpadepay->adu_frames, rtpmpadepay->cur_adu_frame);
       rtpmpadepay->cur_adu_frame = NULL;
       continue;
@@ -614,7 +611,7 @@ gst_rtp_mpa_robust_depay_push_mp3_frames (GstRtpMPARobustDepay * rtpmpadepay)
     flush:
       buf = gst_byte_writer_free_and_get_buffer (rtpmpadepay->mp3_frame);
       rtpmpadepay->mp3_frame = NULL;
-      GST_BUFFER_PTS (buf) = GST_BUFFER_PTS (head->buffer);
+      GST_BUFFER_TIMESTAMP (buf) = GST_BUFFER_TIMESTAMP (head->buffer);
       /* no longer need head ADU frame header and side info */
       /* NOTE maybe head == current, then size and offset go off a bit,
        * but current gets reset to NULL, and then also offset and size */
@@ -646,7 +643,7 @@ gst_rtp_mpa_robust_depay_submit_adu (GstRtpMPARobustDepay * rtpmpadepay,
 
 static GstBuffer *
 gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload * depayload,
-    GstRTPBuffer * rtp)
+    GstBuffer * buf)
 {
   GstRtpMPARobustDepay *rtpmpadepay;
   gint payload_len, offset;
@@ -654,17 +651,19 @@ gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload * depayload,
   gboolean cont, dtype;
   guint av, size;
   GstClockTime timestamp;
-  GstBuffer *buf;
+  GstRTPBuffer rtp = { NULL };
 
   rtpmpadepay = GST_RTP_MPA_ROBUST_DEPAY (depayload);
 
-  timestamp = GST_BUFFER_PTS (rtp->buffer);
+  timestamp = GST_BUFFER_TIMESTAMP (buf);
 
-  payload_len = gst_rtp_buffer_get_payload_len (rtp);
+  gst_rtp_buffer_map (buf, GST_MAP_READ, &rtp);
+
+  payload_len = gst_rtp_buffer_get_payload_len (&rtp);
   if (payload_len <= 1)
     goto short_read;
 
-  payload = gst_rtp_buffer_get_payload (rtp);
+  payload = gst_rtp_buffer_get_payload (&rtp);
   offset = 0;
   GST_LOG_OBJECT (rtpmpadepay, "payload_len: %d", payload_len);
 
@@ -682,8 +681,8 @@ gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload * depayload,
    */
   while (payload_len) {
     if (G_LIKELY (rtpmpadepay->has_descriptor)) {
-      cont = !!(payload[offset] & 0x80);
-      dtype = !!(payload[offset] & 0x40);
+      cont = ! !(payload[offset] & 0x80);
+      dtype = ! !(payload[offset] & 0x40);
       if (dtype) {
         size = (payload[offset] & 0x3f) << 8 | payload[offset + 1];
         payload_len--;
@@ -704,7 +703,7 @@ gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload * depayload,
     GST_LOG_OBJECT (rtpmpadepay, "offset %d has cont: %d, dtype: %d, size: %d",
         offset, cont, dtype, size);
 
-    buf = gst_rtp_buffer_get_payload_subbuffer (rtp, offset,
+    buf = gst_rtp_buffer_get_payload_subbuffer (&rtp, offset,
         MIN (size, payload_len));
 
     if (cont) {
@@ -717,9 +716,9 @@ gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload * depayload,
         av += gst_buffer_get_size (buf);
         gst_adapter_push (rtpmpadepay->adapter, buf);
         if (av == size) {
-          timestamp = gst_adapter_prev_pts (rtpmpadepay->adapter, NULL);
+          timestamp = gst_adapter_prev_timestamp (rtpmpadepay->adapter, NULL);
           buf = gst_adapter_take_buffer (rtpmpadepay->adapter, size);
-          GST_BUFFER_PTS (buf) = timestamp;
+          GST_BUFFER_TIMESTAMP (buf) = timestamp;
           gst_rtp_mpa_robust_depay_submit_adu (rtpmpadepay, buf);
         } else if (av > size) {
           GST_DEBUG_OBJECT (rtpmpadepay,
@@ -733,7 +732,7 @@ gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload * depayload,
       /* not continuation, first fragment or whole ADU */
       if (payload_len == size) {
         /* whole ADU */
-        GST_BUFFER_PTS (buf) = timestamp;
+        GST_BUFFER_TIMESTAMP (buf) = timestamp;
         gst_rtp_mpa_robust_depay_submit_adu (rtpmpadepay, buf);
       } else if (payload_len < size) {
         /* first fragment */
@@ -748,6 +747,7 @@ gst_rtp_mpa_robust_depay_process (GstRTPBaseDepayload * depayload,
     /* timestamp applies to first payload, no idea for subsequent ones */
     timestamp = GST_CLOCK_TIME_NONE;
   }
+  gst_rtp_buffer_unmap (&rtp);
 
   return NULL;
 
@@ -756,6 +756,7 @@ short_read:
   {
     GST_ELEMENT_WARNING (rtpmpadepay, STREAM, DECODE,
         (NULL), ("Packet contains invalid data"));
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 }
@@ -806,3 +807,10 @@ gst_rtp_mpa_robust_change_state (GstElement * element,
 
   return ret;
 }
+
+gboolean
+gst_rtp_mpa_robust_depay_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rtpmparobustdepay",
+      GST_RANK_SECONDARY, GST_TYPE_RTP_MPA_ROBUST_DEPAY);
+}
