@@ -1,4 +1,4 @@
-/*
+/* 
  * RTP Demux element
  *
  * Copyright (C) 2005 Nokia Corporation.
@@ -19,33 +19,35 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
- * SECTION:element-rtpptdemux
- * @title: rtpptdemux
+ * SECTION:element-gstrtpptdemux
  *
- * rtpptdemux acts as a demuxer for RTP packets based on the payload type of
+ * gstrtpptdemux acts as a demuxer for RTP packets based on the payload type of
  * the packets. Its main purpose is to allow an application to easily receive
  * and decode an RTP stream with multiple payload types.
- *
+ * 
  * For each payload type that is detected, a new pad will be created and the
  * #GstRtpPtDemux::new-payload-type signal will be emitted. When the payload for
  * the RTP stream changes, the #GstRtpPtDemux::payload-type-change signal will be
  * emitted.
- *
- * The element will try to set complete and unique application/x-rtp caps
- * on the output pads based on the result of the #GstRtpPtDemux::request-pt-map
- * signal.
- *
- * ## Example pipelines
+ * 
+ * The element will try to set complete and unique application/x-rtp caps on the
+ * outgoing buffers and pads based on the result of the
+ * #GstRtpPtDemux::request-pt-map signal.
+ * 
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
- * gst-launch-1.0 udpsrc caps="application/x-rtp" ! rtpptdemux ! fakesink
+ * gst-launch-1.0 udpsrc caps="application/x-rtp" ! gstrtpptdemux ! fakesink
  * ]| Takes an RTP stream and send the RTP packets with the first detected
  * payload type to fakesink, discarding the other payload types.
+ * </refsect2>
  *
+ * Last reviewed on 2007-05-28 (0.10.5)
  */
 
 /*
@@ -74,6 +76,7 @@
 #include <gst/gst.h>
 #include <gst/rtp/gstrtpbuffer.h>
 
+#include "gstrtpbin-marshal.h"
 #include "gstrtpptdemux.h"
 
 /* generic templates */
@@ -99,8 +102,8 @@ GST_DEBUG_CATEGORY_STATIC (gst_rtp_pt_demux_debug);
  */
 struct _GstRtpPtDemuxPad
 {
-  GstPad *pad;                  /*< pointer to the actual pad */
-  gint pt;                      /*< RTP payload-type attached to pad */
+  GstPad *pad;        /**< pointer to the actual pad */
+  gint pt;             /**< RTP payload-type attached to pad */
   gboolean newcaps;
 };
 
@@ -114,16 +117,8 @@ enum
   LAST_SIGNAL
 };
 
-enum
-{
-  PROP_0,
-  PROP_IGNORED_PTS,
-};
-
 #define gst_rtp_pt_demux_parent_class parent_class
 G_DEFINE_TYPE (GstRtpPtDemux, gst_rtp_pt_demux, GST_TYPE_ELEMENT);
-GST_ELEMENT_REGISTER_DEFINE (rtpptdemux, "rtpptdemux", GST_RANK_NONE,
-    GST_TYPE_RTP_PT_DEMUX);
 
 static void gst_rtp_pt_demux_finalize (GObject * object);
 
@@ -146,38 +141,6 @@ static gboolean gst_rtp_pt_demux_src_event (GstPad * pad, GstObject * parent,
 
 static guint gst_rtp_pt_demux_signals[LAST_SIGNAL] = { 0 };
 
-static void
-gst_rtp_pt_demux_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpPtDemux *rtpptdemux = GST_RTP_PT_DEMUX (object);
-
-  switch (prop_id) {
-    case PROP_IGNORED_PTS:
-      g_value_copy (value, &rtpptdemux->ignored_pts);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_pt_demux_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpPtDemux *rtpptdemux = GST_RTP_PT_DEMUX (object);
-
-  switch (prop_id) {
-    case PROP_IGNORED_PTS:
-      g_value_copy (&rtpptdemux->ignored_pts, value);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static void
 gst_rtp_pt_demux_class_init (GstRtpPtDemuxClass * klass)
 {
@@ -197,7 +160,8 @@ gst_rtp_pt_demux_class_init (GstRtpPtDemuxClass * klass)
   gst_rtp_pt_demux_signals[SIGNAL_REQUEST_PT_MAP] =
       g_signal_new ("request-pt-map", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRtpPtDemuxClass, request_pt_map),
-      NULL, NULL, NULL, GST_TYPE_CAPS, 1, G_TYPE_UINT);
+      NULL, NULL, gst_rtp_bin_marshal_BOXED__UINT, GST_TYPE_CAPS, 1,
+      G_TYPE_UINT);
 
   /**
    * GstRtpPtDemux::new-payload-type:
@@ -205,24 +169,26 @@ gst_rtp_pt_demux_class_init (GstRtpPtDemuxClass * klass)
    * @pt: the payload type
    * @pad: the pad with the new payload
    *
-   * Emitted when a new payload type pad has been created in @demux.
+   * Emited when a new payload type pad has been created in @demux.
    */
   gst_rtp_pt_demux_signals[SIGNAL_NEW_PAYLOAD_TYPE] =
       g_signal_new ("new-payload-type", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRtpPtDemuxClass, new_payload_type),
-      NULL, NULL, NULL, G_TYPE_NONE, 2, G_TYPE_UINT, GST_TYPE_PAD);
+      NULL, NULL, gst_rtp_bin_marshal_VOID__UINT_OBJECT, G_TYPE_NONE, 2,
+      G_TYPE_UINT, GST_TYPE_PAD);
 
   /**
    * GstRtpPtDemux::payload-type-change:
    * @demux: the object which received the signal
    * @pt: the new payload type
    *
-   * Emitted when the payload type changed.
+   * Emited when the payload type changed.
    */
   gst_rtp_pt_demux_signals[SIGNAL_PAYLOAD_TYPE_CHANGE] =
       g_signal_new ("payload-type-change", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRtpPtDemuxClass,
-          payload_type_change), NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_UINT);
+          payload_type_change), NULL, NULL, g_cclosure_marshal_VOID__UINT,
+      G_TYPE_NONE, 1, G_TYPE_UINT);
 
   /**
    * GstRtpPtDemux::clear-pt-map:
@@ -234,32 +200,8 @@ gst_rtp_pt_demux_class_init (GstRtpPtDemuxClass * klass)
   gst_rtp_pt_demux_signals[SIGNAL_CLEAR_PT_MAP] =
       g_signal_new ("clear-pt-map", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_ACTION | G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRtpPtDemuxClass,
-          clear_pt_map), NULL, NULL, NULL, G_TYPE_NONE, 0, G_TYPE_NONE);
-
-  gobject_klass->set_property = gst_rtp_pt_demux_set_property;
-  gobject_klass->get_property = gst_rtp_pt_demux_get_property;
-
-  /**
-   * GstRtpPtDemux:ignored-payload-types:
-   *
-   * If specified, packets with an ignored payload type will be dropped,
-   * instead of causing a new pad to be exposed for these to be pushed on.
-   *
-   * This is for example useful to drop FEC protection packets, as they
-   * need to go through the #GstRtpJitterBuffer, but cease to be useful
-   * past that point, #GstRtpBin will make use of this property for that
-   * purpose.
-   *
-   * Since: 1.14
-   */
-  g_object_class_install_property (gobject_klass, PROP_IGNORED_PTS,
-      gst_param_spec_array ("ignored-payload-types",
-          "Ignored payload types",
-          "Packets with these payload types will be dropped",
-          g_param_spec_int ("payload-types", "payload-types", "Payload types",
-              0, G_MAXINT, 0,
-              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS),
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          clear_pt_map), NULL, NULL, g_cclosure_marshal_VOID__VOID,
+      G_TYPE_NONE, 0, G_TYPE_NONE);
 
   gobject_klass->finalize = gst_rtp_pt_demux_finalize;
 
@@ -268,10 +210,10 @@ gst_rtp_pt_demux_class_init (GstRtpPtDemuxClass * klass)
 
   klass->clear_pt_map = GST_DEBUG_FUNCPTR (gst_rtp_pt_demux_clear_pt_map);
 
-  gst_element_class_add_static_pad_template (gstelement_klass,
-      &rtp_pt_demux_sink_template);
-  gst_element_class_add_static_pad_template (gstelement_klass,
-      &rtp_pt_demux_src_template);
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&rtp_pt_demux_sink_template));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&rtp_pt_demux_src_template));
 
   gst_element_class_set_static_metadata (gstelement_klass, "RTP Demux",
       "Demux/Network/RTP",
@@ -280,8 +222,6 @@ gst_rtp_pt_demux_class_init (GstRtpPtDemuxClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (gst_rtp_pt_demux_debug,
       "rtpptdemux", 0, "RTP codec demuxer");
-
-  GST_DEBUG_REGISTER_FUNCPTR (gst_rtp_pt_demux_chain);
 }
 
 static void
@@ -298,8 +238,6 @@ gst_rtp_pt_demux_init (GstRtpPtDemux * ptdemux)
   gst_pad_set_event_function (ptdemux->sink, gst_rtp_pt_demux_sink_event);
 
   gst_element_add_pad (GST_ELEMENT (ptdemux), ptdemux->sink);
-
-  g_value_init (&ptdemux->ignored_pts, GST_TYPE_ARRAY);
 }
 
 static void
@@ -307,17 +245,13 @@ gst_rtp_pt_demux_finalize (GObject * object)
 {
   gst_rtp_pt_demux_release (GST_RTP_PT_DEMUX (object));
 
-  g_value_unset (&GST_RTP_PT_DEMUX (object)->ignored_pts);
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
 gst_rtp_pt_demux_get_caps (GstRtpPtDemux * rtpdemux, guint pt)
 {
-  guint32 ssrc = 0;
-  gboolean have_ssrc = FALSE;
-  GstCaps *caps, *sink_caps;
+  GstCaps *caps;
   GValue ret = { 0 };
   GValue args[2] = { {0}, {0} };
 
@@ -336,26 +270,12 @@ gst_rtp_pt_demux_get_caps (GstRtpPtDemux * rtpdemux, guint pt)
   g_value_unset (&args[0]);
   g_value_unset (&args[1]);
   caps = g_value_dup_boxed (&ret);
-  sink_caps = gst_pad_get_current_caps (rtpdemux->sink);
   g_value_unset (&ret);
-
   if (caps == NULL) {
-    caps = sink_caps;
-  } else if (sink_caps) {
-    have_ssrc =
-        gst_structure_get_uint (gst_caps_get_structure (sink_caps, 0), "ssrc",
-        &ssrc);
-    gst_caps_unref (sink_caps);
+    caps = gst_pad_get_current_caps (rtpdemux->sink);
   }
 
-  if (caps != NULL) {
-    caps = gst_caps_make_writable (caps);
-    gst_caps_set_simple (caps, "payload", G_TYPE_INT, pt, NULL);
-    if (have_ssrc)
-      gst_caps_set_simple (caps, "ssrc", G_TYPE_UINT, ssrc, NULL);
-  }
-
-  GST_DEBUG_OBJECT (rtpdemux, "pt %d, got caps %" GST_PTR_FORMAT, pt, caps);
+  GST_DEBUG ("pt %d, got caps %" GST_PTR_FORMAT, pt, caps);
 
   return caps;
 }
@@ -366,7 +286,7 @@ gst_rtp_pt_demux_clear_pt_map (GstRtpPtDemux * rtpdemux)
   GSList *walk;
 
   GST_OBJECT_LOCK (rtpdemux);
-  GST_DEBUG_OBJECT (rtpdemux, "clearing pt map");
+  GST_DEBUG ("clearing pt map");
   for (walk = rtpdemux->srcpads; walk; walk = g_slist_next (walk)) {
     GstRtpPtDemuxPad *pad = walk->data;
 
@@ -417,31 +337,11 @@ forward_sticky_events (GstPad * pad, GstEvent ** event, gpointer user_data)
 {
   GstPad *srcpad = GST_PAD_CAST (user_data);
 
-  /* Stream start and caps have already been pushed */
-  if (GST_EVENT_TYPE (*event) >= GST_EVENT_SEGMENT)
-    gst_pad_push_event (srcpad, gst_event_ref (*event));
+  gst_pad_push_event (srcpad, gst_event_ref (*event));
 
   return TRUE;
 }
 
-static gboolean
-gst_rtp_pt_demux_pt_is_ignored (GstRtpPtDemux * ptdemux, guint8 pt)
-{
-  gboolean ret = FALSE;
-  guint i;
-
-  for (i = 0; i < gst_value_array_get_size (&ptdemux->ignored_pts); i++) {
-    const GValue *tmp = gst_value_array_get_value (&ptdemux->ignored_pts, i);
-
-    if (g_value_get_int (tmp) == pt) {
-      ret = TRUE;
-      break;
-    }
-  }
-
-  return ret;
-}
-
 static GstFlowReturn
 gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 {
@@ -460,9 +360,6 @@ gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   pt = gst_rtp_buffer_get_payload_type (&rtp);
   gst_rtp_buffer_unmap (&rtp);
 
-  if (gst_rtp_pt_demux_pt_is_ignored (rtpdemux, pt))
-    goto ignored;
-
   GST_DEBUG_OBJECT (rtpdemux, "received buffer for pt %d", pt);
 
   srcpad = find_pad_for_pt (rtpdemux, pt);
@@ -477,13 +374,6 @@ gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     if (!caps)
       goto no_caps;
 
-    /* must be after the get_caps() call as get_caps() may cause external code
-     * (e.g. rtpbin) to update the ignored-pt list */
-    if (gst_rtp_pt_demux_pt_is_ignored (rtpdemux, pt)) {
-      gst_clear_caps (&caps);
-      goto ignored;
-    }
-
     klass = GST_ELEMENT_GET_CLASS (rtpdemux);
     templ = gst_element_class_get_pad_template (klass, "src_%u");
     padname = g_strdup_printf ("src_%u", pt);
@@ -492,7 +382,7 @@ gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     g_free (padname);
     gst_pad_set_event_function (srcpad, gst_rtp_pt_demux_src_event);
 
-    GST_DEBUG_OBJECT (rtpdemux, "Adding pt=%d to the list.", pt);
+    GST_DEBUG ("Adding pt=%d to the list.", pt);
     rtpdemuxpad = g_slice_new0 (GstRtpPtDemuxPad);
     rtpdemuxpad->pt = pt;
     rtpdemuxpad->newcaps = FALSE;
@@ -504,44 +394,16 @@ gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
     gst_pad_set_active (srcpad, TRUE);
 
-    /* First push the stream-start event, it must always come first */
-    {
-      gchar *stream_id;
-      GstEvent *sink_event, *event;
-      guint group_id;
-      GstStreamFlags flags;
-
-      sink_event =
-          gst_pad_get_sticky_event (rtpdemux->sink, GST_EVENT_STREAM_START, 0);
-
-      stream_id =
-          gst_pad_create_stream_id_printf (srcpad, GST_ELEMENT_CAST (rtpdemux),
-          "%u", pt);
-
-      event = gst_event_new_stream_start (stream_id);
-      if (gst_event_parse_group_id (sink_event, &group_id)) {
-        gst_event_set_group_id (event, group_id);
-      }
-      gst_event_parse_stream_flags (sink_event, &flags);
-      gst_event_set_stream_flags (event, flags);
-
-      gst_pad_push_event (srcpad, event);
-
-      gst_event_unref (sink_event);
-      g_free (stream_id);
-    }
-
-    /* Then caps event is sent */
+    caps = gst_caps_make_writable (caps);
+    gst_caps_set_simple (caps, "payload", G_TYPE_INT, pt, NULL);
     gst_pad_set_caps (srcpad, caps);
     gst_caps_unref (caps);
 
-    /* First sticky events on sink pad are forwarded to the new src pad */
     gst_pad_sticky_events_foreach (rtpdemux->sink, forward_sticky_events,
         srcpad);
-
     gst_element_add_pad (GST_ELEMENT_CAST (rtpdemux), srcpad);
 
-    GST_DEBUG_OBJECT (rtpdemux, "emitting new-payload-type for pt %d", pt);
+    GST_DEBUG ("emitting new-payload-type for pt %d", pt);
     g_signal_emit (G_OBJECT (rtpdemux),
         gst_rtp_pt_demux_signals[SIGNAL_NEW_PAYLOAD_TYPE], 0, pt, srcpad);
   }
@@ -551,20 +413,21 @@ gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
     /* our own signal with an extra flag that this is the only pad */
     rtpdemux->last_pt = pt;
-    GST_DEBUG_OBJECT (rtpdemux, "emitting payload-type-changed for pt %d",
-        emit_pt);
+    GST_DEBUG ("emitting payload-type-changed for pt %d", emit_pt);
     g_signal_emit (G_OBJECT (rtpdemux),
         gst_rtp_pt_demux_signals[SIGNAL_PAYLOAD_TYPE_CHANGE], 0, emit_pt);
   }
 
   while (need_caps_for_pt (rtpdemux, pt)) {
-    GST_DEBUG_OBJECT (rtpdemux, "need new caps for %d", pt);
+    GST_DEBUG ("need new caps for %d", pt);
     caps = gst_rtp_pt_demux_get_caps (rtpdemux, pt);
     if (!caps)
       goto no_caps;
 
     clear_newcaps_for_pt (rtpdemux, pt);
 
+    caps = gst_caps_make_writable (caps);
+    gst_caps_set_simple (caps, "payload", G_TYPE_INT, pt, NULL);
     gst_pad_set_caps (srcpad, caps);
     gst_caps_unref (caps);
   }
@@ -576,21 +439,14 @@ gst_rtp_pt_demux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
   return ret;
 
-ignored:
-  {
-    GST_DEBUG_OBJECT (rtpdemux, "Dropped buffer for pt %d", pt);
-    gst_buffer_unref (buf);
-    return GST_FLOW_OK;
-  }
-
   /* ERRORS */
 invalid_buffer:
   {
-    /* this should not be fatal */
-    GST_ELEMENT_WARNING (rtpdemux, STREAM, DEMUX, (NULL),
+    /* this is fatal and should be filtered earlier */
+    GST_ELEMENT_ERROR (rtpdemux, STREAM, DECODE, (NULL),
         ("Dropping invalid RTP payload"));
     gst_buffer_unref (buf);
-    return GST_FLOW_OK;
+    return GST_FLOW_ERROR;
   }
 no_caps:
   {
@@ -771,8 +627,8 @@ gst_rtp_pt_demux_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       gst_rtp_pt_demux_release (ptdemux);
       break;
