@@ -17,8 +17,8 @@
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
-* Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-* Boston, MA 02110-1301, USA.
+* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+* Boston, MA 02111-1307, USA.
 *
 *
 * The development of this code was made possible due to the involvement
@@ -28,7 +28,6 @@
 
 /**
  * SECTION:element-directsoundsink
- * @title: directsoundsink
  *
  * This element lets you output sound using the DirectSound API.
  *
@@ -37,7 +36,8 @@
  * your pipeline works under all circumstances (those conversion elements will
  * act in passthrough-mode if no conversion is necessary).
  *
- * ## Example pipelines
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
  * gst-launch-1.0 -v audiotestsrc ! audioconvert ! volume volume=0.1 ! directsoundsink
  * ]| will output a sine wave (continuous beep sound) to your sound card (with
@@ -45,7 +45,7 @@
  * |[
  * gst-launch-1.0 -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! directsoundsink
  * ]| will play an Ogg/Vorbis audio file and output it.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,6 +53,7 @@
 #endif
 
 #include <gst/base/gstbasesink.h>
+#include <gst/audio/streamvolume.h>
 #include "gstdirectsoundsink.h"
 #include <gst/audio/gstaudioiec61937.h>
 
@@ -101,28 +102,30 @@ static gdouble gst_directsound_sink_get_volume (GstDirectSoundSink * sink);
 static void gst_directsound_sink_set_mute (GstDirectSoundSink * sink,
     gboolean mute);
 static gboolean gst_directsound_sink_get_mute (GstDirectSoundSink * sink);
-static const gchar *gst_directsound_sink_get_device (GstDirectSoundSink *
-    dsoundsink);
-static void gst_directsound_sink_set_device (GstDirectSoundSink * dsoundsink,
-    const gchar * device_id);
 
 static gboolean gst_directsound_sink_is_spdif_format (GstAudioRingBufferSpec *
     spec);
 
-static gchar *gst_hres_to_string (HRESULT hRes);
-
 static GstStaticPadTemplate directsoundsink_sink_factory =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_DIRECTSOUND_SINK_CAPS));
+    GST_STATIC_CAPS ("audio/x-raw, "
+        "format = (string) S16LE, "
+        "layout = (string) interleaved, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]; "
+        "audio/x-raw, "
+        "format = (string) S8, "
+        "layout = (string) interleaved, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ];"
+        "audio/x-ac3, framed = (boolean) true;"
+        "audio/x-dts, framed = (boolean) true;"));
 
 enum
 {
   PROP_0,
   PROP_VOLUME,
-  PROP_MUTE,
-  PROP_DEVICE
+  PROP_MUTE
 };
 
 #define gst_directsound_sink_parent_class parent_class
@@ -135,14 +138,7 @@ gst_directsound_sink_finalize (GObject * object)
 {
   GstDirectSoundSink *dsoundsink = GST_DIRECTSOUND_SINK (object);
 
-  g_free (dsoundsink->device_id);
-  dsoundsink->device_id = NULL;
-
   g_mutex_clear (&dsoundsink->dsound_lock);
-  gst_object_unref (dsoundsink->system_clock);
-  if (dsoundsink->write_wait_clock_id != NULL) {
-    gst_clock_id_unref (dsoundsink->write_wait_clock_id);
-  }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -160,6 +156,8 @@ gst_directsound_sink_class_init (GstDirectSoundSinkClass * klass)
   GST_DEBUG_CATEGORY_INIT (directsoundsink_debug, "directsoundsink", 0,
       "DirectSound sink");
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gobject_class->finalize = gst_directsound_sink_finalize;
   gobject_class->set_property = gst_directsound_sink_set_property;
   gobject_class->get_property = gst_directsound_sink_get_property;
@@ -194,19 +192,13 @@ gst_directsound_sink_class_init (GstDirectSoundSinkClass * klass)
           "Mute state of this stream", DEFAULT_MUTE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class,
-      PROP_DEVICE,
-      g_param_spec_string ("device", "Device",
-          "DirectSound playback device as a GUID string",
-          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   gst_element_class_set_static_metadata (element_class,
       "Direct Sound Audio Sink", "Sink/Audio",
       "Output to a sound card via Direct Sound",
       "Sebastien Moutte <sebastien@moutte.net>");
 
-  gst_element_class_add_static_pad_template (element_class,
-      &directsoundsink_sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&directsoundsink_sink_factory));
 }
 
 static void
@@ -214,15 +206,13 @@ gst_directsound_sink_init (GstDirectSoundSink * dsoundsink)
 {
   dsoundsink->volume = 100;
   dsoundsink->mute = FALSE;
-  dsoundsink->device_id = NULL;
   dsoundsink->pDS = NULL;
   dsoundsink->cached_caps = NULL;
   dsoundsink->pDSBSecondary = NULL;
   dsoundsink->current_circular_offset = 0;
   dsoundsink->buffer_size = DSBSIZE_MIN;
+  dsoundsink->volume = 100;
   g_mutex_init (&dsoundsink->dsound_lock);
-  dsoundsink->system_clock = gst_system_clock_obtain ();
-  dsoundsink->write_wait_clock_id = NULL;
   dsoundsink->first_buffer_after_reset = FALSE;
 }
 
@@ -239,9 +229,6 @@ gst_directsound_sink_set_property (GObject * object,
     case PROP_MUTE:
       gst_directsound_sink_set_mute (sink, g_value_get_boolean (value));
       break;
-    case PROP_DEVICE:
-      gst_directsound_sink_set_device (sink, g_value_get_string (value));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -261,9 +248,6 @@ gst_directsound_sink_get_property (GObject * object,
     case PROP_MUTE:
       g_value_set_boolean (value, gst_directsound_sink_get_mute (sink));
       break;
-    case PROP_DEVICE:
-      g_value_set_string (value, gst_directsound_sink_get_device (sink));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -277,6 +261,7 @@ gst_directsound_sink_getcaps (GstBaseSink * bsink, GstCaps * filter)
   GstPadTemplate *pad_template;
   GstDirectSoundSink *dsoundsink = GST_DIRECTSOUND_SINK (bsink);
   GstCaps *caps;
+  gchar *caps_string = NULL;
 
   if (dsoundsink->pDS == NULL) {
     GST_DEBUG_OBJECT (dsoundsink, "device not open, using template caps");
@@ -284,23 +269,20 @@ gst_directsound_sink_getcaps (GstBaseSink * bsink, GstCaps * filter)
   }
 
   if (dsoundsink->cached_caps) {
-    caps = gst_caps_ref (dsoundsink->cached_caps);
-  } else {
-    element_class = GST_ELEMENT_GET_CLASS (dsoundsink);
-    pad_template = gst_element_class_get_pad_template (element_class, "sink");
-    g_return_val_if_fail (pad_template != NULL, NULL);
-
-    caps = gst_directsound_probe_supported_formats (dsoundsink,
-        gst_pad_template_get_caps (pad_template));
-    if (caps)
-      dsoundsink->cached_caps = gst_caps_ref (caps);
+    caps_string = gst_caps_to_string (dsoundsink->cached_caps);
+    GST_DEBUG_OBJECT (dsoundsink, "Returning cached caps: %s", caps_string);
+    g_free (caps_string);
+    return gst_caps_ref (dsoundsink->cached_caps);
   }
 
-  if (caps && filter) {
-    GstCaps *tmp =
-        gst_caps_intersect_full (filter, caps, GST_CAPS_INTERSECT_FIRST);
-    gst_caps_unref (caps);
-    caps = tmp;
+  element_class = GST_ELEMENT_GET_CLASS (dsoundsink);
+  pad_template = gst_element_class_get_pad_template (element_class, "sink");
+  g_return_val_if_fail (pad_template != NULL, NULL);
+
+  caps = gst_directsound_probe_supported_formats (dsoundsink,
+      gst_pad_template_get_caps (pad_template));
+  if (caps) {
+    dsoundsink->cached_caps = gst_caps_ref (caps);
   }
 
   if (caps) {
@@ -333,11 +315,10 @@ gst_directsound_sink_acceptcaps (GstBaseSink * sink, GstQuery * query)
 
   pad_caps = gst_pad_query_caps (pad, NULL);
   if (pad_caps) {
-    gboolean cret = gst_caps_is_subset (caps, pad_caps);
+    gboolean cret = gst_caps_can_intersect (pad_caps, caps);
     gst_caps_unref (pad_caps);
     if (!cret) {
-      GST_DEBUG_OBJECT (dsink,
-          "Caps are not a subset of the pad caps, not accepting caps");
+      GST_DEBUG_OBJECT (dsink, "Can't intersect caps, not accepting caps");
       goto done;
     }
   }
@@ -397,64 +378,27 @@ gst_directsound_sink_query (GstBaseSink * sink, GstQuery * query)
   return res;
 }
 
-static LPGUID
-string_to_guid (const gchar * str)
-{
-  HRESULT ret;
-  gunichar2 *wstr;
-  LPGUID out;
-
-  wstr = g_utf8_to_utf16 (str, -1, NULL, NULL, NULL);
-  if (!wstr)
-    return NULL;
-
-  out = g_new (GUID, 1);
-  ret = CLSIDFromString ((LPOLESTR) wstr, out);
-  g_free (wstr);
-  if (ret != NOERROR) {
-    g_free (out);
-    return NULL;
-  }
-
-  return out;
-}
-
 static gboolean
 gst_directsound_sink_open (GstAudioSink * asink)
 {
   GstDirectSoundSink *dsoundsink;
   HRESULT hRes;
-  LPGUID lpGuid = NULL;
 
   dsoundsink = GST_DIRECTSOUND_SINK (asink);
 
-  if (dsoundsink->device_id) {
-    lpGuid = string_to_guid (dsoundsink->device_id);
-    if (lpGuid == NULL) {
-      GST_ELEMENT_ERROR (dsoundsink, RESOURCE, OPEN_READ,
-          ("device set but guid not found: %s", dsoundsink->device_id), (NULL));
-      return FALSE;
-    }
-  }
-
-  /* create and initialize a DirectSound object */
-  if (FAILED (hRes = DirectSoundCreate (lpGuid, &dsoundsink->pDS, NULL))) {
-    gchar *error_text = gst_hres_to_string (hRes);
+  /* create and initialize a DirecSound object */
+  if (FAILED (hRes = DirectSoundCreate (NULL, &dsoundsink->pDS, NULL))) {
     GST_ELEMENT_ERROR (dsoundsink, RESOURCE, OPEN_READ,
-        ("DirectSoundCreate: %s", error_text), (NULL));
-    g_free (lpGuid);
-    g_free (error_text);
+        ("gst_directsound_sink_open: DirectSoundCreate: %s",
+            DXGetErrorString9 (hRes)), (NULL));
     return FALSE;
   }
 
-  g_free (lpGuid);
-
   if (FAILED (hRes = IDirectSound_SetCooperativeLevel (dsoundsink->pDS,
               GetDesktopWindow (), DSSCL_PRIORITY))) {
-    gchar *error_text = gst_hres_to_string (hRes);
     GST_ELEMENT_ERROR (dsoundsink, RESOURCE, OPEN_READ,
-        ("IDirectSound_SetCooperativeLevel: %s", error_text), (NULL));
-    g_free (error_text);
+        ("gst_directsound_sink_open: IDirectSound_SetCooperativeLevel: %s",
+            DXGetErrorString9 (hRes)), (NULL));
     return FALSE;
   }
 
@@ -500,12 +444,12 @@ gst_directsound_sink_prepare (GstAudioSink * asink,
         gst_util_uint64_scale_int (wfx.nAvgBytesPerSec, spec->buffer_time,
         GST_MSECOND);
     /* Make sure we make those numbers multiple of our sample size in bytes */
-    dsoundsink->buffer_size -= dsoundsink->buffer_size % spec->info.bpf;
+    dsoundsink->buffer_size += dsoundsink->buffer_size % spec->info.bpf;
 
     spec->segsize =
         gst_util_uint64_scale_int (wfx.nAvgBytesPerSec, spec->latency_time,
         GST_MSECOND);
-    spec->segsize -= spec->segsize % spec->info.bpf;
+    spec->segsize += spec->segsize % spec->info.bpf;
     spec->segtotal = dsoundsink->buffer_size / spec->segsize;
   } else {
 #ifdef WAVE_FORMAT_DOLBY_AC3_SPDIF
@@ -527,12 +471,11 @@ gst_directsound_sink_prepare (GstAudioSink * asink,
   // Make the final buffer size be an integer number of segments
   dsoundsink->buffer_size = spec->segsize * spec->segtotal;
 
-  GST_INFO_OBJECT (dsoundsink, "channels: %d, rate: %d, bytes_per_sample: %d"
-      " WAVEFORMATEX.nSamplesPerSec: %ld, WAVEFORMATEX.wBitsPerSample: %d,"
-      " WAVEFORMATEX.nBlockAlign: %d, WAVEFORMATEX.nAvgBytesPerSec: %ld\n"
-      "Size of dsound circular buffer=>%d\n",
-      GST_AUDIO_INFO_CHANNELS (&spec->info), GST_AUDIO_INFO_RATE (&spec->info),
-      GST_AUDIO_INFO_BPF (&spec->info), wfx.nSamplesPerSec, wfx.wBitsPerSample,
+  GST_INFO_OBJECT (dsoundsink,
+      "GstAudioRingBufferSpec->channels: %d, GstAudioRingBufferSpec->rate: %d, GstAudioRingBufferSpec->bytes_per_sample: %d\n"
+      "WAVEFORMATEX.nSamplesPerSec: %ld, WAVEFORMATEX.wBitsPerSample: %d, WAVEFORMATEX.nBlockAlign: %d, WAVEFORMATEX.nAvgBytesPerSec: %ld\n"
+      "Size of dsound circular buffer=>%d\n", spec->info.channels,
+      spec->info.rate, spec->info.bpf, wfx.nSamplesPerSec, wfx.wBitsPerSample,
       wfx.nBlockAlign, wfx.nAvgBytesPerSec, dsoundsink->buffer_size);
 
   /* create a secondary directsound buffer */
@@ -548,16 +491,13 @@ gst_directsound_sink_prepare (GstAudioSink * asink,
   hRes = IDirectSound_CreateSoundBuffer (dsoundsink->pDS, &descSecondary,
       &dsoundsink->pDSBSecondary, NULL);
   if (FAILED (hRes)) {
-    gchar *error_text = gst_hres_to_string (hRes);
     GST_ELEMENT_ERROR (dsoundsink, RESOURCE, OPEN_READ,
-        ("IDirectSound_CreateSoundBuffer: %s", error_text), (NULL));
-    g_free (error_text);
+        ("gst_directsound_sink_prepare: IDirectSound_CreateSoundBuffer: %s",
+            DXGetErrorString9 (hRes)), (NULL));
     return FALSE;
   }
 
-  gst_directsound_sink_set_volume (dsoundsink,
-      gst_directsound_sink_get_volume (dsoundsink), FALSE);
-  gst_directsound_sink_set_mute (dsoundsink, dsoundsink->mute);
+  gst_directsound_sink_set_volume (dsoundsink, dsoundsink->volume, FALSE);
 
   return TRUE;
 }
@@ -599,8 +539,8 @@ static gint
 gst_directsound_sink_write (GstAudioSink * asink, gpointer data, guint length)
 {
   GstDirectSoundSink *dsoundsink;
-  DWORD dwStatus = 0;
-  HRESULT hRes, hRes2;
+  DWORD dwStatus;
+  HRESULT hRes;
   LPVOID pLockedBuffer1 = NULL, pLockedBuffer2 = NULL;
   DWORD dwSizeBuffer1, dwSizeBuffer2;
   DWORD dwCurrentPlayCursor;
@@ -613,16 +553,14 @@ gst_directsound_sink_write (GstAudioSink * asink, gpointer data, guint length)
   hRes = IDirectSoundBuffer_GetStatus (dsoundsink->pDSBSecondary, &dwStatus);
 
   /* get current play cursor position */
-  hRes2 = IDirectSoundBuffer_GetCurrentPosition (dsoundsink->pDSBSecondary,
+  hRes = IDirectSoundBuffer_GetCurrentPosition (dsoundsink->pDSBSecondary,
       &dwCurrentPlayCursor, NULL);
 
-  if (SUCCEEDED (hRes) && SUCCEEDED (hRes2) && (dwStatus & DSBSTATUS_PLAYING)) {
-    DWORD dwFreeBufferSize = 0;
-    GstClockTime sleep_time_ms = 0, sleep_until;
-    GstClockID clock_id;
+  if (SUCCEEDED (hRes) && (dwStatus & DSBSTATUS_PLAYING)) {
+    DWORD dwFreeBufferSize;
 
   calculate_freesize:
-    /* Calculate the free space in the circular buffer */
+    /* calculate the free size of the circular buffer */
     if (dwCurrentPlayCursor < dsoundsink->current_circular_offset)
       dwFreeBufferSize =
           dsoundsink->buffer_size - (dsoundsink->current_circular_offset -
@@ -631,91 +569,29 @@ gst_directsound_sink_write (GstAudioSink * asink, gpointer data, guint length)
       dwFreeBufferSize =
           dwCurrentPlayCursor - dsoundsink->current_circular_offset;
 
-    /* Not enough free space, wait for some samples to be played out. We could
-     * write out partial data, but that will result in a tight loop in the
-     * audioringbuffer write thread, and lead to high CPU usage. */
-    if (length > dwFreeBufferSize) {
-      gint rate = GST_AUDIO_BASE_SINK (asink)->ringbuffer->spec.info.rate;
-      /* Wait for a time proportional to the space needed. In reality, the
-       * directsound sink's position does not update frequently enough, so we
-       * will end up waiting for much longer. Note that Sleep() has millisecond
-       * resolution at best. */
-      sleep_time_ms = gst_util_uint64_scale_int ((length - dwFreeBufferSize),
-          1000, dsoundsink->bytes_per_sample * rate);
-      /* Make sure we don't run in a tight loop unnecessarily */
-      sleep_time_ms = MAX (sleep_time_ms, 10);
-      sleep_until = gst_clock_get_time (dsoundsink->system_clock) +
-          sleep_time_ms * GST_MSECOND;
-
-      GST_DEBUG_OBJECT (dsoundsink,
-          "length: %u, FreeBufSiz: %ld, sleep_time_ms: %" G_GUINT64_FORMAT
-          ", bps: %i, rate: %i", length, dwFreeBufferSize, sleep_time_ms,
-          dsoundsink->bytes_per_sample, rate);
-
-      if (G_UNLIKELY (dsoundsink->write_wait_clock_id == NULL ||
-              gst_clock_single_shot_id_reinit (dsoundsink->system_clock,
-                  dsoundsink->write_wait_clock_id, sleep_until) == FALSE)) {
-
-        if (dsoundsink->write_wait_clock_id != NULL) {
-          gst_clock_id_unref (dsoundsink->write_wait_clock_id);
-        }
-
-        dsoundsink->write_wait_clock_id =
-            gst_clock_new_single_shot_id (dsoundsink->system_clock,
-            sleep_until);
-      }
-
-      clock_id = dsoundsink->write_wait_clock_id;
-      dsoundsink->reset_while_sleeping = FALSE;
-
-      GST_DSOUND_UNLOCK (dsoundsink);
-
-      /* don't bother with the return value as we'll detect reset separately,
-         as reset could happen between when this returns and we obtain the lock
-         again -- so we can't use UNSCHEDULED here */
-      gst_clock_id_wait (clock_id, NULL);
-
-      GST_DSOUND_LOCK (dsoundsink);
-
-      /* if a reset occurs, exit now */
-      if (dsoundsink->reset_while_sleeping == TRUE) {
-        GST_DSOUND_UNLOCK (dsoundsink);
-        return -1;
-      }
-
-      /* May we send out? */
+    if (length >= dwFreeBufferSize) {
+      Sleep (100);
       hRes = IDirectSoundBuffer_GetCurrentPosition (dsoundsink->pDSBSecondary,
           &dwCurrentPlayCursor, NULL);
-      hRes2 =
+
+      hRes =
           IDirectSoundBuffer_GetStatus (dsoundsink->pDSBSecondary, &dwStatus);
-      if (SUCCEEDED (hRes) && SUCCEEDED (hRes2)
-          && (dwStatus & DSBSTATUS_PLAYING))
+      if (SUCCEEDED (hRes) && (dwStatus & DSBSTATUS_PLAYING))
         goto calculate_freesize;
       else {
-        gchar *err1, *err2;
-
         dsoundsink->first_buffer_after_reset = FALSE;
         GST_DSOUND_UNLOCK (dsoundsink);
-
-        err1 = gst_hres_to_string (hRes);
-        err2 = gst_hres_to_string (hRes2);
-        GST_ELEMENT_ERROR (dsoundsink, RESOURCE, OPEN_WRITE,
-            ("IDirectSoundBuffer_GetStatus %s, "
-                "IDirectSoundBuffer_GetCurrentPosition: %s, dwStatus: %lu",
-                err2, err1, dwStatus), (NULL));
-        g_free (err1);
-        g_free (err2);
-        return -1;
+        return 0;
       }
     }
   }
 
   if (dwStatus & DSBSTATUS_BUFFERLOST) {
     hRes = IDirectSoundBuffer_Restore (dsoundsink->pDSBSecondary);      /*need a loop waiting the buffer is restored?? */
+
     dsoundsink->current_circular_offset = 0;
   }
 
-  /* Lock a buffer of length @length for writing */
   hRes = IDirectSoundBuffer_Lock (dsoundsink->pDSBSecondary,
       dsoundsink->current_circular_offset, length, &pLockedBuffer1,
       &dwSizeBuffer1, &pLockedBuffer2, &dwSizeBuffer2, 0L);
@@ -726,12 +602,12 @@ gst_directsound_sink_write (GstAudioSink * asink, gpointer data, guint length)
     if (pLockedBuffer2 != NULL)
       memcpy (pLockedBuffer2, (LPBYTE) data + dwSizeBuffer1, dwSizeBuffer2);
 
-    hRes = IDirectSoundBuffer_Unlock (dsoundsink->pDSBSecondary, pLockedBuffer1,
-        dwSizeBuffer1, pLockedBuffer2, dwSizeBuffer2);
-
     // Update where the buffer will lock (for next time)
     dsoundsink->current_circular_offset += dwSizeBuffer1 + dwSizeBuffer2;
     dsoundsink->current_circular_offset %= dsoundsink->buffer_size;     /* Circular buffer */
+
+    hRes = IDirectSoundBuffer_Unlock (dsoundsink->pDSBSecondary, pLockedBuffer1,
+        dwSizeBuffer1, pLockedBuffer2, dwSizeBuffer2);
   }
 
   /* if the buffer was not in playing state yet, call play on the buffer 
@@ -764,7 +640,7 @@ gst_directsound_sink_delay (GstAudioSink * asink)
   /* get current buffer status */
   hRes = IDirectSoundBuffer_GetStatus (dsoundsink->pDSBSecondary, &dwStatus);
 
-  if (SUCCEEDED (hRes) && (dwStatus & DSBSTATUS_PLAYING)) {
+  if (dwStatus & DSBSTATUS_PLAYING) {
     /*evaluate the number of samples in queue in the circular buffer */
     hRes = IDirectSoundBuffer_GetCurrentPosition (dsoundsink->pDSBSecondary,
         &dwCurrentPlayCursor, NULL);
@@ -806,7 +682,7 @@ gst_directsound_sink_reset (GstAudioSink * asink)
 
     /*reset the buffer */
     hRes = IDirectSoundBuffer_Lock (dsoundsink->pDSBSecondary,
-        0, dsoundsink->buffer_size,
+        dsoundsink->current_circular_offset, dsoundsink->buffer_size,
         &pLockedBuffer, &dwSizeBuffer, NULL, NULL, 0L);
 
     if (SUCCEEDED (hRes)) {
@@ -818,11 +694,7 @@ gst_directsound_sink_reset (GstAudioSink * asink)
     }
   }
 
-  dsoundsink->reset_while_sleeping = TRUE;
   dsoundsink->first_buffer_after_reset = TRUE;
-  if (dsoundsink->write_wait_clock_id != NULL) {
-    gst_clock_id_unschedule (dsoundsink->write_wait_clock_id);
-  }
 
   GST_DSOUND_UNLOCK (dsoundsink);
 }
@@ -873,10 +745,9 @@ gst_directsound_probe_supported_formats (GstDirectSoundSink * dsoundsink,
   hRes = IDirectSound_CreateSoundBuffer (dsoundsink->pDS, &descSecondary,
       &tmpBuffer, NULL);
   if (FAILED (hRes)) {
-    gchar *error_text = gst_hres_to_string (hRes);
     GST_INFO_OBJECT (dsoundsink, "AC3 passthrough not supported "
-        "(IDirectSound_CreateSoundBuffer returned: %s)\n", error_text);
-    g_free (error_text);
+        "(IDirectSound_CreateSoundBuffer returned: %s)\n",
+        DXGetErrorString9 (hRes));
     tmp = gst_caps_new_empty_simple ("audio/x-ac3");
     tmp2 = gst_caps_subtract (caps, tmp);
     gst_caps_unref (tmp);
@@ -891,10 +762,9 @@ gst_directsound_probe_supported_formats (GstDirectSoundSink * dsoundsink,
     GST_INFO_OBJECT (dsoundsink, "AC3 passthrough supported");
     hRes = IDirectSoundBuffer_Release (tmpBuffer);
     if (FAILED (hRes)) {
-      gchar *error_text = gst_hres_to_string (hRes);
       GST_DEBUG_OBJECT (dsoundsink,
-          "(IDirectSoundBuffer_Release returned: %s)\n", error_text);
-      g_free (error_text);
+          "(IDirectSoundBuffer_Release returned: %s)\n",
+          DXGetErrorString9 (hRes));
     }
   }
 #else
@@ -971,15 +841,15 @@ gst_directsound_sink_set_volume (GstDirectSoundSink * dsoundsink,
      * here, so remap.
      */
     long dsVolume;
-    if (volume == 0 || dsoundsink->mute)
+    if (dsoundsink->volume == 0)
       dsVolume = -10000;
     else
-      dsVolume = 100 * (long) (20 * log10 ((double) volume / 100.));
+      dsVolume = 100 * (long) (20 * log10 ((double) dsoundsink->volume / 100.));
     dsVolume = CLAMP (dsVolume, -10000, 0);
 
     GST_DEBUG_OBJECT (dsoundsink,
         "Setting volume on secondary buffer to %d from %d", (int) dsVolume,
-        (int) volume);
+        (int) dsoundsink->volume);
     IDirectSoundBuffer_SetVolume (dsoundsink->pDSBSecondary, dsVolume);
   }
 }
@@ -993,58 +863,14 @@ gst_directsound_sink_get_volume (GstDirectSoundSink * dsoundsink)
 static void
 gst_directsound_sink_set_mute (GstDirectSoundSink * dsoundsink, gboolean mute)
 {
-  if (mute) {
+  if (mute)
     gst_directsound_sink_set_volume (dsoundsink, 0, FALSE);
-    dsoundsink->mute = TRUE;
-  } else {
-    gst_directsound_sink_set_volume (dsoundsink,
-        gst_directsound_sink_get_volume (dsoundsink), FALSE);
-    dsoundsink->mute = FALSE;
-  }
-
+  else
+    gst_directsound_sink_set_volume (dsoundsink, dsoundsink->volume, FALSE);
 }
 
 static gboolean
 gst_directsound_sink_get_mute (GstDirectSoundSink * dsoundsink)
 {
-  return dsoundsink->mute;
-}
-
-static const gchar *
-gst_directsound_sink_get_device (GstDirectSoundSink * dsoundsink)
-{
-  return dsoundsink->device_id;
-}
-
-static void
-gst_directsound_sink_set_device (GstDirectSoundSink * dsoundsink,
-    const gchar * device_id)
-{
-  g_free (dsoundsink->device_id);
-  dsoundsink->device_id = g_strdup (device_id);
-}
-
-/* Converts a HRESULT error to a text string
- * LPTSTR is either a */
-static gchar *
-gst_hres_to_string (HRESULT hRes)
-{
-  DWORD flags;
-  gchar *ret_text;
-  LPTSTR error_text = NULL;
-
-  flags = FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER
-      | FORMAT_MESSAGE_IGNORE_INSERTS;
-  FormatMessage (flags, NULL, hRes, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
-      (LPTSTR) & error_text, 0, NULL);
-
-#ifdef UNICODE
-  /* If UNICODE is defined, LPTSTR is LPWSTR which is UTF-16 */
-  ret_text = g_utf16_to_utf8 (error_text, 0, NULL, NULL, NULL);
-#else
-  ret_text = g_strdup (error_text);
-#endif
-
-  LocalFree (error_text);
-  return ret_text;
+  return FALSE;
 }
