@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * The development of this code was made possible due to the involvement of
  * Pioneers of the Inevitable, the creators of the Songbird Music player
@@ -30,22 +30,20 @@
 #include "gstosxaudioelement.h"
 #include "gstosxaudiosink.h"
 #include "gstosxaudiosrc.h"
-#ifndef HAVE_IOS
-#include "gstosxaudiodeviceprovider.h"
-#endif
 
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  gboolean ret = FALSE;
+  if (!gst_element_register (plugin, "osxaudiosink", GST_RANK_PRIMARY,
+          GST_TYPE_OSX_AUDIO_SINK)) {
+    return FALSE;
+  }
+  if (!gst_element_register (plugin, "osxaudiosrc", GST_RANK_PRIMARY,
+          GST_TYPE_OSX_AUDIO_SRC)) {
+    return FALSE;
+  }
 
-  ret |= GST_ELEMENT_REGISTER (osxaudiosrc, plugin);
-  ret |= GST_ELEMENT_REGISTER (osxaudiosink, plugin);
-#ifndef HAVE_IOS
-  ret |= GST_DEVICE_PROVIDER_REGISTER (osxaudiodeviceprovider, plugin);
-#endif
-
-  return ret;
+  return TRUE;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
