@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * The development of this code was made possible due to the involvement of Pioneers 
  * of the Inevitable, the creators of the Songbird Music player
@@ -27,6 +27,7 @@
  */
 
 #import <Cocoa/Cocoa.h>
+#import <QuickTime/QuickTime.h>
 #import <glib.h>
 #import <gst/video/navigation.h>
 
@@ -49,7 +50,6 @@ struct _GstOSXImage;
     GstNavigation *navigation;
     NSRect drawingBounds;
     NSThread *mainThread;
-    NSUInteger savedModifierFlags;
 }
 - (void) drawQuad;
 - (void) drawRect: (NSRect) rect;
@@ -62,16 +62,14 @@ struct _GstOSXImage;
 - (void) setFullScreen: (BOOL) flag;
 - (void) setKeepAspectRatio: (BOOL) flag;
 - (void) reshape;
-- (void) setVideoSize:(int)w : (int)h;
+- (void) setVideoSize: (int) w: (int) h;
 - (NSRect) getDrawingBounds;
 - (BOOL) haveSuperview;
 - (void) haveSuperviewReal: (NSMutableArray *)closure;
 - (void) addToSuperview: (NSView *)superview;
 - (void) removeFromSuperview: (id)unused;
 - (void) setNavigation: (GstNavigation *) nav;
-#ifndef GSTREAMER_GLIB_COCOA_NSAPPLICATION
 - (void) setMainThread: (NSThread *) thread;
-#endif
 
 @end
 
