@@ -1,7 +1,7 @@
 /* GStreamer
  *
  * Copyright (C) 2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
- *               2006 Edgard Lima <edgard.lima@gmail.com>
+ *               2006 Edgard Lima <edgard.lima@indt.org.br>
  *
  * v4l2_calls.c - generic V4L2 calls handling
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,15 +38,25 @@
 #include <stropts.h>
 #include <sys/ioccom.h>
 #endif
-#include "gstv4l2object.h"
+#include "v4l2_calls.h"
 #include "gstv4l2tuner.h"
+#if 0
+#include "gstv4l2xoverlay.h"
+#endif
 #include "gstv4l2colorbalance.h"
 
 #include "gstv4l2src.h"
 #include "gstv4l2sink.h"
-#include "gstv4l2videodec.h"
 
-#include <glib/gi18n-lib.h>
+#include "gst/gst-i18n-plugin.h"
+
+/* Those are ioctl calls */
+#ifndef V4L2_CID_HCENTER
+#define V4L2_CID_HCENTER V4L2_CID_HCENTER_DEPRECATED
+#endif
+#ifndef V4L2_CID_VCENTER
+#define V4L2_CID_VCENTER V4L2_CID_VCENTER_DEPRECATED
+#endif
 
 GST_DEBUG_CATEGORY_EXTERN (v4l2_debug);
 #define GST_CAT_DEFAULT v4l2_debug
@@ -56,7 +66,7 @@ GST_DEBUG_CATEGORY_EXTERN (v4l2_debug);
  *   get the device's capturing capabilities
  * return value: TRUE on success, FALSE on error
  ******************************************************/
-static gboolean
+gboolean
 gst_v4l2_get_capabilities (GstV4l2Object * v4l2object)
 {
   GstElement *e;
@@ -68,20 +78,14 @@ gst_v4l2_get_capabilities (GstV4l2Object * v4l2object)
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_QUERYCAP,
-          &v4l2object->vcap) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_QUERYCAP, &v4l2object->vcap) < 0)
     goto cap_failed;
 
-  if (v4l2object->vcap.capabilities & V4L2_CAP_DEVICE_CAPS)
-    v4l2object->device_caps = v4l2object->vcap.device_caps;
-  else
-    v4l2object->device_caps = v4l2object->vcap.capabilities;
-
   GST_LOG_OBJECT (e, "driver:      '%s'", v4l2object->vcap.driver);
   GST_LOG_OBJECT (e, "card:        '%s'", v4l2object->vcap.card);
   GST_LOG_OBJECT (e, "bus_info:    '%s'", v4l2object->vcap.bus_info);
   GST_LOG_OBJECT (e, "version:     %08x", v4l2object->vcap.version);
-  GST_LOG_OBJECT (e, "capabilities: %08x", v4l2object->device_caps);
+  GST_LOG_OBJECT (e, "capabilites: %08x", v4l2object->vcap.capabilities);
 
   return TRUE;
 
@@ -96,29 +100,6 @@ cap_failed:
   }
 }
 
-/******************************************************
- * The video4linux command line tool v4l2-ctrl
- * normalises the names of the controls received from
- * the kernel like:
- *
- *     "Exposure (absolute)" -> "exposure_absolute"
- *
- * We follow their lead here.  @name is modified
- * in-place.
- ******************************************************/
-static void
-gst_v4l2_normalise_control_name (gchar * name)
-{
-  int i, j;
-  for (i = 0, j = 0; name[j]; ++j) {
-    if (g_ascii_isalnum (name[j])) {
-      if (i > 0 && !g_ascii_isalnum (name[j - 1]))
-        name[i++] = '_';
-      name[i++] = g_ascii_tolower (name[j]);
-    }
-  }
-  name[i++] = '\0';
-}
 
 /******************************************************
  * gst_v4l2_empty_lists() and gst_v4l2_fill_lists():
@@ -128,8 +109,7 @@ gst_v4l2_normalise_control_name (gchar * name)
 static gboolean
 gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
 {
-  gint n, next;
-  struct v4l2_queryctrl control = { 0, };
+  gint n;
 
   GstElement *e;
 
@@ -148,7 +128,7 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
     memset (&input, 0, sizeof (input));
 
     input.index = n;
-    if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_ENUMINPUT, &input) < 0) {
+    if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_ENUMINPUT, &input) < 0) {
       if (errno == EINVAL || errno == ENOTTY)
         break;                  /* end of enumeration */
       else {
@@ -165,8 +145,7 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
     GST_LOG_OBJECT (e, "   name:      '%s'", input.name);
     GST_LOG_OBJECT (e, "   type:      %08x", input.type);
     GST_LOG_OBJECT (e, "   audioset:  %08x", input.audioset);
-    GST_LOG_OBJECT (e, "   std:       %016" G_GINT64_MODIFIER "x",
-        (guint64) input.std);
+    GST_LOG_OBJECT (e, "   std:       %016" G_GINT64_MODIFIER "x", input.std);
     GST_LOG_OBJECT (e, "   status:    %08x", input.status);
 
     v4l2channel = g_object_new (GST_TYPE_V4L2_TUNER_CHANNEL, NULL);
@@ -182,7 +161,7 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
       channel->flags |= GST_TUNER_CHANNEL_FREQUENCY;
 
       vtun.index = input.tuner;
-      if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_TUNER, &vtun) < 0) {
+      if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_TUNER, &vtun) < 0) {
         GST_ELEMENT_ERROR (e, RESOURCE, SETTINGS,
             (_("Failed to get setting of tuner %d on device '%s'."),
                 input.tuner, v4l2object->videodev), GST_ERROR_SYSTEM);
@@ -223,18 +202,14 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
     standard.frameperiod.denominator = 0;
     standard.index = n;
 
-    if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_ENUMSTD, &standard) < 0) {
+    if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_ENUMSTD, &standard) < 0) {
       if (errno == EINVAL || errno == ENOTTY)
         break;                  /* end of enumeration */
-#ifdef ENODATA
-      else if (errno == ENODATA)
-        break;                  /* end of enumeration, as of Linux 3.7-rc1 */
-#endif
       else {
         GST_ELEMENT_ERROR (e, RESOURCE, SETTINGS,
             (_("Failed to query norm on device '%s'."),
                 v4l2object->videodev),
-            ("Failed to get attributes for norm %d on divide '%s'. (%d - %s)",
+            ("Failed to get attributes for norm %d on devide '%s'. (%d - %s)",
                 n, v4l2object->videodev, errno, strerror (errno)));
         return FALSE;
       }
@@ -251,7 +226,7 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
         standard.frameperiod.denominator, standard.frameperiod.numerator);
     v4l2norm->index = standard.id;
 
-    GST_DEBUG_OBJECT (v4l2object->dbg_obj, "index=%08x, label=%s",
+    GST_DEBUG_OBJECT (v4l2object->element, "index=%08x, label=%s",
         (unsigned int) v4l2norm->index, norm->label);
 
     v4l2object->norms = g_list_prepend (v4l2object->norms, (gpointer) norm);
@@ -261,18 +236,11 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
   GST_DEBUG_OBJECT (e, "  controls+menus");
 
   /* and lastly, controls+menus (if appropriate) */
-  next = V4L2_CTRL_FLAG_NEXT_CTRL;
-  n = 0;
-  control.id = next;
-
-  while (TRUE) {
+  for (n = V4L2_CID_BASE;; n++) {
+    struct v4l2_queryctrl control = { 0, };
     GstV4l2ColorBalanceChannel *v4l2channel;
     GstColorBalanceChannel *channel;
 
-    if (!next)
-      n++;
-
-  retry:
     /* when we reached the last official CID, continue with private CIDs */
     if (n == V4L2_CID_LASTP1) {
       GST_DEBUG_OBJECT (e, "checking private CIDs");
@@ -280,25 +248,12 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
     }
     GST_DEBUG_OBJECT (e, "checking control %08x", n);
 
-    control.id = n | next;
-    if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_QUERYCTRL,
-            &control) < 0) {
-      if (next) {
-        if (n > 0) {
-          GST_DEBUG_OBJECT (e, "controls finished");
-          break;
-        } else {
-          GST_DEBUG_OBJECT (e, "V4L2_CTRL_FLAG_NEXT_CTRL not supported.");
-          next = 0;
-          n = V4L2_CID_BASE;
-          goto retry;
-        }
-      }
+    control.id = n;
+    if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_QUERYCTRL, &control) < 0) {
       if (errno == EINVAL || errno == ENOTTY || errno == EIO || errno == ENOENT) {
         if (n < V4L2_CID_PRIVATE_BASE) {
           GST_DEBUG_OBJECT (e, "skipping control %08x", n);
           /* continue so that we also check private controls */
-          n = V4L2_CID_PRIVATE_BASE - 1;
           continue;
         } else {
           GST_DEBUG_OBJECT (e, "controls finished");
@@ -307,49 +262,14 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
       } else {
         GST_WARNING_OBJECT (e, "Failed querying control %d on device '%s'. "
             "(%d - %s)", n, v4l2object->videodev, errno, strerror (errno));
-        if (n > (V4L2_CID_PRIVATE_BASE + V4L2_CID_MAX_CTRLS)) {
-          GST_DEBUG_OBJECT (e, "Finish control by reaching V4L2_CID_MAX_CTRLS");
-          break;
-        }
         continue;
       }
     }
-    /* bogus driver might mess with id in unexpected ways (e.g. set to 0), so
-     * make sure to simply try all if V4L2_CTRL_FLAG_NEXT_CTRL not supported */
-    if (next)
-      n = control.id;
     if (control.flags & V4L2_CTRL_FLAG_DISABLED) {
       GST_DEBUG_OBJECT (e, "skipping disabled control");
       continue;
     }
 
-    if (control.type == V4L2_CTRL_TYPE_CTRL_CLASS) {
-      GST_DEBUG_OBJECT (e, "starting control class '%s'", control.name);
-      continue;
-    }
-
-    switch (control.type) {
-      case V4L2_CTRL_TYPE_INTEGER:
-      case V4L2_CTRL_TYPE_BOOLEAN:
-      case V4L2_CTRL_TYPE_MENU:
-      case V4L2_CTRL_TYPE_INTEGER_MENU:
-      case V4L2_CTRL_TYPE_BITMASK:
-      case V4L2_CTRL_TYPE_BUTTON:
-      case V4L2_CTRL_TYPE_STRING:{
-        control.name[31] = '\0';
-        gst_v4l2_normalise_control_name ((gchar *) control.name);
-        g_datalist_id_set_data (&v4l2object->controls,
-            g_quark_from_string ((const gchar *) control.name),
-            GINT_TO_POINTER (n));
-        break;
-      }
-      default:
-        GST_DEBUG_OBJECT (e,
-            "Control type for '%s' not supported for extra controls.",
-            control.name);
-        break;
-    }
-
     switch (n) {
       case V4L2_CID_BRIGHTNESS:
       case V4L2_CID_CONTRAST:
@@ -364,13 +284,21 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
       case V4L2_CID_EXPOSURE:
       case V4L2_CID_AUTOGAIN:
       case V4L2_CID_GAIN:
+#ifdef V4L2_CID_SHARPNESS
       case V4L2_CID_SHARPNESS:
+#endif
         /* we only handle these for now (why?) */
         break;
       case V4L2_CID_HFLIP:
       case V4L2_CID_VFLIP:
+      case V4L2_CID_HCENTER:
+      case V4L2_CID_VCENTER:
+#ifdef V4L2_CID_PAN_RESET
       case V4L2_CID_PAN_RESET:
+#endif
+#ifdef V4L2_CID_TILT_RESET
       case V4L2_CID_TILT_RESET:
+#endif
         /* not handled here, handled by VideoOrientation interface */
         control.id++;
         break;
@@ -380,12 +308,8 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
       case V4L2_CID_AUDIO_TREBLE:
       case V4L2_CID_AUDIO_MUTE:
       case V4L2_CID_AUDIO_LOUDNESS:
-        /* FIXME: We should implement GstMixer interface instead */
-        /* but let's not be pedantic and make element more useful for now */
-        break;
-      case V4L2_CID_ALPHA_COMPONENT:
-        v4l2object->has_alpha_component = TRUE;
-        break;
+        /* FIXME: We should implement GstMixer interface */
+        /* fall through */
       default:
         GST_DEBUG_OBJECT (e,
             "ControlID %s (%x) unhandled, FIXME", control.name, n);
@@ -412,8 +336,7 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
       menu.id = n;
       for (i = 0;; i++) {
         menu.index = i;
-        if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_QUERYMENU,
-                &menu) < 0) {
+        if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_QUERYMENU, &menu) < 0) {
           if (errno == EINVAL)
             break;              /* end of enumeration */
           else {
@@ -466,7 +389,7 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
 static void
 gst_v4l2_empty_lists (GstV4l2Object * v4l2object)
 {
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "deleting enumerations");
+  GST_DEBUG_OBJECT (v4l2object->element, "deleting enumerations");
 
   g_list_foreach (v4l2object->channels, (GFunc) g_object_unref, NULL);
   g_list_free (v4l2object->channels);
@@ -479,39 +402,6 @@ gst_v4l2_empty_lists (GstV4l2Object * v4l2object)
   g_list_foreach (v4l2object->colors, (GFunc) g_object_unref, NULL);
   g_list_free (v4l2object->colors);
   v4l2object->colors = NULL;
-
-  g_datalist_clear (&v4l2object->controls);
-}
-
-static void
-gst_v4l2_adjust_buf_type (GstV4l2Object * v4l2object)
-{
-  /* when calling gst_v4l2_object_new the user decides the initial type
-   * so adjust it if multi-planar is supported
-   * the driver should make it exclusive. So the driver should
-   * not support both MPLANE and non-PLANE.
-   * Because even when using MPLANE it still possibles to use it
-   * in a contiguous manner. In this case the first v4l2 plane
-   * contains all the gst planes.
-   */
-  switch (v4l2object->type) {
-    case V4L2_BUF_TYPE_VIDEO_OUTPUT:
-      if (v4l2object->device_caps &
-          (V4L2_CAP_VIDEO_OUTPUT_MPLANE | V4L2_CAP_VIDEO_M2M_MPLANE)) {
-        GST_DEBUG ("adjust type to multi-planar output");
-        v4l2object->type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
-      }
-      break;
-    case V4L2_BUF_TYPE_VIDEO_CAPTURE:
-      if (v4l2object->device_caps &
-          (V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_VIDEO_M2M_MPLANE)) {
-        GST_DEBUG ("adjust type to multi-planar capture");
-        v4l2object->type = V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE;
-      }
-      break;
-    default:
-      break;
-  }
 }
 
 /******************************************************
@@ -520,12 +410,13 @@ gst_v4l2_adjust_buf_type (GstV4l2Object * v4l2object)
  * return value: TRUE on success, FALSE on error
  ******************************************************/
 gboolean
-gst_v4l2_open (GstV4l2Object * v4l2object, GstV4l2Error * error)
+gst_v4l2_open (GstV4l2Object * v4l2object)
 {
   struct stat st;
-  int libv4l2_fd = -1;
+  int libv4l2_fd;
+  GstPollFD pollfd = GST_POLL_FD_INIT;
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "Trying to open device %s",
+  GST_DEBUG_OBJECT (v4l2object->element, "Trying to open device %s",
       v4l2object->videodev);
 
   GST_V4L2_CHECK_NOT_OPEN (v4l2object);
@@ -549,12 +440,8 @@ gst_v4l2_open (GstV4l2Object * v4l2object, GstV4l2Error * error)
   if (!GST_V4L2_IS_OPEN (v4l2object))
     goto not_open;
 
-#ifdef HAVE_LIBV4L2
-  if (v4l2object->fd_open)
-    libv4l2_fd = v4l2object->fd_open (v4l2object->video_fd,
-        V4L2_ENABLE_ENUM_FMT_EMULATION);
-#endif
-
+  libv4l2_fd = v4l2_fd_open (v4l2object->video_fd,
+      V4L2_ENABLE_ENUM_FMT_EMULATION);
   /* Note the v4l2_xxx functions are designed so that if they get passed an
      unknown fd, the will behave exactly as their regular xxx counterparts, so
      if v4l2_fd_open fails, we continue as normal (missing the libv4l2 custom
@@ -564,96 +451,78 @@ gst_v4l2_open (GstV4l2Object * v4l2object, GstV4l2Error * error)
   if (libv4l2_fd != -1)
     v4l2object->video_fd = libv4l2_fd;
 
+  v4l2object->can_poll_device = TRUE;
+
   /* get capabilities, error will be posted */
   if (!gst_v4l2_get_capabilities (v4l2object))
     goto error;
 
   /* do we need to be a capture device? */
   if (GST_IS_V4L2SRC (v4l2object->element) &&
-      !(v4l2object->device_caps & (V4L2_CAP_VIDEO_CAPTURE |
-              V4L2_CAP_VIDEO_CAPTURE_MPLANE)))
+      !(v4l2object->vcap.capabilities & V4L2_CAP_VIDEO_CAPTURE))
     goto not_capture;
 
   if (GST_IS_V4L2SINK (v4l2object->element) &&
-      !(v4l2object->device_caps & (V4L2_CAP_VIDEO_OUTPUT |
-              V4L2_CAP_VIDEO_OUTPUT_MPLANE)))
+      !(v4l2object->vcap.capabilities & V4L2_CAP_VIDEO_OUTPUT))
     goto not_output;
 
-  if (GST_IS_V4L2_VIDEO_DEC (v4l2object->element) &&
-      !GST_V4L2_IS_M2M (v4l2object->device_caps))
-    goto not_m2m;
-
-  gst_v4l2_adjust_buf_type (v4l2object);
-
   /* create enumerations, posts errors. */
   if (!gst_v4l2_fill_lists (v4l2object))
     goto error;
 
-  GST_INFO_OBJECT (v4l2object->dbg_obj,
+  GST_INFO_OBJECT (v4l2object->element,
       "Opened device '%s' (%s) successfully",
       v4l2object->vcap.card, v4l2object->videodev);
 
-  if (v4l2object->extra_controls)
-    gst_v4l2_set_controls (v4l2object, v4l2object->extra_controls);
-
-  /* UVC devices are never interlaced, and doing VIDIOC_TRY_FMT on them
-   * causes expensive and slow USB IO, so don't probe them for interlaced
-   */
-  if (!strcmp ((char *) v4l2object->vcap.driver, "uvcusb") ||
-      !strcmp ((char *) v4l2object->vcap.driver, "uvcvideo")) {
-    v4l2object->never_interlaced = TRUE;
-  }
+  pollfd.fd = v4l2object->video_fd;
+  gst_poll_add_fd (v4l2object->poll, &pollfd);
+  gst_poll_fd_ctl_read (v4l2object->poll, &pollfd, TRUE);
 
   return TRUE;
 
   /* ERRORS */
 stat_failed:
   {
-    GST_V4L2_ERROR (error, RESOURCE, NOT_FOUND,
+    GST_ELEMENT_ERROR (v4l2object->element, RESOURCE, NOT_FOUND,
         (_("Cannot identify device '%s'."), v4l2object->videodev),
         GST_ERROR_SYSTEM);
     goto error;
   }
 no_device:
   {
-    GST_V4L2_ERROR (error, RESOURCE, NOT_FOUND,
+    GST_ELEMENT_ERROR (v4l2object->element, RESOURCE, NOT_FOUND,
         (_("This isn't a device '%s'."), v4l2object->videodev),
         GST_ERROR_SYSTEM);
     goto error;
   }
 not_open:
   {
-    GST_V4L2_ERROR (error, RESOURCE, OPEN_READ_WRITE,
+    GST_ELEMENT_ERROR (v4l2object->element, RESOURCE, OPEN_READ_WRITE,
         (_("Could not open device '%s' for reading and writing."),
             v4l2object->videodev), GST_ERROR_SYSTEM);
     goto error;
   }
 not_capture:
   {
-    GST_V4L2_ERROR (error, RESOURCE, NOT_FOUND,
-        (_("Device '%s' is not a capture device."), v4l2object->videodev),
-        ("Capabilities: 0x%x", v4l2object->device_caps));
+    GST_ELEMENT_ERROR (v4l2object->element, RESOURCE, NOT_FOUND,
+        (_("Device '%s' is not a capture device."),
+            v4l2object->videodev),
+        ("Capabilities: 0x%x", v4l2object->vcap.capabilities));
     goto error;
   }
 not_output:
   {
-    GST_V4L2_ERROR (error, RESOURCE, NOT_FOUND,
-        (_("Device '%s' is not a output device."), v4l2object->videodev),
-        ("Capabilities: 0x%x", v4l2object->device_caps));
-    goto error;
-  }
-not_m2m:
-  {
-    GST_V4L2_ERROR (error, RESOURCE, NOT_FOUND,
-        (_("Device '%s' is not a M2M device."), v4l2object->videodev),
-        ("Capabilities: 0x%x", v4l2object->device_caps));
+    GST_ELEMENT_ERROR (v4l2object->element, RESOURCE, NOT_FOUND,
+        (_("Device '%s' is not a output device."),
+            v4l2object->videodev),
+        ("Capabilities: 0x%x", v4l2object->vcap.capabilities));
     goto error;
   }
 error:
   {
     if (GST_V4L2_IS_OPEN (v4l2object)) {
       /* close device */
-      v4l2object->close (v4l2object->video_fd);
+      v4l2_close (v4l2object->video_fd);
       v4l2object->video_fd = -1;
     }
     /* empty lists */
@@ -663,47 +532,6 @@ error:
   }
 }
 
-gboolean
-gst_v4l2_dup (GstV4l2Object * v4l2object, GstV4l2Object * other)
-{
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "Trying to dup device %s",
-      other->videodev);
-
-  GST_V4L2_CHECK_OPEN (other);
-  GST_V4L2_CHECK_NOT_OPEN (v4l2object);
-  GST_V4L2_CHECK_NOT_ACTIVE (other);
-  GST_V4L2_CHECK_NOT_ACTIVE (v4l2object);
-
-  v4l2object->vcap = other->vcap;
-  v4l2object->device_caps = other->device_caps;
-  gst_v4l2_adjust_buf_type (v4l2object);
-
-  v4l2object->video_fd = v4l2object->dup (other->video_fd);
-  if (!GST_V4L2_IS_OPEN (v4l2object))
-    goto not_open;
-
-  g_free (v4l2object->videodev);
-  v4l2object->videodev = g_strdup (other->videodev);
-
-  GST_INFO_OBJECT (v4l2object->dbg_obj,
-      "Cloned device '%s' (%s) successfully",
-      v4l2object->vcap.card, v4l2object->videodev);
-
-  v4l2object->never_interlaced = other->never_interlaced;
-  v4l2object->no_initial_format = other->no_initial_format;
-
-  return TRUE;
-
-not_open:
-  {
-    GST_ELEMENT_ERROR (v4l2object->element, RESOURCE, OPEN_READ_WRITE,
-        (_("Could not dup device '%s' for reading and writing."),
-            v4l2object->videodev), GST_ERROR_SYSTEM);
-
-    return FALSE;
-  }
-}
-
 
 /******************************************************
  * gst_v4l2_close():
@@ -713,14 +541,17 @@ not_open:
 gboolean
 gst_v4l2_close (GstV4l2Object * v4l2object)
 {
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "Trying to close %s",
+  GstPollFD pollfd = GST_POLL_FD_INIT;
+  GST_DEBUG_OBJECT (v4l2object->element, "Trying to close %s",
       v4l2object->videodev);
 
   GST_V4L2_CHECK_OPEN (v4l2object);
   GST_V4L2_CHECK_NOT_ACTIVE (v4l2object);
 
   /* close device */
-  v4l2object->close (v4l2object->video_fd);
+  v4l2_close (v4l2object->video_fd);
+  pollfd.fd = v4l2object->video_fd;
+  gst_poll_remove_fd (v4l2object->poll, &pollfd);
   v4l2object->video_fd = -1;
 
   /* empty lists */
@@ -738,12 +569,12 @@ gst_v4l2_close (GstV4l2Object * v4l2object)
 gboolean
 gst_v4l2_get_norm (GstV4l2Object * v4l2object, v4l2_std_id * norm)
 {
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "getting norm");
+  GST_DEBUG_OBJECT (v4l2object->element, "getting norm");
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_STD, norm) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_STD, norm) < 0)
     goto std_failed;
 
   return TRUE;
@@ -766,13 +597,13 @@ std_failed:
 gboolean
 gst_v4l2_set_norm (GstV4l2Object * v4l2object, v4l2_std_id norm)
 {
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "trying to set norm to "
+  GST_DEBUG_OBJECT (v4l2object->element, "trying to set norm to "
       "%" G_GINT64_MODIFIER "x", (guint64) norm);
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_S_STD, &norm) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_S_STD, &norm) < 0)
     goto std_failed;
 
   return TRUE;
@@ -800,7 +631,7 @@ gst_v4l2_get_frequency (GstV4l2Object * v4l2object,
 
   GstTunerChannel *channel;
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "getting current tuner frequency");
+  GST_DEBUG_OBJECT (v4l2object->element, "getting current tuner frequency");
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
@@ -808,7 +639,7 @@ gst_v4l2_get_frequency (GstV4l2Object * v4l2object,
   channel = gst_tuner_get_channel (GST_TUNER (v4l2object->element));
 
   freq.tuner = tunernum;
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_FREQUENCY, &freq) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_FREQUENCY, &freq) < 0)
     goto freq_failed;
 
   *frequency = freq.frequency * channel->freq_multiplicator;
@@ -839,7 +670,7 @@ gst_v4l2_set_frequency (GstV4l2Object * v4l2object,
 
   GstTunerChannel *channel;
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj,
+  GST_DEBUG_OBJECT (v4l2object->element,
       "setting current tuner frequency to %lu", frequency);
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
@@ -849,10 +680,10 @@ gst_v4l2_set_frequency (GstV4l2Object * v4l2object,
 
   freq.tuner = tunernum;
   /* fill in type - ignore error */
-  (void) v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_FREQUENCY, &freq);
+  v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_FREQUENCY, &freq);
   freq.frequency = frequency / channel->freq_multiplicator;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_S_FREQUENCY, &freq) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_S_FREQUENCY, &freq) < 0)
     goto freq_failed;
 
   return TRUE;
@@ -878,13 +709,13 @@ gst_v4l2_signal_strength (GstV4l2Object * v4l2object,
 {
   struct v4l2_tuner tuner = { 0, };
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "trying to get signal strength");
+  GST_DEBUG_OBJECT (v4l2object->element, "trying to get signal strength");
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
   tuner.index = tunernum;
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_TUNER, &tuner) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_TUNER, &tuner) < 0)
     goto tuner_failed;
 
   *signal_strength = tuner.signal;
@@ -912,7 +743,7 @@ gst_v4l2_get_attribute (GstV4l2Object * v4l2object,
 {
   struct v4l2_control control = { 0, };
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "getting value of attribute %d",
+  GST_DEBUG_OBJECT (v4l2object->element, "getting value of attribute %d",
       attribute_num);
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
@@ -920,7 +751,7 @@ gst_v4l2_get_attribute (GstV4l2Object * v4l2object,
 
   control.id = attribute_num;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_CTRL, &control) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_CTRL, &control) < 0)
     goto ctrl_failed;
 
   *value = control.value;
@@ -930,9 +761,9 @@ gst_v4l2_get_attribute (GstV4l2Object * v4l2object,
   /* ERRORS */
 ctrl_failed:
   {
-    GST_WARNING_OBJECT (v4l2object->dbg_obj,
-        _("Failed to get value for control %d on device '%s'."),
-        attribute_num, v4l2object->videodev);
+    GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
+        (_("Failed to get value for control %d on device '%s'."),
+            attribute_num, v4l2object->videodev), GST_ERROR_SYSTEM);
     return FALSE;
   }
 }
@@ -949,7 +780,7 @@ gst_v4l2_set_attribute (GstV4l2Object * v4l2object,
 {
   struct v4l2_control control = { 0, };
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "setting value of attribute %d to %d",
+  GST_DEBUG_OBJECT (v4l2object->element, "setting value of attribute %d to %d",
       attribute_num, value);
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
@@ -957,7 +788,7 @@ gst_v4l2_set_attribute (GstV4l2Object * v4l2object,
 
   control.id = attribute_num;
   control.value = value;
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_S_CTRL, &control) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_S_CTRL, &control) < 0)
     goto ctrl_failed;
 
   return TRUE;
@@ -965,152 +796,35 @@ gst_v4l2_set_attribute (GstV4l2Object * v4l2object,
   /* ERRORS */
 ctrl_failed:
   {
-    GST_WARNING_OBJECT (v4l2object,
-        _("Failed to set value %d for control %d on device '%s'."),
-        value, attribute_num, v4l2object->videodev);
+    GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
+        (_("Failed to set value %d for control %d on device '%s'."),
+            value, attribute_num, v4l2object->videodev), GST_ERROR_SYSTEM);
     return FALSE;
   }
 }
 
-/******************************************************
- * gst_v4l2_set_string_attribute():
- *   try to set the string value of one specific attribute
- * return value: TRUE on success, FALSE on error
- ******************************************************/
 gboolean
-gst_v4l2_set_string_attribute (GstV4l2Object * v4l2object,
-    int attribute_num, const char *value)
+gst_v4l2_get_input (GstV4l2Object * v4l2object, gint * input)
 {
-  struct v4l2_ext_controls ctrls = { {0}, 1 };
-  struct v4l2_ext_control ctrl;
-  struct v4l2_queryctrl control = { 0, };
+  gint n;
+
+  GST_DEBUG_OBJECT (v4l2object->element, "trying to get input");
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
-  control.id = attribute_num;
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_QUERYCTRL, &control) < 0) {
-    GST_WARNING_OBJECT (v4l2object,
-        "Failed to find control %d on device '%s'.",
-        attribute_num, v4l2object->videodev);
-    return TRUE;
-  }
-
-  if (control.type != V4L2_CTRL_TYPE_STRING) {
-    GST_WARNING_OBJECT (v4l2object,
-        "control %d is not string type on device '%s'.",
-        attribute_num, v4l2object->videodev);
-    return TRUE;
-  }
-
-  ctrl.id = attribute_num;
-  ctrl.size = strlen (value) + 1;
-  ctrl.string = g_malloc (ctrl.size);
-  strcpy (ctrl.string, value);
-
-  ctrls.which = V4L2_CTRL_ID2WHICH (attribute_num);
-  ctrls.count = 1;
-  ctrls.controls = &ctrl;
-
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "setting value of attribute %d to %s",
-      attribute_num, value);
-
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_S_EXT_CTRLS, &ctrls) < 0)
-    goto ctrl_failed;
-
-  g_free (ctrl.string);
-
-  return TRUE;
-
-  /* ERRORS */
-ctrl_failed:
-  {
-    GST_WARNING_OBJECT (v4l2object,
-        _("Failed to set value %s for control %d on device '%s'."),
-        value, attribute_num, v4l2object->videodev);
-    g_free (ctrl.string);
-    return FALSE;
-  }
-}
-
-static gboolean
-set_control (GQuark field_id, const GValue * value, gpointer user_data)
-{
-  GstV4l2Object *v4l2object = user_data;
-  GQuark normalised_field_id;
-  gpointer *d;
-
-  /* 32 bytes is the maximum size for a control name according to v4l2 */
-  gchar name[32];
-
-  /* Backwards compatibility: in the past GStreamer would normalise strings in
-     a subtly different way to v4l2-ctl.  e.g. the kernel's "Focus (absolute)"
-     would become "focus__absolute_" whereas now it becomes "focus_absolute".
-     Please remove the following in GStreamer 1.5 for 1.6 */
-  strncpy (name, g_quark_to_string (field_id), sizeof (name));
-  name[31] = '\0';
-  gst_v4l2_normalise_control_name (name);
-  normalised_field_id = g_quark_from_string (name);
-  if (normalised_field_id != field_id)
-    g_warning ("In GStreamer 1.4 the way V4L2 control names were normalised "
-        "changed.  Instead of setting \"%s\" please use \"%s\".  The former is "
-        "deprecated and will be removed in a future version of GStreamer",
-        g_quark_to_string (field_id), name);
-  field_id = normalised_field_id;
-
-  d = g_datalist_id_get_data (&v4l2object->controls, field_id);
-  if (!d) {
-    GST_WARNING_OBJECT (v4l2object,
-        "Control '%s' does not exist or has an unsupported type.",
-        g_quark_to_string (field_id));
-    return TRUE;
-  }
-  if (G_VALUE_HOLDS (value, G_TYPE_INT)) {
-    gst_v4l2_set_attribute (v4l2object, GPOINTER_TO_INT (d),
-        g_value_get_int (value));
-  } else if (G_VALUE_HOLDS (value, G_TYPE_BOOLEAN)) {
-    gst_v4l2_set_attribute (v4l2object, GPOINTER_TO_INT (d),
-        g_value_get_boolean (value));
-  } else if (G_VALUE_HOLDS (value, G_TYPE_STRING)) {
-    gst_v4l2_set_string_attribute (v4l2object, GPOINTER_TO_INT (d),
-        g_value_get_string (value));
-  } else {
-    GST_WARNING_OBJECT (v4l2object,
-        "no compatible value expected for control '%s'.",
-        g_quark_to_string (field_id));
-    return TRUE;
-  }
-  return TRUE;
-}
-
-gboolean
-gst_v4l2_set_controls (GstV4l2Object * v4l2object, GstStructure * controls)
-{
-  return gst_structure_foreach (controls, set_control, v4l2object);
-}
-
-gboolean
-gst_v4l2_get_input (GstV4l2Object * v4l2object, guint32 * input)
-{
-  guint32 n;
-
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "trying to get input");
-
-  if (!GST_V4L2_IS_OPEN (v4l2object))
-    return FALSE;
-
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_INPUT, &n) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_INPUT, &n) < 0)
     goto input_failed;
 
   *input = n;
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "input: %u", n);
+  GST_DEBUG_OBJECT (v4l2object->element, "input: %d", n);
 
   return TRUE;
 
   /* ERRORS */
 input_failed:
-  if (v4l2object->device_caps & V4L2_CAP_TUNER) {
+  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
     /* only give a warning message if driver actually claims to have tuner
      * support
      */
@@ -1121,68 +835,53 @@ input_failed:
 }
 
 gboolean
-gst_v4l2_set_input (GstV4l2Object * v4l2object, guint32 input)
+gst_v4l2_set_input (GstV4l2Object * v4l2object, gint input)
 {
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "trying to set input to %u", input);
+  GST_DEBUG_OBJECT (v4l2object->element, "trying to set input to %d", input);
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_S_INPUT, &input) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_S_INPUT, &input) < 0)
     goto input_failed;
 
   return TRUE;
 
   /* ERRORS */
 input_failed:
-  if (v4l2object->device_caps & V4L2_CAP_TUNER) {
+  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
     /* only give a warning message if driver actually claims to have tuner
      * support
      */
     GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
-        (_("Failed to set input %u on device %s."),
+        (_("Failed to set input %d on device %s."),
             input, v4l2object->videodev), GST_ERROR_SYSTEM);
   }
   return FALSE;
 }
 
 gboolean
-gst_v4l2_query_input (GstV4l2Object * obj, struct v4l2_input *input)
+gst_v4l2_get_output (GstV4l2Object * v4l2object, gint * output)
 {
-  gint ret;
+  gint n;
 
-  ret = obj->ioctl (obj->video_fd, VIDIOC_ENUMINPUT, input);
-  if (ret < 0) {
-    GST_WARNING_OBJECT (obj->dbg_obj, "Failed to read input state: %s (%i)",
-        g_strerror (errno), errno);
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-gboolean
-gst_v4l2_get_output (GstV4l2Object * v4l2object, guint32 * output)
-{
-  guint32 n;
-
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "trying to get output");
+  GST_DEBUG_OBJECT (v4l2object->element, "trying to get output");
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_OUTPUT, &n) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_OUTPUT, &n) < 0)
     goto output_failed;
 
   *output = n;
 
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "output: %u", n);
+  GST_DEBUG_OBJECT (v4l2object->element, "output: %d", n);
 
   return TRUE;
 
   /* ERRORS */
 output_failed:
-  if (v4l2object->device_caps & V4L2_CAP_TUNER) {
+  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
     /* only give a warning message if driver actually claims to have tuner
      * support
      */
@@ -1193,179 +892,27 @@ output_failed:
 }
 
 gboolean
-gst_v4l2_set_output (GstV4l2Object * v4l2object, guint32 output)
+gst_v4l2_set_output (GstV4l2Object * v4l2object, gint output)
 {
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "trying to set output to %u", output);
+  GST_DEBUG_OBJECT (v4l2object->element, "trying to set output to %d", output);
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
 
-  if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_S_OUTPUT, &output) < 0)
+  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_S_OUTPUT, &output) < 0)
     goto output_failed;
 
   return TRUE;
 
   /* ERRORS */
 output_failed:
-  if (v4l2object->device_caps & V4L2_CAP_TUNER) {
+  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
     /* only give a warning message if driver actually claims to have tuner
      * support
      */
     GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
-        (_("Failed to set output %u on device %s."),
+        (_("Failed to set output %d on device %s."),
             output, v4l2object->videodev), GST_ERROR_SYSTEM);
   }
   return FALSE;
 }
-
-static const gchar *
-gst_v4l2_event_to_string (guint32 event)
-{
-  switch (event) {
-    case V4L2_EVENT_ALL:
-      return "ALL";
-    case V4L2_EVENT_VSYNC:
-      return "VSYNC";
-    case V4L2_EVENT_EOS:
-      return "EOS";
-    case V4L2_EVENT_CTRL:
-      return "CTRL";
-    case V4L2_EVENT_FRAME_SYNC:
-      return "FRAME_SYNC";
-    case V4L2_EVENT_SOURCE_CHANGE:
-      return "SOURCE_CHANGE";
-    case V4L2_EVENT_MOTION_DET:
-      return "MOTION_DET";
-    default:
-      break;
-  }
-
-  return "UNKNOWN";
-}
-
-gboolean
-gst_v4l2_subscribe_event (GstV4l2Object * v4l2object, guint32 event, guint32 id)
-{
-  struct v4l2_event_subscription sub = {.type = event,.id = id, };
-  gint ret;
-
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "Subscribing to '%s' event",
-      gst_v4l2_event_to_string (event));
-
-  if (!GST_V4L2_IS_OPEN (v4l2object))
-    return FALSE;
-
-  ret = v4l2object->ioctl (v4l2object->video_fd, VIDIOC_SUBSCRIBE_EVENT, &sub);
-  if (ret < 0)
-    goto failed;
-
-  return TRUE;
-
-  /* ERRORS */
-failed:
-  {
-    if (errno == ENOTTY || errno == EINVAL) {
-      GST_DEBUG_OBJECT (v4l2object->dbg_obj,
-          "Cannot subscribe to '%s' event: %s",
-          gst_v4l2_event_to_string (event), "not supported");
-    } else {
-      GST_ERROR_OBJECT (v4l2object->dbg_obj,
-          "Cannot subscribe to '%s' event: %s",
-          gst_v4l2_event_to_string (event), g_strerror (errno));
-    }
-    return FALSE;
-  }
-}
-
-gboolean
-gst_v4l2_dequeue_event (GstV4l2Object * v4l2object, struct v4l2_event *event)
-{
-  gint ret;
-
-  if (!GST_V4L2_IS_OPEN (v4l2object))
-    return FALSE;
-
-  ret = v4l2object->ioctl (v4l2object->video_fd, VIDIOC_DQEVENT, event);
-
-  if (ret < 0) {
-    GST_ERROR_OBJECT (v4l2object->dbg_obj, "DQEVENT failed: %s",
-        g_strerror (errno));
-    return FALSE;
-  }
-
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "Dequeued a '%s' event.",
-      gst_v4l2_event_to_string (event->type));
-
-  return TRUE;
-}
-
-gboolean
-gst_v4l2_set_dv_timings (GstV4l2Object * v4l2object,
-    struct v4l2_dv_timings *timings)
-{
-  gint ret;
-
-  if (!GST_V4L2_IS_OPEN (v4l2object))
-    return FALSE;
-
-  ret = v4l2object->ioctl (v4l2object->video_fd, VIDIOC_S_DV_TIMINGS, timings);
-
-  if (ret < 0) {
-    GST_ERROR_OBJECT (v4l2object->dbg_obj, "S_DV_TIMINGS failed: %s (%i)",
-        g_strerror (errno), errno);
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-gboolean
-gst_v4l2_query_dv_timings (GstV4l2Object * v4l2object,
-    struct v4l2_dv_timings *timings)
-{
-  gint ret;
-
-  if (!GST_V4L2_IS_OPEN (v4l2object))
-    return FALSE;
-
-  ret = v4l2object->ioctl (v4l2object->video_fd, VIDIOC_QUERY_DV_TIMINGS,
-      timings);
-
-  if (ret < 0) {
-    switch (errno) {
-      case ENODATA:
-        GST_DEBUG_OBJECT (v4l2object->dbg_obj,
-            "QUERY_DV_TIMINGS not supported for this input/output");
-        break;
-      case ENOLINK:
-        GST_DEBUG_OBJECT (v4l2object->dbg_obj,
-            "No timings could be detected because no signal was found.");
-        break;
-      case ENOLCK:
-        GST_INFO_OBJECT (v4l2object->dbg_obj,
-            "The signal was unstable and the hardware could not lock on to it.");
-        break;
-      case ERANGE:
-        GST_INFO_OBJECT (v4l2object->dbg_obj,
-            "Timings were found, but they are out of range of the hardware capabilities.");
-        break;
-      default:
-        GST_ERROR_OBJECT (v4l2object->dbg_obj,
-            "QUERY_DV_TIMINGS failed: %s (%i)", g_strerror (errno), errno);
-        break;
-    }
-
-    return FALSE;
-  }
-
-  if (timings->type != V4L2_DV_BT_656_1120) {
-    GST_FIXME_OBJECT (v4l2object->dbg_obj, "Unsupported DV Timings type (%i)",
-        timings->type);
-    return FALSE;
-  }
-
-  GST_DEBUG_OBJECT (v4l2object->dbg_obj, "Detected DV Timings (%i x %i)",
-      timings->bt.width, timings->bt.height);
-
-  return TRUE;
-}
