@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -154,7 +154,8 @@ GST_START_TEST (test_2_channels)
   gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
       NULL);
 
-  gst_check_setup_events (mysrcpad, deinterleave, caps, GST_FORMAT_TIME);
+  fail_unless (gst_pad_set_caps (mysrcpad, caps));
+  gst_pad_use_fixed_caps (mysrcpad);
 
   sinkpad = gst_element_get_static_pad (deinterleave, "sink");
   fail_unless (sinkpad != NULL);
@@ -179,6 +180,7 @@ GST_START_TEST (test_2_channels)
     indata[i + 1] = 1.0;
   }
   gst_buffer_unmap (inbuf, &map);
+  gst_pad_set_caps (mysrcpad, caps);
 
   fail_unless (gst_pad_push (mysrcpad, inbuf) == GST_FLOW_OK);
 
@@ -191,10 +193,8 @@ GST_START_TEST (test_2_channels)
   mysinkpads = NULL;
 
   g_object_unref (deinterleave);
-  gst_bus_set_flushing (bus, TRUE);
   g_object_unref (bus);
   gst_caps_unref (caps);
-  gst_object_unref (mysrcpad);
 }
 
 GST_END_TEST;
@@ -227,7 +227,8 @@ GST_START_TEST (test_2_channels_1_linked)
   gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
       NULL);
 
-  gst_check_setup_events (mysrcpad, deinterleave, caps, GST_FORMAT_TIME);
+  fail_unless (gst_pad_set_caps (mysrcpad, caps));
+  gst_pad_use_fixed_caps (mysrcpad);
 
   sinkpad = gst_element_get_static_pad (deinterleave, "sink");
   fail_unless (sinkpad != NULL);
@@ -252,6 +253,7 @@ GST_START_TEST (test_2_channels_1_linked)
     indata[i + 1] = 1.0;
   }
   gst_buffer_unmap (inbuf, &map);
+  gst_pad_set_caps (mysrcpad, caps);
 
   fail_unless (gst_pad_push (mysrcpad, inbuf) == GST_FLOW_OK);
 
@@ -264,10 +266,8 @@ GST_START_TEST (test_2_channels_1_linked)
   mysinkpads = NULL;
 
   g_object_unref (deinterleave);
-  gst_bus_set_flushing (bus, TRUE);
   g_object_unref (bus);
   gst_caps_unref (caps);
-  gst_object_unref (mysrcpad);
 }
 
 GST_END_TEST;
@@ -276,7 +276,6 @@ GST_START_TEST (test_2_channels_caps_change)
 {
   GstPad *sinkpad;
   GstCaps *caps, *caps2;
-  GstCaps *ret_caps;
   gint i;
   GstBuffer *inbuf;
   gfloat *indata;
@@ -302,18 +301,14 @@ GST_START_TEST (test_2_channels_caps_change)
       G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
   gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
       NULL);
+  fail_unless (gst_pad_set_caps (mysrcpad, caps));
+  gst_pad_use_fixed_caps (mysrcpad);
 
   sinkpad = gst_element_get_static_pad (deinterleave, "sink");
   fail_unless (sinkpad != NULL);
   fail_unless (gst_pad_link (mysrcpad, sinkpad) == GST_PAD_LINK_OK);
   g_object_unref (sinkpad);
 
-  ret_caps = gst_pad_peer_query_caps (mysrcpad, caps);
-  fail_if (gst_caps_is_empty (ret_caps));
-  fail_unless (gst_pad_peer_query_accept_caps (mysrcpad, caps));
-  gst_caps_unref (ret_caps);
-  gst_check_setup_events (mysrcpad, deinterleave, caps, GST_FORMAT_TIME);
-
   g_signal_connect (deinterleave, "pad-added",
       G_CALLBACK (deinterleave_pad_added), GINT_TO_POINTER (2));
 
@@ -332,6 +327,7 @@ GST_START_TEST (test_2_channels_caps_change)
     indata[i + 1] = 1.0;
   }
   gst_buffer_unmap (inbuf, &map);
+  gst_pad_set_caps (mysrcpad, caps);
 
   fail_unless (gst_pad_push (mysrcpad, inbuf) == GST_FLOW_OK);
 
@@ -343,10 +339,6 @@ GST_START_TEST (test_2_channels_caps_change)
       G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
   gst_caps_set_simple (caps2, "channel-mask", GST_TYPE_BITMASK, channel_mask,
       NULL);
-  ret_caps = gst_pad_peer_query_caps (mysrcpad, caps2);
-  fail_if (gst_caps_is_empty (ret_caps));
-  fail_unless (gst_pad_peer_query_accept_caps (mysrcpad, caps2));
-  gst_caps_unref (ret_caps);
   gst_pad_set_caps (mysrcpad, caps2);
 
   inbuf = gst_buffer_new_and_alloc (2 * 48000 * sizeof (gfloat));
@@ -358,6 +350,7 @@ GST_START_TEST (test_2_channels_caps_change)
     indata[i + 1] = 1.0;
   }
   gst_buffer_unmap (inbuf, &map);
+  gst_pad_set_caps (mysrcpad, caps2);
 
   /* Should work fine because the caps changed in a compatible way */
   fail_unless (gst_pad_push (mysrcpad, inbuf) == GST_FLOW_OK);
@@ -374,10 +367,6 @@ GST_START_TEST (test_2_channels_caps_change)
       G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
   gst_caps_set_simple (caps2, "channel-mask", GST_TYPE_BITMASK, channel_mask,
       NULL);
-  ret_caps = gst_pad_peer_query_caps (mysrcpad, caps2);
-  fail_unless (gst_caps_is_empty (ret_caps));
-  gst_caps_unref (ret_caps);
-  fail_if (gst_pad_peer_query_accept_caps (mysrcpad, caps2));
   gst_pad_set_caps (mysrcpad, caps2);
 
   inbuf = gst_buffer_new_and_alloc (3 * 48000 * sizeof (gfloat));
@@ -390,6 +379,7 @@ GST_START_TEST (test_2_channels_caps_change)
     indata[i + 2] = 0.0;
   }
   gst_buffer_unmap (inbuf, &map);
+  gst_pad_set_caps (mysrcpad, caps2);
 
   /* Should break because the caps changed in an incompatible way */
   fail_if (gst_pad_push (mysrcpad, inbuf) == GST_FLOW_OK);
@@ -403,11 +393,9 @@ GST_START_TEST (test_2_channels_caps_change)
   mysinkpads = NULL;
 
   g_object_unref (deinterleave);
-  gst_bus_set_flushing (bus, TRUE);
   g_object_unref (bus);
   gst_caps_unref (caps);
   gst_caps_unref (caps2);
-  gst_object_unref (mysrcpad);
 }
 
 GST_END_TEST;
@@ -439,6 +427,21 @@ src_handoff_float32_8ch (GstElement * src, GstBuffer * buf, GstPad * pad,
 {
   gfloat *data, *p;
   guint size, i, c;
+  GstAudioChannelPosition layout[NUM_CHANNELS];
+  GstCaps *caps;
+
+  caps = gst_caps_new_simple ("audio/x-raw",
+      "format", G_TYPE_STRING, GST_AUDIO_NE (F32),
+      "channels", G_TYPE_INT, NUM_CHANNELS,
+      "layout", G_TYPE_STRING, "interleaved",
+      "rate", G_TYPE_INT, SAMPLE_RATE, NULL);
+
+  for (i = 0; i < NUM_CHANNELS; ++i)
+    layout[i] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT + i;
+
+  set_channel_positions (caps, NUM_CHANNELS, layout);
+  gst_pad_set_caps (pad, caps);
+  gst_caps_unref (caps);
 
   size = sizeof (gfloat) * SAMPLES_PER_BUFFER * NUM_CHANNELS;
   data = p = (gfloat *) g_malloc (size);
@@ -461,34 +464,6 @@ src_handoff_float32_8ch (GstElement * src, GstBuffer * buf, GstPad * pad,
   GST_BUFFER_TIMESTAMP (buf) = 0;
 }
 
-static GstPadProbeReturn
-src_event_probe (GstPad * pad, GstPadProbeInfo * info, gpointer userdata)
-{
-  GstAudioChannelPosition layout[NUM_CHANNELS];
-  GstCaps *caps;
-  guint i;
-
-  if ((info->type & GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM)
-      && GST_EVENT_TYPE (info->data) == GST_EVENT_STREAM_START) {
-    gst_pad_remove_probe (pad, info->id);
-
-    caps = gst_caps_new_simple ("audio/x-raw",
-        "format", G_TYPE_STRING, GST_AUDIO_NE (F32),
-        "channels", G_TYPE_INT, NUM_CHANNELS,
-        "layout", G_TYPE_STRING, "interleaved",
-        "rate", G_TYPE_INT, SAMPLE_RATE, NULL);
-
-    for (i = 0; i < NUM_CHANNELS; ++i)
-      layout[i] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT + i;
-
-    set_channel_positions (caps, NUM_CHANNELS, layout);
-    gst_pad_set_caps (pad, caps);
-    gst_caps_unref (caps);
-  }
-
-  return GST_PAD_PROBE_OK;
-}
-
 static GstPadProbeReturn
 float_buffer_check_probe (GstPad * pad, GstPadProbeInfo * info,
     gpointer userdata)
@@ -568,6 +543,7 @@ pad_added_setup_data_check_float32_8ch_cb (GstElement * deinterleave,
   fail_unless_equals_int (gst_pad_link (pad, sinkpad), GST_PAD_LINK_OK);
   gst_object_unref (sinkpad);
 
+
   gst_pad_add_probe (pad, GST_PAD_PROBE_TYPE_BUFFER, float_buffer_check_probe,
       GUINT_TO_POINTER (pads_created), NULL);
 
@@ -582,7 +558,6 @@ static GstElement *
 make_fake_src_8chans_float32 (void)
 {
   GstElement *src;
-  GstPad *pad;
 
   src = gst_element_factory_make ("fakesrc", "src");
   fail_unless (src != NULL, "failed to create fakesrc element");
@@ -592,11 +567,6 @@ make_fake_src_8chans_float32 (void)
 
   g_signal_connect (src, "handoff", G_CALLBACK (src_handoff_float32_8ch), NULL);
 
-  pad = gst_element_get_static_pad (src, "src");
-  gst_pad_add_probe (pad, GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM, src_event_probe,
-      NULL, NULL);
-  gst_object_unref (pad);
-
   return src;
 }
 
