@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -81,26 +81,20 @@ static void
 on_ssrc_active_cb (GstElement * rtpbin, guint sessid, guint ssrc,
     GstElement * depay)
 {
-  GObject *session, *osrc;
+  GObject *session, *isrc, *osrc;
 
   g_print ("got RTCP from session %u, SSRC %u\n", sessid, ssrc);
 
   /* get the right session */
   g_signal_emit_by_name (rtpbin, "get-internal-session", sessid, &session);
 
-#if 0
-  /* FIXME: This is broken in rtpbin */
   /* get the internal source (the SSRC allocated to us, the receiver */
   g_object_get (session, "internal-source", &isrc, NULL);
   print_source_stats (isrc);
-  g_object_unref (isrc);
-#endif
 
   /* get the remote source that sent us RTCP */
   g_signal_emit_by_name (session, "get-source-by-ssrc", ssrc, &osrc);
   print_source_stats (osrc);
-  g_object_unref (osrc);
-  g_object_unref (session);
 }
 
 /* will be called when rtpbin has validated a payload that we can depayload */
@@ -108,7 +102,7 @@ static void
 pad_added_cb (GstElement * rtpbin, GstPad * new_pad, GstElement * depay)
 {
   GstPad *sinkpad;
-  GstPadLinkReturn lres GST_UNUSED_ASSERT;
+  GstPadLinkReturn lres;
 
   g_print ("new payload on pad: %s\n", GST_PAD_NAME (new_pad));
 
@@ -122,9 +116,9 @@ pad_added_cb (GstElement * rtpbin, GstPad * new_pad, GstElement * depay)
 
 /* build a pipeline equivalent to:
  *
- * gst-launch-1.0 -v rtpbin name=rtpbin                                                \
+ * gst-launch -v rtpbin name=rtpbin                                                \
  *      udpsrc caps=$AUDIO_CAPS port=5002 ! rtpbin.recv_rtp_sink_0              \
- *        rtpbin. ! rtppcmadepay ! alawdec ! audioconvert ! audioresample ! autoaudiosink \
+ *        rtpbin. ! rtppcmadepay ! alawdec ! audioconvert ! audioresample ! alsasink \
  *      udpsrc port=5003 ! rtpbin.recv_rtcp_sink_0                              \
  *        rtpbin.send_rtcp_src_0 ! udpsink port=5007 host=$DEST sync=false async=false
  */
@@ -136,8 +130,8 @@ main (int argc, char *argv[])
   GstElement *pipeline;
   GMainLoop *loop;
   GstCaps *caps;
-  gboolean res GST_UNUSED_ASSERT;
-  GstPadLinkReturn lres GST_UNUSED_ASSERT;
+  gboolean res;
+  GstPadLinkReturn lres;
   GstPad *srcpad, *sinkpad;
 
   /* always init first */
@@ -197,21 +191,21 @@ main (int argc, char *argv[])
 
   /* now link all to the rtpbin, start by getting an RTP sinkpad for session 0 */
   srcpad = gst_element_get_static_pad (rtpsrc, "src");
-  sinkpad = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_0");
+  sinkpad = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_0");
   lres = gst_pad_link (srcpad, sinkpad);
   g_assert (lres == GST_PAD_LINK_OK);
   gst_object_unref (srcpad);
 
   /* get an RTCP sinkpad in session 0 */
   srcpad = gst_element_get_static_pad (rtcpsrc, "src");
-  sinkpad = gst_element_request_pad_simple (rtpbin, "recv_rtcp_sink_0");
+  sinkpad = gst_element_get_request_pad (rtpbin, "recv_rtcp_sink_0");
   lres = gst_pad_link (srcpad, sinkpad);
   g_assert (lres == GST_PAD_LINK_OK);
   gst_object_unref (srcpad);
   gst_object_unref (sinkpad);
 
   /* get an RTCP srcpad for sending RTCP back to the sender */
-  srcpad = gst_element_request_pad_simple (rtpbin, "send_rtcp_src_0");
+  srcpad = gst_element_get_request_pad (rtpbin, "send_rtcp_src_0");
   sinkpad = gst_element_get_static_pad (rtcpsink, "sink");
   lres = gst_pad_link (srcpad, sinkpad);
   g_assert (lres == GST_PAD_LINK_OK);
